@@ -1,14 +1,20 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.12
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
-from polyglot.builtins import environ_item, hasenv
+import codecs
+import collections
+import collections.abc
+import locale
+import os
+import sys
 from functools import lru_cache
-import sys, locale, codecs, os, collections, collections.abc
+
+from polyglot.builtins import environ_item, hasenv
 
 __appname__   = 'calibre'
-numeric_version = (5, 44, 0)
+numeric_version = (8, 0, 1)
 __version__   = '.'.join(map(str, numeric_version))
 git_version   = None
-__author__    = "Kovid Goyal <kovid@kovidgoyal.net>"
+__author__    = 'Kovid Goyal <kovid@kovidgoyal.net>'
 
 '''
 Various run time constants.
@@ -25,7 +31,7 @@ isdragonflybsd = 'dragonfly' in _plat
 isopenbsd = 'openbsd' in _plat
 isbsd = isfreebsd or isnetbsd or isdragonflybsd or isopenbsd
 ishaiku = 'haiku1' in _plat
-islinux   = not(iswindows or ismacos or isbsd or ishaiku)
+islinux   = not (iswindows or ismacos or isbsd or ishaiku)
 isfrozen  = hasattr(sys, 'frozen')
 isunix = ismacos or islinux or ishaiku
 isportable = hasenv('CALIBRE_PORTABLE_BUILD')
@@ -35,11 +41,12 @@ if iswindows:
     wver = sys.getwindowsversion()
     isxp = wver.major < 6
     isoldvista = wver.build < 6002
-is64bit = sys.maxsize > (1 << 32)
+is64bit = True
 isworker = hasenv('CALIBRE_WORKER') or hasenv('CALIBRE_SIMPLE_WORKER')
 if isworker:
     os.environ.pop(environ_item('CALIBRE_FORCE_ANSI'), None)
 FAKE_PROTOCOL, FAKE_HOST = 'clbr', 'internal.invalid'
+SPECIAL_TITLE_FOR_WEBENGINE_COMMS = '__webengine_messages_pending__'
 VIEWER_APP_UID = 'com.calibre-ebook.viewer'
 EDITOR_APP_UID = 'com.calibre-ebook.edit-book'
 MAIN_APP_UID = 'com.calibre-ebook.main-gui'
@@ -60,7 +67,7 @@ builtin_colors_light = {
     'purple': '#d9b2ff',
 }
 builtin_colors_dark = {
-    'yellow': '#c18d18',
+    'yellow': '#906e00',
     'green': '#306f50',
     'blue': '#265589',
     'red': '#a23e5a',
@@ -140,7 +147,7 @@ def _get_cache_dir():
 
     if iswindows:
         try:
-            candidate = os.path.join(winutil.special_folder_path(winutil.CSIDL_LOCAL_APPDATA), '%s-cache'%__appname__)
+            candidate = os.path.join(winutil.special_folder_path(winutil.CSIDL_LOCAL_APPDATA), f'{__appname__}-cache')
         except ValueError:
             return confcache
     elif ismacos:
@@ -173,9 +180,9 @@ def cache_dir():
 plugins_loc = sys.extensions_location
 system_plugins_loc = getattr(sys, 'system_plugins_location', None)
 
-from importlib.machinery import ModuleSpec, EXTENSION_SUFFIXES, ExtensionFileLoader
-from importlib.util import find_spec
 from importlib import import_module
+from importlib.machinery import EXTENSION_SUFFIXES, ExtensionFileLoader, ModuleSpec
+from importlib.util import find_spec
 
 
 class DeVendorLoader:
@@ -204,6 +211,15 @@ class DeVendor:
             return find_spec('feedparser')
         if fullname.startswith('calibre.ebooks.markdown'):
             return ModuleSpec(fullname, DeVendorLoader(fullname[len('calibre.ebooks.'):]))
+        if fullname.startswith('PyQt5'):
+            # this is present for third party plugin compat
+            if fullname == 'PyQt5':
+                return ModuleSpec(fullname, DeVendorLoader('qt'))
+            return ModuleSpec(fullname, DeVendorLoader('qt.webengine' if 'QWebEngine' in fullname else 'qt.core'))
+        if fullname.startswith('Cryptodome'):
+            # this is needed for py7zr which uses pycryptodomex instead of
+            # pycryptodome for some reason
+            return ModuleSpec(fullname, DeVendorLoader(fullname.replace('dome', '', 1)))
 
 
 class ExtensionsPackageLoader:
@@ -240,13 +256,16 @@ class ExtensionsImporter:
             'podofo',
             'cPalmdoc',
             'progress_indicator',
+            'rcc_backend',
             'icu',
             'speedup',
             'html_as_json',
             'fast_css_transform',
+            'fast_html_entities',
             'unicode_names',
             'html_syntax_highlighter',
             'hyphen',
+            'ffmpeg',
             'freetype',
             'imageops',
             'hunspell',
@@ -256,9 +275,10 @@ class ExtensionsImporter:
             'tokenizer',
             'certgen',
             'sqlite_extension',
+            'uchardet',
         )
         if iswindows:
-            extra = ('winutil', 'wpd', 'winfonts', 'winsapi')
+            extra = ('winutil', 'wpd', 'winfonts', 'wintoast')
         elif ismacos:
             extra = ('usbobserver', 'cocoa', 'libusb', 'libmtp')
         elif isfreebsd or ishaiku or islinux or isopenbsd:
@@ -322,7 +342,7 @@ class Plugins(collections.abc.Mapping):
         try:
             return import_module('calibre_extensions.' + name), ''
         except ModuleNotFoundError:
-            raise KeyError('No plugin named %r'%name)
+            raise KeyError(f'No plugin named {name!r}')
         except Exception as err:
             return None, str(err)
 
@@ -335,6 +355,15 @@ class Plugins(collections.abc.Mapping):
         finally:
             conn.enableloadextension(False)
 
+    def load_sqlite3_extension(self, conn, name):
+        conn.enable_load_extension(True)
+        try:
+            ext = 'pyd' if iswindows else 'so'
+            path = os.path.join(plugins_loc, f'{name}.{ext}')
+            conn.load_extension(path)
+        finally:
+            conn.enable_load_extension(False)
+
 
 plugins = None
 if plugins is None:
@@ -369,7 +398,8 @@ else:
             not os.access(config_dir, os.W_OK) or not \
             os.access(config_dir, os.X_OK):
         print('No write access to', config_dir, 'using a temporary dir instead')
-        import tempfile, atexit
+        import atexit
+        import tempfile
         config_dir = tempfile.mkdtemp(prefix='calibre-config-')
 
         def cleanup_cdir():
@@ -381,7 +411,6 @@ else:
         atexit.register(cleanup_cdir)
 # }}}
 
-
 is_running_from_develop = False
 if getattr(sys, 'frozen', False):
     try:
@@ -392,6 +421,9 @@ if getattr(sys, 'frozen', False):
         is_running_from_develop = running_in_develop_mode()
 
 in_develop_mode = os.getenv('CALIBRE_ENABLE_DEVELOP_MODE') == '1'
+if iswindows:
+    # Needed to get Qt to use the correct cache dir, relies on a patched Qt
+    os.environ['CALIBRE_QT_CACHE_LOCATION'] = cache_dir()
 
 
 def get_version():
@@ -404,8 +436,6 @@ def get_version():
             v = v[:-2]
     if is_running_from_develop:
         v += '*'
-    if iswindows and is64bit:
-        v += ' [64bit]'
 
     return v
 
@@ -460,7 +490,36 @@ def get_umask():
     return mask
 
 
-# call this at startup as it changed process global state, which doesnt work
+# call this at startup as it changes process global state, which doesn't work
 # with multi-threading. It's absurd there is no way to safely read the current
 # umask of a process.
 get_umask()
+
+
+@lru_cache(maxsize=2)
+def bundled_binaries_dir() -> str:
+    if ismacos and hasattr(sys, 'frameworks_dir'):
+        base = os.path.join(os.path.dirname(sys.frameworks_dir), 'utils.app', 'Contents', 'MacOS')
+        return base
+    if iswindows and hasattr(sys, 'frozen'):
+        base = sys.extensions_location if hasattr(sys, 'new_app_layout') else os.path.dirname(sys.executable)
+        return base
+    if (islinux or isbsd) and getattr(sys, 'frozen', False):
+        return os.path.join(sys.executables_location, 'bin')
+    return ''
+
+
+@lru_cache(2)
+def piper_cmdline() -> tuple[str, ...]:
+    ext = '.exe' if iswindows else ''
+    if bbd := bundled_binaries_dir():
+        if ismacos:
+            return (os.path.join(sys.frameworks_dir, 'piper', 'piper'),)
+        return (os.path.join(bbd, 'piper', 'piper' + ext),)
+    if pd := os.environ.get('PIPER_TTS_DIR'):
+        return (os.path.join(pd, 'piper' + ext),)
+    import shutil
+    exe = shutil.which('piper-tts')
+    if exe:
+        return (exe,)
+    return ()
