@@ -1,9 +1,13 @@
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os, sys, zipfile, importlib, enum
+import enum
+import importlib
+import os
+import sys
+import zipfile
 
-from calibre.constants import numeric_version, iswindows, ismacos
+from calibre.constants import ismacos, iswindows, numeric_version
 from calibre.ptempfile import PersistentTemporaryFile
 
 if iswindows:
@@ -141,24 +145,14 @@ class Plugin:  # {{{
         True if the user clicks OK, False otherwise. The changes are
         automatically applied.
         '''
-        from qt.core import QDialog, QDialogButtonBox, QVBoxLayout, \
-                QLabel, Qt, QLineEdit
+        from qt.core import QApplication, QDialog, QDialogButtonBox, QLabel, QLineEdit, QScrollArea, QSize, Qt, QVBoxLayout
+
         from calibre.gui2 import gprefs
 
         prefname = 'plugin config dialog:'+self.type + ':' + self.name
-        geom = gprefs.get(prefname, None)
-
         config_dialog = QDialog(parent)
         button_box = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
         v = QVBoxLayout(config_dialog)
-
-        def size_dialog():
-            if geom is None:
-                config_dialog.resize(config_dialog.sizeHint())
-            else:
-                from qt.core import QApplication
-                QApplication.instance().safe_restore_geometry(config_dialog, geom)
-
         button_box.accepted.connect(config_dialog.accept)
         button_box.rejected.connect(config_dialog.reject)
         config_dialog.setWindowTitle(_('Customize') + ' ' + self.name)
@@ -174,9 +168,18 @@ class Plugin:  # {{{
             return False
 
         if config_widget is not None:
-            v.addWidget(config_widget)
+            class SA(QScrollArea):
+                def sizeHint(self):
+                    sz = self.widget().sizeHint()
+                    fw = 2 * self.frameWidth()
+                    return QSize(sz.width() + self.verticalScrollBar().sizeHint().width() + fw, sz.height() + fw)
+            sa = SA(config_dialog)
+            sa.setWidget(config_widget)
+            sa.setWidgetResizable(True)
+            v.addWidget(sa)
             v.addWidget(button_box)
-            size_dialog()
+            if not config_dialog.restore_geometry(gprefs, prefname):
+                QApplication.instance().ensure_window_on_screen(config_dialog)
             config_dialog.exec()
 
             if config_dialog.result() == QDialog.DialogCode.Accepted:
@@ -186,8 +189,7 @@ class Plugin:  # {{{
                 else:
                     self.save_settings(config_widget)
         else:
-            from calibre.customize.ui import plugin_customization, \
-                customize_plugin
+            from calibre.customize.ui import customize_plugin, plugin_customization
             help_text = self.customization_help(gui=True)
             help_text = QLabel(help_text, config_dialog)
             help_text.setWordWrap(True)
@@ -201,16 +203,14 @@ class Plugin:  # {{{
             sc = QLineEdit(sc, config_dialog)
             v.addWidget(sc)
             v.addWidget(button_box)
-            size_dialog()
+            config_dialog.restore_geometry(gprefs, prefname)
             config_dialog.exec()
 
             if config_dialog.result() == QDialog.DialogCode.Accepted:
                 sc = str(sc.text()).strip()
                 customize_plugin(self, sc)
 
-        geom = bytearray(config_dialog.saveGeometry())
-        gprefs[prefname] = geom
-
+        config_dialog.save_geometry(gprefs, prefname)
         return config_dialog.result()
 
     def load_resources(self, names):
@@ -285,8 +285,9 @@ class Plugin:  # {{{
                 import something
         '''
         if self.plugin_path is not None:
-            from calibre.utils.zipfile import ZipFile
             from importlib.machinery import EXTENSION_SUFFIXES
+
+            from calibre.utils.zipfile import ZipFile
             with ZipFile(self.plugin_path) as zf:
                 extensions = {x.lower() for x in EXTENSION_SUFFIXES}
                 zip_safe = True
@@ -321,8 +322,7 @@ class Plugin:  # {{{
         interface. It is called when the user does: calibre-debug -r "Plugin
         Name". Any arguments passed are present in the args variable.
         '''
-        raise NotImplementedError('The %s plugin has no command line interface'
-                                  %self.name)
+        raise NotImplementedError(f'The {self.name} plugin has no command line interface')
 
 # }}}
 
@@ -346,6 +346,15 @@ class FileTypePlugin(Plugin):  # {{{
     #: methods of the plugin are called.
     on_postimport  = False
 
+    #: If True, this plugin is run after a book is converted.
+    #: In this case the postconvert method of the plugin is called.
+    on_postconvert = False
+
+    #: If True, this plugin is run after a book file is deleted
+    #: from the database. In this case the postdelete method of
+    #: the plugin is called.
+    on_postdelete = False
+
     #: If True, this plugin is run just before a conversion
     on_preprocess  = False
 
@@ -390,6 +399,31 @@ class FileTypePlugin(Plugin):  # {{{
         '''
         pass  # Default implementation does nothing
 
+    def postconvert(self, book_id, book_format, db):
+        '''
+        Called post conversion, i.e., after the conversion output book file has been added to the database.
+        Note that it is run after a conversion only, not after a book is added. It is useful for modifying
+        the book record based on the contents of the newly added file.
+
+        :param book_id: Database id of the added book.
+        :param book_format: The file type of the book that was added.
+        :param db: Library database.
+        '''
+        pass  # Default implementation does nothing
+
+    def postdelete(self, book_id, book_format, db):
+        '''
+        Called post deletion, i.e., after the book file has been deleted from the database. Note
+        that it is not run when a book record is deleted, only when one or more formats from the
+        book are deleted. It is useful for modifying the book record based on the format of the
+        deleted file.
+
+        :param book_id: Database id of the added book.
+        :param book_format: The file type of the book that was added.
+        :param db: Library database.
+        '''
+        pass  # Default implementation does nothing
+
     def postadd(self, book_id, fmt_map, db):
         '''
         Called post add, i.e. after a book has been added to the db. Note that
@@ -439,7 +473,8 @@ class MetadataReaderPlugin(Plugin):  # {{{
             in :attr:`file_types`.
         :return: A :class:`calibre.ebooks.metadata.book.Metadata` object
         '''
-        return None
+        return
+
 # }}}
 
 
@@ -504,7 +539,7 @@ class CatalogPlugin(Plugin):  # {{{
         Custom fields sort after standard fields
         '''
         if key.startswith('#'):
-            return '~%s' % key[1:]
+            return f'~{key[1:]}'
         else:
             return key
 
@@ -538,11 +573,10 @@ class CatalogPlugin(Plugin):  # {{{
             # Validate requested_fields
             if requested_fields - all_fields:
                 from calibre.library import current_library_name
-                invalid_fields = sorted(list(requested_fields - all_fields))
-                print("invalid --fields specified: %s" % ', '.join(invalid_fields))
-                print("available fields in '%s': %s" %
-                      (current_library_name(), ', '.join(sorted(list(all_fields)))))
-                raise ValueError("unable to generate catalog with specified fields")
+                invalid_fields = sorted(requested_fields - all_fields)
+                print('invalid --fields specified: {}'.format(', '.join(invalid_fields)))
+                print("available fields in '{}': {}".format(current_library_name(), ', '.join(sorted(all_fields))))
+                raise ValueError('unable to generate catalog with specified fields')
 
             fields = [x for x in of if x in all_fields]
         else:
@@ -564,8 +598,8 @@ class CatalogPlugin(Plugin):  # {{{
         from calibre.customize.ui import config
         from calibre.ptempfile import PersistentTemporaryDirectory
 
-        if not type(self) in builtin_plugins and self.name not in config['disabled_plugins']:
-            files_to_copy = [f"{self.name.lower()}.{ext}" for ext in ["ui","py"]]
+        if type(self) not in builtin_plugins and self.name not in config['disabled_plugins']:
+            files_to_copy = [f'{self.name.lower()}.{ext}' for ext in ['ui','py']]
             resources = zipfile.ZipFile(self.plugin_path,'r')
 
             if self.resources_path is None:
@@ -575,7 +609,7 @@ class CatalogPlugin(Plugin):  # {{{
                 try:
                     resources.extract(file, self.resources_path)
                 except:
-                    print(f" customize:__init__.initialize(): {file} not found in {os.path.basename(self.plugin_path)}")
+                    print(f' customize:__init__.initialize(): {file} not found in {os.path.basename(self.plugin_path)}')
                     continue
             resources.close()
 
