@@ -34,8 +34,8 @@ class Plugin(_Plugin):
         self.width_pts = self.width * 72./self.dpi
         self.height_pts = self.height * 72./self.dpi
 
-# Input profiles {{{
 
+# Input profiles {{{
 
 class InputProfile(Plugin):
 
@@ -299,8 +299,8 @@ class iPadOutput(OutputProfile):
         }
     ]
 
-    ratings_char = '\u2605'            # filled star
-    empty_ratings_char = '\u2606'      # hollow star
+    ratings_char = '★'            # filled star
+    empty_ratings_char = '☆'      # hollow star
 
     touchscreen = True
     # touchscreen_news_css {{{
@@ -677,8 +677,8 @@ class KindleOutput(OutputProfile):
     supports_mobi_indexing = True
     periodical_date_in_title = False
 
-    empty_ratings_char = '\u2606'
-    ratings_char = '\u2605'
+    empty_ratings_char = '☆'
+    ratings_char = '★'
 
     mobi_ems_per_blockquote = 2.0
 
@@ -696,8 +696,8 @@ class KindleDXOutput(OutputProfile):
     # comic_screen_size         = (741, 1022)
     supports_mobi_indexing = True
     periodical_date_in_title = False
-    empty_ratings_char = '\u2606'
-    ratings_char = '\u2605'
+    empty_ratings_char = '☆'
+    ratings_char = '★'
     mobi_ems_per_blockquote = 2.0
 
 
@@ -750,6 +750,18 @@ class KindleOasisOutput(KindlePaperWhite3Output):
     comic_screen_size = screen_size
 
 
+class KindleScribeOutput(KindlePaperWhite3Output):
+
+    name = 'Kindle Scribe'
+    short_name = 'kindle_scribe'
+    description = _('This profile is intended for the Amazon Kindle Scribe 2022 and above')
+    # Screen size is currently just the spec size, actual renderable area will
+    # depend on someone with the device doing tests.
+    screen_size               = (1860, 2480)
+    dpi                       = 300.0
+    comic_screen_size = screen_size
+
+
 class KindleFireOutput(KindleDXOutput):
 
     name = 'Kindle Fire'
@@ -901,7 +913,7 @@ output_profiles = [
     PocketBookPro912Output, PocketBookLuxOutput, PocketBookHDOutput,
     PocketBookInkpad3Output, GenericEink, GenericEinkLarge, GenericEinkHD,
     KindleFireOutput, KindlePaperWhiteOutput, KindleVoyageOutput,
-    KindlePaperWhite3Output, KindleOasisOutput
+    KindlePaperWhite3Output, KindleOasisOutput, KindleScribeOutput,
 ]
 
 output_profiles.sort(key=lambda x: x.name.lower())
