@@ -1,17 +1,23 @@
-#!/usr/local/bin/python3.9
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import weakref, traceback
-from threading import Thread, Event
+import sys
+import traceback
+import weakref
+from threading import Event, Thread
 
-from calibre import prints
 from calibre.ebooks.metadata.opf2 import metadata_to_opf
 
 
+def prints(*a, **kw):
+    kw['file'] = sys.stderr
+    return print(*a, **kw)
+
+
 class Abort(Exception):
     pass
 
@@ -64,6 +70,7 @@ class MetadataBackup(Thread):
                 if self.stop_running.is_set() or self.db.is_closed:
                     return
                 traceback.print_exc()
+
         try:
             book_id = self.db.get_a_dirtied_book()
             if book_id is None:
