@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.12
 # License: GPLv3 Copyright: 2017, Kovid Goyal <kovid at kovidgoyal.net>
 
 
@@ -43,5 +43,5 @@ def main(opts, args, dbctx):
             prints(pformat(data))
             print('\n')
         else:
-            prints(col, '(%d)'%data['num'])
+            prints(col, f"({data['num']})")
     return 0
