@@ -1,10 +1,9 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.13
 # License: GPLv3 Copyright: 2017, Kovid Goyal <kovid at kovidgoyal.net>
 
 
 from calibre.constants import trash_name
 from calibre.db.cli import integers_from_string
-from calibre.db.delete_service import delete_service
 from calibre.srv.changes import books_deleted
 
 readonly = False
@@ -13,8 +12,6 @@ version = 0  # change this if you change signature of implementation()
 
 def implementation(db, notify_changes, ids, permanent):
     db.remove_books(ids, permanent=permanent)
-    if not permanent:
-        delete_service().wait()
     if notify_changes is not None:
         notify_changes(books_deleted(ids))
 
