@@ -1,8 +1,7 @@
-#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.13
 # License: GPLv3 Copyright: 2017, Kovid Goyal <kovid at kovidgoyal.net>
 
 
-from calibre.db.delete_service import delete_service
 from calibre.srv.changes import formats_removed
 
 readonly = False
@@ -13,7 +12,6 @@ def implementation(db, notify_changes, book_id, fmt):
     is_remote = notify_changes is not None
     fmt_map = {book_id: (fmt, )}
     db.remove_formats(fmt_map)
-    delete_service().wait()
     if is_remote:
         notify_changes(formats_removed(fmt_map))
 
