@@ -1,11 +1,11 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.12
 # License: GPL v3 Copyright: 2021, Kovid Goyal <kovid at kovidgoyal.net>
 
 import weakref
 from contextlib import suppress
+from enum import Enum, auto
 from queue import Queue
 from threading import Thread
-from enum import Enum, auto
 
 
 class EventType(Enum):
@@ -40,6 +40,9 @@ class EventType(Enum):
     #: When a book format is edited, with arguments: (book_id, fmt)
     book_edited = auto()
 
+    #: When the indexing progress changes
+    indexing_progress_changed = auto()
+
 
 class EventDispatcher(Thread):
 
@@ -67,6 +70,10 @@ class EventDispatcher(Thread):
         with suppress(ValueError):
             self.refs.remove(ref)
 
+    def __contains__(self, callback):
+        ref = weakref.ref(callback)
+        return ref in self.refs
+
     def __call__(self, event_name, *args):
         if self.activated:
             self.queue.put((event_name, self.library_id, args))
