@@ -1,14 +1,17 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.12
 # License: GPLv3 Copyright: 2021, Kovid Goyal <kovid at kovidgoyal.net>
 
 
 import builtins
+import os
 import sys
+import tempfile
+
 from apsw import Connection
 
 from calibre.constants import plugins
-from calibre.db.tests.base import BaseTest
 from calibre.db.annotations import unicode_normalize
+from calibre.db.tests.base import BaseTest
 
 
 def print(*args, **kwargs):
@@ -44,14 +47,14 @@ CREATE VIRTUAL TABLE fts_row USING fts5vocab(fts_table, row);
     def search(self, query, highlight_start='>', highlight_end='<', snippet_size=4):
         snippet_size=max(1, min(snippet_size, 64))
         stmt = (
-            f'SELECT snippet(fts_table, 0, "{highlight_start}", "{highlight_end}", "…", {snippet_size})'
+            f"SELECT snippet(fts_table, 0, '{highlight_start}', '{highlight_end}', '…', {snippet_size})"
             ' FROM fts_table WHERE fts_table MATCH ? ORDER BY RANK'
         )
         return list(self.execute(stmt, (unicode_normalize(query),)))
 
 
 def tokenize(text, flags=None, remove_diacritics=True):
-    from calibre_extensions.sqlite_extension import tokenize, FTS5_TOKENIZE_DOCUMENT
+    from calibre_extensions.sqlite_extension import FTS5_TOKENIZE_DOCUMENT, tokenize
     if flags is None:
         flags = FTS5_TOKENIZE_DOCUMENT
     return tokenize(unicode_normalize(text), remove_diacritics, flags)
@@ -69,17 +72,18 @@ class FTSTest(BaseTest):
         set_ui_language('en')
 
     def test_fts_tokenize(self):  # {{{
-        from calibre_extensions.sqlite_extension import set_ui_language
+        from calibre_extensions.sqlite_extension import FTS5_TOKENIZE_DOCUMENT, FTS5_TOKENIZE_QUERY, set_ui_language
 
         def t(x, s, e, f=0):
             return {'text': x, 'start': s, 'end': e, 'flags': f}
 
-        def tt(text, *expected_tokens):
-            q = tuple(x['text'] for x in tokenize(text))
+        def tt(text, *expected_tokens, for_query=False):
+            flags = FTS5_TOKENIZE_QUERY if for_query else FTS5_TOKENIZE_DOCUMENT
+            q = tuple(x['text'] for x in tokenize(text, flags=flags))
             self.ae(q, expected_tokens)
 
         self.ae(
-            tokenize("Some wörds"),
+            tokenize('Some wörds'),
             [t('some', 0, 4), t('wörds', 5, 11), t('words', 5, 11, 1)]
         )
         self.ae(
@@ -87,22 +91,24 @@ class FTSTest(BaseTest):
             [t("don't", 0, 5), t('bug', 7, 10)]
         )
         self.ae(
-            tokenize("a,b. c"),
-            [t("a", 0, 1), t('b', 2, 3), t('c', 5, 6)]
+            tokenize('a,b. c'),
+            [t('a', 0, 1), t('b', 2, 3), t('c', 5, 6)]
         )
         self.ae(
-            tokenize("a*b+c"),
-            [t("a", 0, 1), t('b', 2, 3), t('c', 4, 5)]
+            tokenize('a*b+c'),
+            [t('a', 0, 1), t('b', 2, 3), t('c', 4, 5)]
         )
         self.ae(
-            tokenize("a(b[{^c"),
-            [t("a", 0, 1), t('b', 2, 3), t('c', 6, 7)]
+            tokenize('a(b[{^c'),
+            [t('a', 0, 1), t('b', 2, 3), t('c', 6, 7)]
         )
         self.ae(
-            tokenize("a😀smile"),
-            [t("a", 0, 1), t('😀', 1, 5), t('smile', 5, 10)]
+            tokenize('a😀smile'),
+            [t('a', 0, 1), t('😀', 1, 5), t('smile', 5, 10)]
         )
 
+        tt("你don't叫mess", '你', "don't", '叫', 'mess')
+        tt("你don't叫mess", '你', "don't", '叫', 'mess', for_query=True)
         tt('你叫什么名字', '你', '叫', '什么', '名字')
         tt('你叫abc', '你', '叫', 'abc')
         tt('a你b叫什么名字', 'a', '你', 'b', '叫', '什么', '名字')
@@ -119,24 +125,28 @@ class FTSTest(BaseTest):
         conn = TestConn()
         conn.insert_text('two words, and a period. With another.')
         conn.insert_text('and another re-init')
-        self.ae(conn.search("another"), [('and >another< re-init',), ('…With >another<.',)])
-        self.ae(conn.search("period"), [('…a >period<. With another.',)])
+        self.ae(conn.search('another'), [('and >another< re-init',), ('…With >another<.',)])
+        self.ae(conn.search('period'), [('…a >period<. With another.',)])
         self.ae(conn.term_row_counts(), {'a': 1, 're': 1, 'init': 1, 'and': 2, 'another': 2, 'period': 1, 'two': 1, 'with': 1, 'words': 1})
         conn = TestConn()
         conn.insert_text('coộl')
         self.ae(conn.term_row_counts(), {'cool': 1, 'coộl': 1})
-        self.ae(conn.search("cool"), [('>coộl<',)])
-        self.ae(conn.search("coộl"), [('>coộl<',)])
+        self.ae(conn.search('cool'), [('>coộl<',)])
+        self.ae(conn.search('coộl'), [('>coộl<',)])
         conn = TestConn(remove_diacritics=False)
         conn.insert_text('coộl')
         self.ae(conn.term_row_counts(), {'coộl': 1})
 
         conn = TestConn()
         conn.insert_text("你don't叫mess")
-        self.ae(conn.search("mess"), [("你don't叫>mess<",)])
+        self.ae(conn.term_row_counts(), {"don't": 1, 'mess': 1, '你': 1, '叫': 1})
+        self.ae(conn.search('mess'), [("你don't叫>mess<",)])
         self.ae(conn.search('''"don't"'''), [("你>don't<叫mess",)])
-        self.ae(conn.search("你"), [(">你<don't叫mess",)])
-        self.ae(conn.search("叫"), [("你don't>叫<mess",)])
+        self.ae(conn.search('你'), [(">你<don't叫mess",)])
+        import apsw
+        if apsw.sqlitelibversion() not in ('3.44.0', '3.44.1', '3.44.2'):
+            # see https://www.sqlite.org/forum/forumpost/d16aeb397d
+            self.ae(conn.search('叫'), [("你don't>叫<mess",)])
     # }}}
 
     def test_fts_stemming(self):  # {{{
@@ -152,10 +162,10 @@ class FTSTest(BaseTest):
         conn = TestConn(stem_words=True)
         conn.insert_text('a simplistic connection')
         self.ae(conn.term_row_counts(), {'a': 1, 'connect': 1, 'simplist': 1})
-        self.ae(conn.search("connection"), [('a simplistic >connection<',),])
-        self.ae(conn.search("connect"), [('a simplistic >connection<',),])
-        self.ae(conn.search("simplistic connect"), [('a >simplistic< >connection<',),])
-        self.ae(conn.search("simplist"), [('a >simplistic< connection',),])
+        self.ae(conn.search('connection'), [('a simplistic >connection<',),])
+        self.ae(conn.search('connect'), [('a simplistic >connection<',),])
+        self.ae(conn.search('simplistic connect'), [('a >simplistic< >connection<',),])
+        self.ae(conn.search('simplist'), [('a >simplistic< connection',),])
 
     # }}}
 
@@ -183,6 +193,78 @@ class FTSTest(BaseTest):
 
     # }}}
 
+    def test_fts_indexing(self):
+        pdf_data = '''\
+%PDF-1.1
+%¥±ë
+
+1 0 obj
+  << /Type /Catalog
+     /Pages 2 0 R
+  >>
+endobj
+
+2 0 obj
+  << /Type /Pages
+     /Kids [3 0 R]
+     /Count 1
+     /MediaBox [0 0 300 144]
+  >>
+endobj
+
+3 0 obj
+  <<  /Type /Page
+      /Parent 2 0 R
+      /Resources
+       << /Font
+           << /F1
+               << /Type /Font
+                  /Subtype /Type1
+                  /BaseFont /Times-Roman
+               >>
+           >>
+       >>
+      /Contents 4 0 R
+  >>
+endobj
+
+4 0 obj
+  << /Length 55 >>
+stream
+  BT
+    /F1 18 Tf
+    0 0 Td
+    (Hello World) Tj
+  ET
+endstream
+endobj
+
+xref
+0 5
+0000000000 65535 f
+0000000018 00000 n
+0000000077 00000 n
+0000000178 00000 n
+0000000457 00000 n
+trailer
+  <<  /Root 1 0 R
+      /Size 5
+  >>
+startxref
+565
+%%EOF'''
+        with tempfile.TemporaryDirectory() as tdir:
+            pdf = os.path.join(tdir, 'test.pdf')
+            with open(pdf, 'w') as f:
+                f.write(pdf_data)
+            from calibre.db.fts.text import extract_text
+            self.assertEqual(extract_text(pdf).strip(), 'Hello World')
+            from zipfile import ZipFile
+            zip = os.path.join(tdir, 'test.zip')
+            with ZipFile(zip, 'w') as zf:
+                zf.writestr('text.pdf', pdf_data)
+            self.assertEqual(extract_text(zip).strip(), 'Hello World')
+
 
 def find_tests():
     import unittest
