@@ -1,13 +1,15 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.13
 
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import time, random
+import random
+import time
 from threading import Thread
+
+from calibre.db.locking import LockingError, RWLockWrapper, SHLock
 from calibre.db.tests.base import BaseTest
-from calibre.db.locking import SHLock, RWLockWrapper, LockingError
 
 
 def wait_for(period):
@@ -20,7 +22,7 @@ def wait_for(period):
 
 
 class TestLock(BaseTest):
-    """Tests for db locking """
+    '''Tests for db locking '''
 
     def test_owns_locks(self):
         lock = SHLock()
