@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.9
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
@@ -14,7 +14,7 @@ def find_tests():
 
 if __name__ == '__main__':
     try:
-        import init_calibre  # noqa
+        import init_calibre  # noqa: F401
     except ImportError:
         pass
     run_tests(find_tests)
