@@ -1,10 +1,11 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.13
 
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os, cProfile
+import cProfile
+import os
 from tempfile import gettempdir
 
 from calibre.db.legacy import LibraryDatabase
@@ -28,7 +29,7 @@ def main():
     stats = os.path.join(gettempdir(), 'read_db.stats')
     pr = cProfile.Profile()
     initdb('~/test library')
-    all_ids = db.new_api.all_book_ids()  # noqa
+    all_ids = db.new_api.all_book_ids()
     pr.enable()
     for book_id in all_ids:
         db.new_api._composite_for('#isbn', book_id)
