@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
@@ -6,12 +6,13 @@ __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 import datetime
+import os
 from io import BytesIO
 from time import time
 
+from calibre.db.tests.base import BaseTest
 from calibre.utils.date import utc_tz
 from calibre.utils.localization import calibre_langcode_to_name
-from calibre.db.tests.base import BaseTest
 from polyglot.builtins import iteritems, itervalues
 
 
@@ -25,12 +26,12 @@ class ReadingTest(BaseTest):
         'Test the reading of data from the database'
         cache = self.init_cache(self.library_path)
         tests = {
-                3  : {
+                3: {
                     'title': 'Unknown',
                     'sort': 'Unknown',
                     'authors': ('Unknown',),
                     'author_sort': 'Unknown',
-                    'series' : None,
+                    'series': None,
                     'series_index': 1.0,
                     'rating': None,
                     'tags': (),
@@ -57,12 +58,12 @@ class ReadingTest(BaseTest):
                     'size':None,
                 },
 
-                2 : {
+                2: {
                     'title': 'Title One',
                     'sort': 'One',
                     'authors': ('Author One',),
                     'author_sort': 'One, Author',
-                    'series' : 'A Series One',
+                    'series': 'A Series One',
                     'series_index': 1.0,
                     'tags':('Tag One', 'Tag Two'),
                     'formats': ('FMT1',),
@@ -87,12 +88,12 @@ class ReadingTest(BaseTest):
                     '#comments': '<div>My Comments One<p></p></div>',
                     'size':9,
                 },
-                1  : {
+                1: {
                     'title': 'Title Two',
                     'sort': 'Title Two',
                     'authors': ('Author Two', 'Author One'),
                     'author_sort': 'Two, Author & One, Author',
-                    'series' : 'A Series One',
+                    'series': 'A Series One',
                     'series_index': 2.0,
                     'rating': 6.0,
                     'tags': ('Tag One', 'News'),
@@ -125,8 +126,7 @@ class ReadingTest(BaseTest):
                 if isinstance(val, tuple) and 'authors' not in field and 'languages' not in field:
                     val, expected_val = set(val), set(expected_val)
                 self.assertEqual(expected_val, val,
-                        'Book id: %d Field: %s failed: %r != %r'%(
-                            book_id, field, expected_val, val))
+                        f'Book id: {book_id} Field: {field} failed: {expected_val!r} != {val!r}')
         # }}}
 
     def test_sorting(self):  # {{{
@@ -164,10 +164,10 @@ class ReadingTest(BaseTest):
             x = list(reversed(order))
             ae(order, cache.multisort([(field, True)],
                 ids_to_sort=x),
-                    'Ascending sort of %s failed'%field)
+                    f'Ascending sort of {field} failed')
             ae(x, cache.multisort([(field, False)],
                 ids_to_sort=order),
-                    'Descending sort of %s failed'%field)
+                    f'Descending sort of {field} failed')
 
         # Test sorting of is_multiple fields.
 
@@ -205,7 +205,7 @@ class ReadingTest(BaseTest):
             ('title', True)]), 'Subsort failed')
         from calibre.ebooks.metadata.book.base import Metadata
         for i in range(7):
-            cache.create_book_entry(Metadata('title%d' % i), apply_import_tags=False)
+            cache.create_book_entry(Metadata(f'title{i}'), apply_import_tags=False)
         cache.create_custom_column('one', 'CC1', 'int', False)
         cache.create_custom_column('two', 'CC2', 'int', False)
         cache.create_custom_column('three', 'CC3', 'int', False)
@@ -216,7 +216,7 @@ class ReadingTest(BaseTest):
         cache.set_field('#two', {10:2})
         cache.set_field('#three', {i:i for i in range(1, 11)})
         ae(list(range(1, 11)), cache.multisort([('#one', True), ('#two', True)], ids_to_sort=sorted(cache.all_book_ids())))
-        ae([4, 5, 1, 2, 3, 7,8, 9, 10, 6], cache.multisort([('#one', True), ('#two', False)], ids_to_sort=sorted(cache.all_book_ids())))
+        ae([4, 5, 1, 2, 3, 7, 8, 9, 10, 6], cache.multisort([('#one', True), ('#two', False)], ids_to_sort=sorted(cache.all_book_ids())))
         ae([5, 4, 3, 2, 1, 10, 9, 8, 7, 6], cache.multisort([('#one', True), ('#two', False), ('#three', False)], ids_to_sort=sorted(cache.all_book_ids())))
     # }}}
 
@@ -244,8 +244,8 @@ class ReadingTest(BaseTest):
     # }}}
 
     def test_serialize_metadata(self):  # {{{
-        from calibre.utils.serialize import json_dumps, json_loads, msgpack_dumps, msgpack_loads
         from calibre.library.field_metadata import fm_as_dict
+        from calibre.utils.serialize import json_dumps, json_loads, msgpack_dumps, msgpack_loads
         cache = self.init_cache(self.library_path)
         fm = cache.field_metadata
         for d, l in ((json_dumps, json_loads), (msgpack_dumps, msgpack_loads)):
@@ -270,7 +270,11 @@ class ReadingTest(BaseTest):
         for book_id, cdata in iteritems(covers):
             self.assertEqual(cdata, cache.cover(book_id), 'Reading of cover failed')
             f = cache.cover(book_id, as_file=True)
-            self.assertEqual(cdata, f.read() if f else f, 'Reading of cover as file failed')
+            try:
+                self.assertEqual(cdata, f.read() if f else f, 'Reading of cover as file failed')
+            finally:
+                if f:
+                    f.close()
             if cdata:
                 with open(cache.cover(book_id, as_path=True), 'rb') as f:
                     self.assertEqual(cdata, f.read(), 'Reading of cover as path failed')
@@ -332,8 +336,7 @@ class ReadingTest(BaseTest):
         for query, ans in iteritems(oldvals):
             nr = cache.search(query, '')
             self.assertEqual(ans, nr,
-                'Old result: %r != New result: %r for search: %s'%(
-                    ans, nr, query))
+                f'Old result: {ans!r} != New result: {nr!r} for search: {query}')
 
         # Test searching by id, which was introduced in the new backend
         self.assertEqual(cache.search('id:1', ''), {1})
@@ -366,8 +369,8 @@ class ReadingTest(BaseTest):
         cache.set_field('timestamp', {1:p('2002-02-06'), 2:p('2000-10-06'), 3:p('2001-06-06')})
         # Test numeric compare search
         self.assertEqual(cache.search("template:\"program: "
-                                      "floor(days_between(field(\'pubdate\'), "
-                                      "field(\'timestamp\')))#@#:n:>0\""), {2,3})
+                                      "floor(days_between(field('pubdate'), "
+                                      "field('timestamp')))#@#:n:>0\""), {2,3})
         # Test date search
         self.assertEqual(cache.search('template:{pubdate}#@#:d:<2001-09-01"'), {1,3})
         # Test boolean search
@@ -375,7 +378,7 @@ class ReadingTest(BaseTest):
         self.assertEqual(cache.search('template:{series}#@#:b:false'), {3})
 
         # test primary search
-        cache.set_field('title', {1: "Gravity’s Raiñbow"})
+        cache.set_field('title', {1: 'Gravity’s Raiñbow'})
         self.assertEqual(cache.search('title:"Gravity\'s Rainbow"'), {1})
         # Note that the old db searched uuid for un-prefixed searches, the new
         # db does not, for performance
@@ -409,13 +412,12 @@ class ReadingTest(BaseTest):
                 ):
                     continue
                 self.assertEqual(oval, nval,
-                    'The attribute %s for %s in category %s does not match. Old is %r, New is %r'
-                                %(attr, old.name, category, oval, nval))
+                    f'The attribute {attr} for {old.name} in category {category} does not match. Old is {oval!r}, New is {nval!r}')
 
         for category in old_categories:
             old, new = old_categories[category], new_categories[category]
             self.assertEqual(len(old), len(new),
-                'The number of items in the category %s is not the same'%category)
+                f'The number of items in the category {category} is not the same')
             for o, n in zip(old, new):
                 compare_category(category, o, n)
 
@@ -423,8 +425,8 @@ class ReadingTest(BaseTest):
 
     def test_get_formats(self):  # {{{
         'Test reading ebook formats using the format() method'
-        from calibre.library.database2 import LibraryDatabase2
         from calibre.db.cache import NoSuchFormat
+        from calibre.library.database2 import LibraryDatabase2
         old = LibraryDatabase2(self.library_path)
         ids = old.all_ids()
         lf = {i:set(old.formats(i, index_is_id=True).split(',')) if old.formats(
@@ -441,9 +443,9 @@ class ReadingTest(BaseTest):
                 old = formats[book_id][fmt]
                 self.assertEqual(old, cache.format(book_id, fmt),
                                  'Old and new format disagree')
-                f = cache.format(book_id, fmt, as_file=True)
-                self.assertEqual(old, f.read(),
-                                 'Failed to read format as file')
+                with cache.format(book_id, fmt, as_file=True) as f:
+                    self.assertEqual(old, f.read(),
+                                    'Failed to read format as file')
                 with open(cache.format(book_id, fmt, as_path=True,
                                        preserve_filename=True), 'rb') as f:
                     self.assertEqual(old, f.read(),
@@ -455,6 +457,12 @@ class ReadingTest(BaseTest):
         buf = BytesIO()
         self.assertRaises(NoSuchFormat, cache.copy_format_to, 99999, 'X', buf, 'copy_format_to() failed to raise an exception for non-existent book')
         self.assertRaises(NoSuchFormat, cache.copy_format_to, 1, 'X', buf, 'copy_format_to() failed to raise an exception for non-existent format')
+        fmt = cache.formats(1)[0]
+        path = cache.format_abspath(1, fmt)
+        changed_path = os.path.join(os.path.dirname(path), 'x' + os.path.basename(path))
+        os.rename(path, changed_path)
+        self.assertEqual(cache.format_abspath(1, fmt), path)
+        self.assertFalse(os.path.exists(changed_path))
 
     # }}}
 
@@ -505,8 +513,8 @@ class ReadingTest(BaseTest):
 
     def test_datetime(self):  # {{{
         ' Test the reading of datetimes stored in the db '
+        from calibre.db.tables import UNDEFINED_DATE, _c_speedup, c_parse
         from calibre.utils.date import parse_date
-        from calibre.db.tables import c_parse, UNDEFINED_DATE, _c_speedup
 
         # First test parsing of string to UTC time
         for raw in ('2013-07-22 15:18:29+05:30', '  2013-07-22 15:18:29+00:00', '2013-07-22 15:18:29', '2003-09-21 23:30:00-06:00'):
@@ -584,7 +592,7 @@ class ReadingTest(BaseTest):
         test(True, {3}, 'Unknown')
         c.limit = 5
         for i in range(6):
-            test(False, set(), 'nomatch_%s' % i)
+            test(False, set(), f'nomatch_{i}')
         test(False, {3}, 'Unknown')  # cached search expired
         test(False, {3}, '', 'unknown', num=1)
         test(True, {3}, '', 'unknown', num=1)
@@ -606,9 +614,12 @@ class ReadingTest(BaseTest):
             self.assertSetEqual(set(mi.custom_field_keys()), set(pmi.custom_field_keys()))
 
             for field in STANDARD_METADATA_FIELDS | {'#series_index'}:
-                f = lambda x: x
                 if field == 'formats':
-                    f = lambda x: x if x is None else tuple(x)
+                    def f(x):
+                        return (x if x is None else tuple(x))
+                else:
+                    def f(x):
+                        return x
                 self.assertEqual(f(getattr(mi, field)), f(getattr(pmi, field)),
                                 f'Standard field: {field} not the same for book {book_id}')
                 self.assertEqual(mi.format_field(field), pmi.format_field(field),
@@ -624,7 +635,7 @@ class ReadingTest(BaseTest):
                     v = pmi.get_standard_metadata(field)
                     self.assertTrue(v is None or isinstance(v, dict))
                     self.assertEqual(f(mi.get_standard_metadata(field, False)), f(v),
-                                     'get_standard_metadata() failed for field %s' % field)
+                                     f'get_standard_metadata() failed for field {field}')
             for field, meta in cache.field_metadata.custom_iteritems():
                 if meta['datatype'] != 'composite':
                     self.assertEqual(f(getattr(mi, field)), f(getattr(pmi, field)),
@@ -640,6 +651,64 @@ class ReadingTest(BaseTest):
         mi, pmi = cache.get_metadata(1), cache.get_proxy_metadata(1)
         self.assertEqual(mi.get('#comp1'), pmi.get('#comp1'))
 
+        # Test overridden Metadata methods
+
+        self.assertTrue(pmi.has_key('tags') == mi.has_key('tags'))
+
+        self.assertFalse(pmi.has_key('taggs'), 'taggs attribute')
+        self.assertTrue(pmi.has_key('taggs') == mi.has_key('taggs'))
+
+        self.assertSetEqual(set(pmi.custom_field_keys()), set(mi.custom_field_keys()))
+
+        self.assertEqual(pmi.get_extra('#series', 0), 3)
+        self.assertEqual(pmi.get_extra('#series', 0), mi.get_extra('#series', 0))
+
+        self.assertDictEqual(pmi.get_identifiers(), {'test': 'two'})
+        self.assertDictEqual(pmi.get_identifiers(), mi.get_identifiers())
+
+        self.assertTrue(pmi.has_identifier('test'))
+        self.assertTrue(pmi.has_identifier('test') == mi.has_identifier('test'))
+
+        self.assertListEqual(list(pmi.custom_field_keys()), list(mi.custom_field_keys()))
+
+        # ProxyMetadata has the virtual fields while Metadata does not.
+        self.assertSetEqual(set(pmi.all_field_keys())-{'id', 'series_sort', 'path',
+                                                       'in_tag_browser', 'sort', 'ondevice',
+                                                       'au_map', 'marked', '#series_index'},
+                            set(mi.all_field_keys()))
+
+        # mi.get_standard_metadata() doesn't include the rec_index metadata key
+        fm_pmi = pmi.get_standard_metadata('series')
+        fm_pmi.pop('rec_index')
+        self.assertDictEqual(fm_pmi, mi.get_standard_metadata('series', make_copy=False))
+
+        # The ProxyMetadata versions don't include the values. Note that the mi
+        # version of get_standard_metadata won't return custom columns while the
+        # ProxyMetadata version will
+        fm_mi = mi.get_user_metadata('#series', make_copy=False)
+        fm_mi.pop('#extra#')
+        fm_mi.pop('#value#')
+        self.assertDictEqual(pmi.get_standard_metadata('#series'), fm_mi)
+        self.assertDictEqual(pmi.get_user_metadata('#series'), fm_mi)
+
+        fm_mi = mi.get_all_user_metadata(make_copy=False)
+        for one in fm_mi:
+            fm_mi[one].pop('#extra#', None)
+            fm_mi[one].pop('#value#', None)
+        self.assertDictEqual(pmi.get_all_user_metadata(make_copy=False), fm_mi)
+
+        # Check the unimplemented methods
+        self.assertRaises(NotImplementedError, lambda: 'foo' in pmi)
+        self.assertRaises(NotImplementedError, pmi.set, 'a', 'a')
+        self.assertRaises(NotImplementedError, pmi.set_identifiers, 'a', 'a')
+        self.assertRaises(NotImplementedError, pmi.set_identifier, 'a', 'a')
+        self.assertRaises(NotImplementedError, pmi.all_non_none_fields)
+        self.assertRaises(NotImplementedError, pmi.set_all_user_metadata, {})
+        self.assertRaises(NotImplementedError, pmi.set_user_metadata, 'a', {})
+        self.assertRaises(NotImplementedError, pmi.remove_stale_user_metadata, {})
+        self.assertRaises(NotImplementedError, pmi.template_to_attribute, {}, {})
+        self.assertRaises(NotImplementedError, pmi.smart_update, {})
+
     # }}}
 
     def test_marked_field(self):  # {{{
@@ -703,8 +772,8 @@ class ReadingTest(BaseTest):
 
     def test_find_identical_books(self):  # {{{
         ' Test find_identical_books '
-        from calibre.ebooks.metadata.book.base import Metadata
         from calibre.db.utils import find_identical_books
+        from calibre.ebooks.metadata.book.base import Metadata
         # 'find_identical_books': [(,), (Metadata('unknown'),), (Metadata('xxxx'),)],
         cache = self.init_cache(self.library_path)
         cache.set_field('languages', {1: ('fra', 'deu')})
@@ -745,3 +814,144 @@ class ReadingTest(BaseTest):
                 val = val.encode('utf-8')
             self.assertEqual(got, val)
     # }}}
+
+    def test_template_db_functions(self):  # {{{
+        from calibre.ebooks.metadata.book.formatter import SafeFormat
+        formatter = SafeFormat()
+
+        db = self.init_cache(self.library_path)
+        db.create_custom_column('mult', 'CC1', 'composite', True, display={'composite_template': 'b,a,c'})
+
+        # need an empty metadata object to pass to the formatter
+        db = self.init_legacy(self.library_path)
+        mi = db.get_metadata(1)
+
+        # test counting books matching the search
+        v = formatter.safe_format('program: book_count("series:true", 0)', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(v, '2')
+
+        # test counting books when none match the search
+        v = formatter.safe_format('program: book_count("series:afafaf", 0)', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(v, '0')
+
+        # test is_multiple values
+        v = formatter.safe_format('program: book_values("tags", "tags:true", ",", 0)', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(set(v.split(',')), {'Tag One', 'News', 'Tag Two'})
+
+        # test not is_multiple values
+        v = formatter.safe_format('program: book_values("series", "series:true", ",", 0)', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(v, 'A Series One')
+
+        # test returning values for a column not searched for
+        v = formatter.safe_format('program: book_values("tags", "series:\\"A Series One\\"", ",", 0)', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(set(v.split(',')), {'Tag One', 'News', 'Tag Two'})
+
+        # test getting a singleton value from books where the column is empty
+        v = formatter.safe_format('program: book_values("series", "series:false", ",", 0)', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(v, '')
+
+        # test getting a multiple value from books where the column is empty
+        v = formatter.safe_format('program: book_values("tags", "tags:false", ",", 0)', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(v, '')
+
+        # test fetching an unknown column
+        v = formatter.safe_format('program: book_values("taaags", "tags:false", ",", 0)', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(v, "TEMPLATE ERROR The column taaags doesn't exist")
+
+        # test finding all books
+        v = formatter.safe_format('program: book_values("id", "title:true", ",", 0)', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(set(v.split(',')), {'1', '2', '3'})
+
+        # test getting value of a composite
+        v = formatter.safe_format('program: book_values("#mult", "id:1", ",", 0)', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(set(v.split(',')), {'b', 'c', 'a'})
+
+        # test getting value of a custom float
+        v = formatter.safe_format('program: book_values("#float", "title:true", ",", 0)', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(set(v.split(',')), {'20.02', '10.01'})
+
+        # test getting value of an int (rating)
+        v = formatter.safe_format('program: book_values("rating", "title:true", ",", 0)', {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(set(v.split(',')), {'4', '6'})
+    # }}}
+
+    def test_python_templates(self):  # {{{
+        from calibre.ebooks.metadata.book.formatter import SafeFormat
+        formatter = SafeFormat()
+
+        # need an empty metadata object to pass to the formatter
+        db = self.init_legacy(self.library_path)
+        mi = db.get_metadata(1)
+
+        # test counting books matching a search
+        template = '''python:
+def evaluate(book, ctx):
+    ids = ctx.db.new_api.search("series:true")
+    return str(len(ids))
+'''
+        v = formatter.safe_format(template, {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(v, '2')
+
+        # test counting books when none match the search
+        template = '''python:
+def evaluate(book, ctx):
+    ids = ctx.db.new_api.search("series:afafaf")
+    return str(len(ids))
+'''
+        v = formatter.safe_format(template, {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(v, '0')
+
+        # test is_multiple values
+        template = '''python:
+def evaluate(book, ctx):
+    tags = ctx.db.new_api.all_field_names('tags')
+    return ','.join(list(tags))
+'''
+        v = formatter.safe_format(template, {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(set(v.split(',')), {'Tag One', 'News', 'Tag Two'})
+
+        # test using a custom context class
+        template = '''python:
+def evaluate(book, ctx):
+    tags = ctx.db.new_api.all_field_names('tags')
+    return ','.join(list(ctx.helper_function(tags)))
+'''
+        from calibre.utils.formatter import PythonTemplateContext
+
+        class CustomContext(PythonTemplateContext):
+            def helper_function(self, arg):
+                s = set(arg)
+                s.add('helper called')
+                return s
+
+        v = formatter.safe_format(template, {}, 'TEMPLATE ERROR', mi,
+                                  python_context_object=CustomContext())
+        self.assertEqual(set(v.split(',')), {'Tag One', 'News', 'Tag Two','helper called'})
+
+        # test is_multiple values
+        template = '''python:
+def evaluate(book, ctx):
+    tags = ctx.db.new_api.all_field_names('tags')
+    return ','.join(list(tags))
+'''
+        v = formatter.safe_format(template, {}, 'TEMPLATE ERROR', mi)
+        self.assertEqual(set(v.split(',')), {'Tag One', 'News', 'Tag Two'})
+
+        # test calling a python stored template from a GPM template
+        from calibre.utils.formatter_functions import load_user_template_functions, unload_user_template_functions
+        load_user_template_functions('aaaaa',
+                                     [['python_stored_template',
+                                      '',
+                                      0,
+                                      '''python:
+def evaluate(book, ctx):
+    tags = set(ctx.db.new_api.all_field_names('tags'))
+    tags.add(ctx.arguments[0])
+    return ','.join(list(tags))
+'''
+                                      ]], None)
+        v = formatter.safe_format('program: python_stored_template("one argument")', {},
+                                  'TEMPLATE ERROR', mi)
+        unload_user_template_functions('aaaaa')
+        self.assertEqual(set(v.split(',')), {'Tag One', 'News', 'Tag Two', 'one argument'})
+    # }}}
