@@ -1,17 +1,20 @@
-#!/usr/local/bin/python3.9
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import weakref, operator, numbers
+import numbers
+import operator
+import sys
+import weakref
 from functools import partial
-from polyglot.builtins import iteritems, itervalues
 
-from calibre.ebooks.metadata import title_sort
-from calibre.utils.config_base import tweaks, prefs
 from calibre.db.write import uniq
+from calibre.ebooks.metadata import title_sort
+from calibre.utils.config_base import prefs, tweaks
+from polyglot.builtins import iteritems, itervalues
 
 
 def sanitize_sort_field_name(field_metadata, field):
@@ -35,6 +38,29 @@ class MarkedVirtualField:
         return lambda book_id:g(book_id, '')
 
 
+class InTagBrowserVirtualField:
+
+    def __init__(self, _ids):
+        self._ids = _ids
+
+    def iter_searchable_values(self, get_metadata, candidates, default_value=None):
+        # The returned value can be any string. For example it could be book_id
+        # as a string, but there is little point in spending the cpu cycles for
+        # that as no one will do a search like in_tag_browser:1544 (a book id)
+        for book_id in candidates:
+            yield 'A' if self._ids is None or book_id in self._ids else default_value, {book_id}
+
+    def sort_keys_for_books(self, get_metadata, lang_map):
+        null = sys.maxsize
+        if self._ids is None:
+            def key(_id):
+                return null
+        else:
+            def key(_id):
+                return _id if _id in self._ids else null
+        return key
+
+
 class TableRow:
 
     def __init__(self, book_id, view):
@@ -69,17 +95,17 @@ def format_is_multiple(x, sep=',', repl=None):
 def format_identifiers(x):
     if not x:
         return None
-    return ','.join('%s:%s'%(k, v) for k, v in iteritems(x))
+    return ','.join(f'{k}:{v}' for k, v in iteritems(x))
 
 
 class View:
-
     ''' A table view of the database, with rows and columns. Also supports
     filtering and sorting.  '''
 
     def __init__(self, cache):
         self.cache = cache
         self.marked_ids = {}
+        self.tag_browser_ids = None
         self.marked_listeners = {}
         self.search_restriction_book_count = 0
         self.search_restriction = self.base_restriction = ''
@@ -125,6 +151,8 @@ class View:
                 fmt = partial(format_is_multiple, sep=sep)
             self._field_getters[idx] = partial(func, label, fmt=fmt) if func == self._get else func
 
+        self._real_map_filtered = ()
+        self._real_map_filtered_id_to_row = {}
         self._map = tuple(sorted(self.cache.all_book_ids()))
         self._map_filtered = tuple(self._map)
         self.full_map_is_sorted = True
@@ -147,13 +175,22 @@ class View:
     def sanitize_sort_field_name(self, field):
         return sanitize_sort_field_name(self.field_metadata, field)
 
+    @property
+    def _map_filtered(self):
+        return self._real_map_filtered
+
+    @_map_filtered.setter
+    def _map_filtered(self, v):
+        self._real_map_filtered = v
+        self._real_map_filtered_id_to_row = {id_:row for row, id_ in enumerate(self._map_filtered)}
+
     @property
     def field_metadata(self):
         return self.cache.field_metadata
 
     def _get_id(self, idx, index_is_id=True):
         if index_is_id and not self.cache.has_id(idx):
-            raise IndexError('No book with id %s present'%idx)
+            raise IndexError(f'No book with id {idx} present')
         return idx if index_is_id else self.index_to_id(idx)
 
     def has_id(self, book_id):
@@ -191,17 +228,21 @@ class View:
         return self._map_filtered[idx]
 
     def id_to_index(self, book_id):
-        return self._map_filtered.index(book_id)
+        try:
+            return self._real_map_filtered_id_to_row[book_id]
+        except KeyError:
+            raise ValueError(f'No such book_id {book_id} in current view')
     row = index_to_id
 
     def index(self, book_id, cache=False):
-        x = self._map if cache else self._map_filtered
-        return x.index(book_id)
+        if cache:
+            return self._map.index(book_id)
+        return self.id_to_index(book_id)
 
     def _get(self, field, idx, index_is_id=True, default_value=None, fmt=lambda x:x):
         id_ = idx if index_is_id else self.index_to_id(idx)
         if index_is_id and not self.cache.has_id(id_):
-            raise IndexError('No book with id %s present'%idx)
+            raise IndexError(f'No book with id {idx} present')
         return fmt(self.cache.field_for(field, id_, default_value=default_value))
 
     def get_series_sort(self, idx, index_is_id=True, default_value=''):
@@ -235,7 +276,8 @@ class View:
 
     def get_virtual_libraries_for_books(self, ids):
         return self.cache.virtual_libraries_for_books(
-            ids, virtual_fields={'marked':MarkedVirtualField(self.marked_ids)})
+            ids, virtual_fields={'marked':MarkedVirtualField(self.marked_ids),
+                                 'in_tag_browser': InTagBrowserVirtualField(self.tag_browser_ids)})
 
     def _do_sort(self, ids_to_sort, fields=(), subsort=False):
         fields = [(sanitize_sort_field_name(self.field_metadata, x), bool(y)) for x, y in fields]
@@ -248,7 +290,8 @@ class View:
 
         return self.cache.multisort(
             fields, ids_to_sort=ids_to_sort,
-            virtual_fields={'marked':MarkedVirtualField(self.marked_ids)})
+            virtual_fields={'marked':MarkedVirtualField(self.marked_ids),
+                            'in_tag_browser': InTagBrowserVirtualField(self.tag_browser_ids)})
 
     def multisort(self, fields=[], subsort=False, only_ids=None):
         sorted_book_ids = self._do_sort(self._map if only_ids is None else only_ids, fields=fields, subsort=subsort)
@@ -309,7 +352,8 @@ class View:
                 self.full_map_is_sorted = True
             return rv
         matches = self.cache.search(
-            query, search_restriction, virtual_fields={'marked':MarkedVirtualField(self.marked_ids)})
+            query, search_restriction, virtual_fields={'marked':MarkedVirtualField(self.marked_ids),
+                                           'in_tag_browser': InTagBrowserVirtualField(self.tag_browser_ids)})
         if len(matches) == len(self._map):
             rv = list(self._map)
         else:
@@ -361,6 +405,12 @@ class View:
     def change_search_locations(self, newlocs):
         self.cache.change_search_locations(newlocs)
 
+    def set_in_tag_browser(self, id_set):
+        self.tag_browser_ids = id_set
+
+    def get_in_tag_browser(self):
+        return self.tag_browser_ids
+
     def set_marked_ids(self, id_dict):
         '''
         ids in id_dict are "marked". They can be searched for by
@@ -399,6 +449,9 @@ class View:
         common = mids.intersection(book_ids)
         self.set_marked_ids((mids | book_ids) - common)
 
+    def add_marked_ids(self, book_ids):
+        self.set_marked_ids(set(self.marked_ids) | set(book_ids))
+
     def refresh(self, field=None, ascending=True, clear_caches=True, do_search=True):
         self._map = tuple(sorted(self.cache.all_book_ids()))
         self._map_filtered = tuple(self._map)
@@ -413,16 +466,9 @@ class View:
 
     def refresh_ids(self, ids):
         self.cache.clear_caches(book_ids=ids)
-
         # The ids list can contain invalid ids (deleted etc). We want to filter
         # those out while keeping the valid ones.
-        def f(id_):
-            try:
-                return self.id_to_index(id_)
-            except ValueError:
-                return None
-        res = [i for i in map(f, ids) if i is not None]
-        return res if res else None
+        return [self._real_map_filtered_id_to_row[id_] for id_ in ids if id_ in self._real_map_filtered_id_to_row] or None
 
     def remove(self, book_id):
         try:
