@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
@@ -7,20 +7,24 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 Embedded console for debugging.
 '''
 
-import sys, os, functools
-from calibre.utils.config import OptionParser
-from calibre.constants import iswindows
+import functools
+import os
+import sys
+
 from calibre import prints
+from calibre.constants import iswindows
 from calibre.startup import get_debug_executable
+from calibre.utils.config import OptionParser
 from polyglot.builtins import exec_path
 
 
 def run_calibre_debug(*args, **kw):
     import subprocess
     creationflags = 0
+    headless = bool(kw.pop('headless', False))
     if iswindows:
         creationflags = subprocess.CREATE_NO_WINDOW
-    cmd = get_debug_executable() + list(args)
+    cmd = get_debug_executable(headless=headless) + list(args)
     kw['creationflags'] = creationflags
     return subprocess.Popen(cmd, **kw)
 
@@ -41,22 +45,26 @@ You can also use %prog to run standalone scripts. To do that use it like this:
 
     {1}
 
-Everything after the -- is passed to the script.
-''').format(_('%prog [options]'), '%prog myscript.py -- --option1 --option2 file1 file2 ...'))
+Everything after the -- is passed to the script. You can also use calibre-debug
+as a shebang in scripts, like this:
+
+    {2}
+''').format(_('%prog [options]'), '%prog -e myscript.py -- --option1 --option2 file1 file2 ...', '#!/usr/bin/env -S calibre-debug -e -- --'))
     parser.add_option('-c', '--command', help=_('Run Python code.'))
     parser.add_option('-e', '--exec-file', help=_('Run the Python code in file.'))
     parser.add_option('-f', '--subset-font', action='store_true', default=False,
                       help=_('Subset the specified font. Use -- after this option to pass option to the font subsetting program.'))
     parser.add_option('-d', '--debug-device-driver', default=False, action='store_true',
                       help=_('Debug device detection'))
-    parser.add_option('-g', '--gui',  default=False, action='store_true',
+    parser.add_option('-g', '--gui', default=False, action='store_true',
                       help=_('Run the GUI with debugging enabled. Debug output is '
                       'printed to stdout and stderr.'))
-    parser.add_option('--gui-debug',  default=None,
+    parser.add_option('--gui-debug', default=None,
                       help=_('Run the GUI with a debug console, logging to the'
                       ' specified path. For internal use only, use the -g'
                       ' option to run the GUI in debug mode'))
-    parser.add_option('-w', '--viewer',  default=False, action='store_true',
+    parser.add_option('--run-without-debug', default=False, action='store_true', help=_("Don't run with the DEBUG flag set"))
+    parser.add_option('-w', '--viewer', default=False, action='store_true',
                       help=_('Run the E-book viewer in debug mode'))
     parser.add_option('--paths', default=False, action='store_true',
             help=_('Output the paths necessary to setup the calibre environment'))
@@ -64,9 +72,6 @@ Everything after the -- is passed to the script.
             help=_('Add a simple plugin (i.e. a plugin that consists of only a '
             '.py file), by specifying the path to the py file containing the '
             'plugin code.'))
-    parser.add_option('--reinitialize-db', default=None,
-            help=_('Re-initialize the sqlite calibre database at the '
-            'specified path. Useful to recover from db corruption.'))
     parser.add_option('-m', '--inspect-mobi', action='store_true',
             default=False,
             help=_('Inspect the MOBI file(s) at the specified path(s)'))
@@ -88,7 +93,11 @@ Everything after the -- is passed to the script.
             ' be asked for the export folder and the libraries to export. You can also specify them'
             ' as command line arguments to skip the questions.'
             ' Use absolute paths for the export folder and libraries.'
-            ' The special keyword "all" can be used to export all libraries.'))
+            ' The special keyword "all" can be used to export all libraries. Examples:\n\n'
+            '  calibre-debug --export-all-calibre-data  # for interactive use\n'
+            '  calibre-debug --export-all-calibre-data /path/to/empty/export/folder /path/to/library/folder1 /path/to/library2\n'
+            '  calibre-debug --export-all-calibre-data /export/folder all  # export all known libraries'
+    ))
     parser.add_option('--import-calibre-data', default=False, action='store_true',
         help=_('Import previously exported calibre data'))
     parser.add_option('-s', '--shutdown-running-calibre', default=False,
@@ -102,56 +111,26 @@ Everything after the -- is passed to the script.
         'Run a plugin that provides a command line interface. For example:\n'
         'calibre-debug -r "Plugin name" -- file1 --option1\n'
         'Everything after the -- will be passed to the plugin as arguments.'))
+    parser.add_option('-t', '--run-test', help=_(
+        'Run the named test(s). Use the special value "all" to run all tests.'
+        ' If the test name starts with a period it is assumed to be a module name.'
+        ' If the test name starts with @ it is assumed to be a category name.'
+    ))
     parser.add_option('--diff', action='store_true', default=False, help=_(
         'Run the calibre diff tool. For example:\n'
         'calibre-debug --diff file1 file2'))
     parser.add_option('--default-programs', default=None, choices=['register', 'unregister'],
                           help=_('(Un)register calibre from Windows Default Programs.') + ' --default-programs=(register|unregister)')
+    parser.add_option('--kepubify', default=False, action='store_true', help=_(
+        'Convert the specified EPUB file to KEPUB without doing a full conversion. This is what the Kobo driver does when sending files to the device.'))
+    parser.add_option('--un-kepubify', default=False, action='store_true', help=_(
+        'Convert the specified KEPUB file to EPUB without doing a full conversion. This is what the Kobo driver does when importing files from the device.'))
     parser.add_option('--fix-multiprocessing', default=False, action='store_true',
         help=_('For internal use'))
 
     return parser
 
 
-def reinit_db(dbpath):
-    from contextlib import closing
-    from calibre import as_unicode
-    from calibre.ptempfile import TemporaryFile
-    from calibre.utils.filenames import atomic_rename
-    # We have to use sqlite3 instead of apsw as apsw has no way to discard
-    # problematic statements
-    import sqlite3
-    from calibre.library.sqlite import do_connect
-    with TemporaryFile(suffix='_tmpdb.db', dir=os.path.dirname(dbpath)) as tmpdb:
-        with closing(do_connect(dbpath)) as src, closing(do_connect(tmpdb)) as dest:
-            dest.execute('create temporary table temp_sequence(id INTEGER PRIMARY KEY AUTOINCREMENT)')
-            dest.commit()
-            uv = int(src.execute('PRAGMA user_version;').fetchone()[0])
-            dump = src.iterdump()
-            last_restore_error = None
-            while True:
-                try:
-                    statement = next(dump)
-                except StopIteration:
-                    break
-                except sqlite3.OperationalError as e:
-                    prints('Failed to dump a line:', as_unicode(e))
-                if last_restore_error:
-                    prints('Failed to restore a line:', last_restore_error)
-                    last_restore_error = None
-                try:
-                    dest.execute(statement)
-                except sqlite3.OperationalError as e:
-                    last_restore_error = as_unicode(e)
-                    # The dump produces an extra commit at the end, so
-                    # only print this error if there are more
-                    # statements to be restored
-            dest.execute('PRAGMA user_version=%d;'%uv)
-            dest.commit()
-        atomic_rename(tmpdb, dbpath)
-    prints('Database successfully re-initialized')
-
-
 def debug_device_driver():
     from calibre.devices import debug
     debug(ioreg_to_tmp=True, buf=sys.stdout)
@@ -160,7 +139,9 @@ def debug_device_driver():
 
 
 def add_simple_plugin(path_to_plugin):
-    import tempfile, zipfile, shutil
+    import shutil
+    import tempfile
+    import zipfile
     tdir = tempfile.mkdtemp()
     open(os.path.join(tdir, 'custom_plugin.py'),
             'wb').write(open(path_to_plugin, 'rb').read())
@@ -180,18 +161,12 @@ def print_basic_debug_info(out=None):
         out = sys.stdout
     out = functools.partial(prints, file=out)
     import platform
-    from contextlib import suppress
-    from calibre.constants import (__appname__, get_version, isportable, ismacos,
-                                   isfrozen, is64bit)
+
+    from calibre.constants import __appname__, get_version, isfrozen, ismacos, isportable
     from calibre.utils.localization import set_translators
     out(__appname__, get_version(), 'Portable' if isportable else '',
-        'embedded-python:', isfrozen, 'is64bit:', is64bit)
+        'embedded-python:', isfrozen)
     out(platform.platform(), platform.system(), platform.architecture())
-    if iswindows and not is64bit:
-        from calibre_extensions.winutil import is_wow64_process
-        with suppress(Exception):
-            if is_wow64_process():
-                out('32bit process running on 64bit windows')
     out(platform.system_alias(platform.system(), platform.release(),
             platform.version()))
     out('Python', platform.python_version())
@@ -205,6 +180,7 @@ def print_basic_debug_info(out=None):
     except:
         pass
     out('Interface language:', str(set_translators.lang))
+    out('EXE path:', sys.executable)
     from calibre.customize.ui import has_external_plugins, initialized_plugins
     if has_external_plugins():
         from calibre.customize import PluginInstallationType
@@ -258,7 +234,8 @@ def main(args=sys.argv):
         sys.argv = [sys.argv[0], '--multiprocessing-fork']
         exec(args[-1])
         return
-    debug()
+    if not opts.run_without_debug:
+        debug()
     if opts.gui:
         from calibre.gui_launch import calibre
         calibre(['calibre'] + args[1:])
@@ -278,8 +255,6 @@ def main(args=sys.argv):
         prints('CALIBRE_RESOURCES_PATH='+sys.resources_location)
         prints('CALIBRE_EXTENSIONS_PATH='+sys.extensions_location)
         prints('CALIBRE_PYTHON_PATH='+os.pathsep.join(sys.path))
-    elif opts.reinitialize_db is not None:
-        reinit_db(opts.reinitialize_db)
     elif opts.inspect_mobi:
         for path in args[1:]:
             inspect_mobi(path)
@@ -305,7 +280,10 @@ def main(args=sys.argv):
         from calibre.utils.fonts.sfnt.subset import main
         main(['subset-font'] + args[1:])
     elif opts.exec_file:
-        run_script(opts.exec_file, args[1:])
+        if opts.exec_file == '--':
+            run_script(args[1], args[2:])
+        else:
+            run_script(opts.exec_file, args[1:])
     elif opts.run_plugin:
         from calibre.customize.ui import find_plugin
         plugin = find_plugin(opts.run_plugin)
@@ -313,6 +291,10 @@ def main(args=sys.argv):
             prints(_('No plugin named %s found')%opts.run_plugin)
             raise SystemExit(1)
         plugin.cli_main([plugin.name] + args[1:])
+    elif opts.run_test:
+        debug(False)
+        from calibre.utils.run_tests import run_test
+        run_test(opts.run_test)
     elif opts.diff:
         from calibre.gui2.tweak_book.diff.main import main
         main(['calibre-diff'] + args[1:])
@@ -328,10 +310,16 @@ def main(args=sys.argv):
     elif opts.export_all_calibre_data:
         args = args[1:]
         from calibre.utils.exim import run_exporter
-        run_exporter(args=args)
+        run_exporter(args=args, check_known_libraries=False)
     elif opts.import_calibre_data:
         from calibre.utils.exim import run_importer
         run_importer()
+    elif opts.kepubify:
+        from calibre.ebooks.oeb.polish.kepubify import kepubify_main
+        kepubify_main(args)
+    elif opts.un_kepubify:
+        from calibre.ebooks.oeb.polish.kepubify import unkepubify_main
+        unkepubify_main(args)
     elif len(args) >= 2 and args[1].rpartition('.')[-1] in {'py', 'recipe'}:
         run_script(args[1], args[2:])
     elif len(args) >= 2 and args[1].rpartition('.')[-1] in {'mobi', 'azw', 'azw3', 'docx', 'odt'}:
@@ -343,7 +331,7 @@ def main(args=sys.argv):
             elif ext in {'mobi', 'azw', 'azw3'}:
                 inspect_mobi(path)
             else:
-                print('Cannot dump unknown filetype: %s' % path)
+                print(f'Cannot dump unknown filetype: {path}')
     elif len(args) >= 2 and os.path.exists(os.path.join(args[1], '__main__.py')):
         sys.path.insert(0, args[1])
         run_script(os.path.join(args[1], '__main__.py'), args[2:])
