@@ -1,20 +1,38 @@
-#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.13
 
 __license__   = 'GPL v3'
 __copyright__ = '2015-2019, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
+import json
 import textwrap
 
-from qt.core import (QWidget, QLabel, QGridLayout, QLineEdit, QVBoxLayout, QDialog,
-                     QDialogButtonBox, QCheckBox, QPushButton)
+from qt.core import (
+    QCheckBox,
+    QDialog,
+    QDialogButtonBox,
+    QFormLayout,
+    QGridLayout,
+    QHBoxLayout,
+    QLabel,
+    QLineEdit,
+    QListWidget,
+    QListWidgetItem,
+    QPlainTextEdit,
+    QPushButton,
+    QSpinBox,
+    Qt,
+    QVBoxLayout,
+    QWidget,
+)
 
-from calibre.gui2.device_drivers.tabbed_device_config import TabbedDeviceConfig, DeviceConfigTab, DeviceOptionsGroupBox
-from calibre.devices.usbms.driver import debug_print
+from calibre.devices.interface import ModelMetadata
 from calibre.gui2 import error_dialog
-from calibre.gui2.widgets2 import ColorButton
+from calibre.gui2.device_drivers.tabbed_device_config import DeviceConfigTab, DeviceOptionsGroupBox, TabbedDeviceConfig
 from calibre.gui2.dialogs.template_dialog import TemplateDialog
 from calibre.gui2.dialogs.template_line_editor import TemplateLineEditor
+from calibre.gui2.widgets2 import ColorButton
+from calibre.prints import debug_print
 
 
 def wrap_msg(msg):
@@ -52,9 +70,13 @@ class KOBOTOUCHConfig(TabbedDeviceConfig):
 
         self.tab1 = Tab1Config(self, self.device)
         self.tab2 = Tab2Config(self, self.device)
+        self.tab3 = Tab3Config(self, self.device)
+        self.tab4 = Tab4Config(self, self.device)
 
-        self.addDeviceTab(self.tab1, _("Collections, covers && uploads"))
+        self.addDeviceTab(self.tab1, _('Collections, covers && uploads'))
         self.addDeviceTab(self.tab2, _('Metadata, on device && advanced'))
+        self.addDeviceTab(self.tab3, _('Hyphenation'))
+        self.addDeviceTab(self.tab4, _('Modify CSS'))
 
     def get_pref(self, key):
         return self.device.get_pref(key)
@@ -93,7 +115,7 @@ class KOBOTOUCHConfig(TabbedDeviceConfig):
         return self.tab2.metadata_options
 
     def commit(self):
-        debug_print("KOBOTOUCHConfig::commit: start")
+        debug_print('KOBOTOUCHConfig::commit: start')
         p = super().commit()
 
         p['manage_collections'] = self.manage_collections
@@ -119,6 +141,7 @@ class KOBOTOUCHConfig(TabbedDeviceConfig):
 
         p['update_device_metadata'] = self.update_device_metadata
         p['update_series'] = self.update_series
+        p['force_series_id'] = self.force_series_id
         p['update_core_metadata'] = self.update_core_metadata
         p['update_purchased_kepubs'] = self.update_purchased_kepubs
         p['subtitle_template'] = self.subtitle_template
@@ -130,12 +153,22 @@ class KOBOTOUCHConfig(TabbedDeviceConfig):
         p['bookstats_timetoread_lower_template'] = self.bookstats_timetoread_lower_template
 
         p['modify_css'] = self.modify_css
+        p['per_device_css'] = self.per_device_css
+        p['kepubify'] = self.kepubify
+        p['template_for_kepubify'] = self.template_for_kepubify
         p['override_kobo_replace_existing'] = self.override_kobo_replace_existing
 
         p['support_newer_firmware'] = self.support_newer_firmware
         p['debugging_title'] = self.debugging_title
         p['driver_version'] = '.'.join([str(i) for i in self.device.version])
 
+        p['affect_hyphenation'] = self.affect_hyphenation
+        p['disable_hyphenation'] = self.disable_hyphenation
+        p['hyphenation_min_chars'] = self.hyphenation_min_chars
+        p['hyphenation_min_chars_before'] = self.hyphenation_min_chars_before
+        p['hyphenation_min_chars_after'] = self.hyphenation_min_chars_after
+        p['hyphenation_limit_lines'] = self.hyphenation_limit_lines
+
         return p
 
 
@@ -191,27 +224,162 @@ class Tab2Config(DeviceConfigTab):  # {{{
 # }}}
 
 
+class Tab3Config(DeviceConfigTab):  # {{{
+
+    def __init__(self, parent, device):
+        super().__init__(parent)
+        self.l = l = QVBoxLayout(self)
+        self.hyphenation_options = h = HyphenationGroupBox(self, device)
+        self.addDeviceWidget(h)
+        l.addWidget(h)
+        l.addStretch()
+
+    def validate(self):
+        return self.hyphenation_options.validate()
+
+# }}}
+
+
+class Tab4Config(DeviceConfigTab):  # {{{
+
+    def __init__(self, parent, device):
+        super().__init__(parent)
+        self.l = l = QVBoxLayout(self)
+        self.modify_css_options = h = ModifyCSSGroupBox(self, device)
+        self.addDeviceWidget(h)
+        l.addWidget(h)
+        l.addStretch()
+
+    def validate(self):
+        return self.modify_css_options.validate()
+
+# }}}
+
+
+class ModifyCSSGroupBox(DeviceOptionsGroupBox):
+
+    def __init__(self, parent, device):
+        super().__init__(parent, device)
+        self.setTitle(_('Modify CSS of books sent to the device'))
+        self.setCheckable(True)
+        self.setChecked(device.get_pref('modify_css'))
+        self.l = l = QVBoxLayout(self)
+        self.la = la = QLabel(
+            _('This allows addition of user CSS rules and removal of some CSS. '
+            'When sending a book, the driver adds the contents of {0} to all stylesheets in the book. '
+            'This file is searched for in the root folder of the main memory of the device. '
+            'As well as this, if the file contains settings for "orphans" or "widows", '
+            'these are removed from all styles in the original stylesheet.').format(device.KOBO_EXTRA_CSSFILE),
+        )
+        la.setWordWrap(True)
+        l.addWidget(la)
+        self.la2 = la = QLabel(_(
+            'Additionally, model specific CSS can be specified below:'))
+        la.setWordWrap(True)
+        l.addWidget(la)
+
+        try:
+            pdcss = json.loads(device.get_pref('per_device_css') or '{}')
+        except Exception:
+            pdcss = {}
+        self.dev_list = QListWidget(self)
+        self.css_edit = QPlainTextEdit(self)
+        self.css_edit.setPlaceholderText(_('Enter the CSS to use for books on this model of device'))
+        self.css_edit.textChanged.connect(self.css_text_changed)
+        h = QHBoxLayout()
+        h.addWidget(self.dev_list), h.addWidget(self.css_edit, stretch=100)
+        l.addLayout(h)
+        for mm in [ModelMetadata('', _('All models'), -1, -1, -1, type(device))] + sorted(
+                device.model_metadata(), key=lambda x: x.model_name.lower()):
+            css = pdcss.get(f'pid={mm.product_id}', '')
+            i = QListWidgetItem(mm.model_name, self.dev_list)
+            i.setData(Qt.ItemDataRole.UserRole, (mm, css or ''))
+        self.dev_list.setCurrentRow(0)
+        self.dev_list.currentItemChanged.connect(self.current_device_changed)
+        self.current_device_changed()
+        self.clear_button = b = QPushButton(_('&Clear all model specific CSS'))
+        l.addWidget(b)
+        b.clicked.connect(self.clear_all_css)
+
+    def items(self):
+        for i in range(self.dev_list.count()):
+            yield self.dev_list.item(i)
+
+    def clear_all_css(self):
+        for item in self.items():
+            mm, css = item.data(Qt.ItemDataRole.UserRole)
+            item.setData(Qt.ItemDataRole.UserRole, (mm, ''))
+        self.current_device_changed()
+
+    def current_device_changed(self):
+        i = self.dev_list.currentItem()
+        css = ''
+        if i is not None:
+            mm, css = i.data(Qt.ItemDataRole.UserRole)
+        self.css_edit.setPlainText(css or '')
+
+    def css_text_changed(self):
+        i = self.dev_list.currentItem()
+        if i is not None:
+            mm, css = i.data(Qt.ItemDataRole.UserRole)
+            css = self.css_edit.toPlainText().strip()
+            i.setData(Qt.ItemDataRole.UserRole, (mm, css))
+
+    def validate(self):
+        return True
+
+    @property
+    def modify_css(self):
+        return self.isChecked()
+
+    @property
+    def per_device_css(self):
+        ans = {}
+        for item in self.items():
+            mm, css = item.data(Qt.ItemDataRole.UserRole)
+            if css:
+                ans[f'pid={mm.product_id}'] = css
+        return json.dumps(ans)
+
+
 class BookUploadsGroupBox(DeviceOptionsGroupBox):
 
     def __init__(self, parent, device):
         super().__init__(parent, device)
-        self.setTitle(_("Uploading of books"))
+        self.setTitle(_('Uploading of books'))
 
         self.options_layout = QGridLayout()
-        self.options_layout.setObjectName("options_layout")
+        self.options_layout.setObjectName('options_layout')
         self.setLayout(self.options_layout)
 
-        self.modify_css_checkbox = create_checkbox(
-                _("Modify CSS"),
-                _('This allows addition of user CSS rules and removal of some CSS. '
-                'When sending a book, the driver adds the contents of {0} to all stylesheets in the EPUB. '
-                'This file is searched for in the root folder of the main memory of the device. '
-                'As well as this, if the file contains settings for the "orphans" or "widows", '
-                'these are removed for all styles in the original stylesheet.').format(device.KOBO_EXTRA_CSSFILE),
-                device.get_pref('modify_css')
-                )
+        self.kepubify_checkbox = create_checkbox(
+            _('Use Kobo viewer for EPUB books'), _(
+                'Kobo devices have two viewer programs for EPUB files on their devices. An older one from Adobe and'
+                ' the Kobo one. The Kobo one has much better performance and features and so, by default,'
+                ' calibre will auto-convert EPUB books to the Kobo KEPUB format so that they are viewed by'
+                ' the Kobo viewer. If you would rather use the legacy viewer for EPUB, disable this option. Note'
+                ' that this option has no effect if the device does not support KEPUB, such as for Tolino devices'
+                ' that also use this driver.'
+            ), device.get_pref('kepubify'))
+
+        self.template_la = la = QLabel('\xa0\xa0' + _('Template to decide conversion:'))
+        self.kepubify_template_edit = TemplateConfig(
+            device.get_pref('template_for_kepubify'),
+            tooltip='<p>' + _(
+                'Enter a template to decide if an EPUB book is to be auto converted to KEPUB. '
+                'If the template returns false or no result, the book will not be '
+                'converted to KEPUB. For example to only kepubify books that have the tag <i>{0}</i>, use the template: <code>{1}</code>'
+                ' or to only convert books that do not have the tag <i>{2}</i>, use the template: <code>{3}</code>').format(
+                    'as_kepub', r'{tags:str_in_list(\,,as_kepub,true,false)}', 'as_epub', r'{tags:str_in_list(\,,as_epub,false,true)}'
+                ) + '<p>'+_(
+                'If no template is specified conversion to KEPUB is controlled only by the setting above to use the Kobo viewer. '
+                'Note that the setting above must be enabled for the template to be checked.'
+            )
+        )
+        la.setBuddy(self.kepubify_template_edit)
+
         self.override_kobo_replace_existing_checkbox = create_checkbox(
-                _("Do not treat replacements as new books"),
+                _('Do not treat replacements as new books'),
                 _('When a new book is side-loaded, the Kobo firmware imports details of the book into the internal database. '
                 'Even if the book is a replacement for an existing book, the Kobo will remove the book from the database and then treat it as a new book. '
                 'This means that the reading status, bookmarks and collections for the book will be lost. '
@@ -221,45 +389,131 @@ class BookUploadsGroupBox(DeviceOptionsGroupBox):
                 device.get_pref('override_kobo_replace_existing')
                 )
 
-        self.options_layout.addWidget(self.modify_css_checkbox, 0, 0, 1, 2)
-        self.options_layout.addWidget(self.override_kobo_replace_existing_checkbox, 1, 0, 1, 2)
+        self.options_layout.addWidget(self.kepubify_checkbox, 0, 0, 1, 2)
+        self.options_layout.addWidget(self.template_la, 1, 0, 1, 1)
+        self.options_layout.addWidget(self.kepubify_template_edit, 1, 1, 1, 1)
+        self.options_layout.addWidget(self.override_kobo_replace_existing_checkbox, 2, 0, 1, 2)
+        self.update_template_state()
+        self.kepubify_checkbox.toggled.connect(self.update_template_state)
 
-    @property
-    def modify_css(self):
-        return self.modify_css_checkbox.isChecked()
+    def update_template_state(self):
+        self.kepubify_template_edit.setEnabled(self.kepubify)
 
     @property
     def override_kobo_replace_existing(self):
         return self.override_kobo_replace_existing_checkbox.isChecked()
 
+    @property
+    def kepubify(self):
+        return self.kepubify_checkbox.isChecked()
+
+    @property
+    def template_for_kepubify(self):
+        return (self.kepubify_template_edit.template or '').strip()
+
+
+class HyphenationGroupBox(DeviceOptionsGroupBox):
+
+    def __init__(self, parent, device):
+        super().__init__(parent, device)
+        self.setTitle(_('Enable/disable hyphenation in KEPUB books'))
+        self.setCheckable(True)
+        self.setChecked(device.get_pref('affect_hyphenation'))
+        self.l = l = QFormLayout(self)
+        la = QLabel(_(
+            'When sending EPUB as converted KEPUB to the device, you can optionally'
+            ' modify how the device will perform hyphenation for the book. Note that hyphenation'
+            ' does not work well for all languages, as it depends on dictionaries present on the device,'
+            ' which are not always of the highest quality.'))
+        la.setWordWrap(True)
+        l.addRow(la)
+
+        self.disable_hyphenation_checkbox = d = QCheckBox(_('Turn off all hyphenation'))
+        d.setChecked(device.get_pref('disable_hyphenation'))
+        d.setToolTip(_('Override all hyphenation settings in book, forcefully disabling hyphenation completely'))
+        l.addRow(d)
+
+        self.min_chars = mc = QSpinBox(self)
+        l.addRow(_('Minimum word length to hyphenate') + ':', mc)
+        mc.setSuffix(_(' characters'))
+        mc.setSpecialValueText(_('Disabled'))
+        mc.setRange(0, 20)
+        mc.setValue(device.get_pref('hyphenation_min_chars'))
+
+        self.min_chars_before = mc = QSpinBox(self)
+        l.addRow(_('Minimum characters before hyphens') + ':', mc)
+        mc.setSuffix(_(' characters'))
+        mc.setRange(2, 20)
+        mc.setValue(device.get_pref('hyphenation_min_chars_before'))
+
+        self.min_chars_after = mc = QSpinBox(self)
+        l.addRow(_('Minimum characters after hyphens') + ':', mc)
+        mc.setSuffix(_(' characters'))
+        mc.setRange(2, 20)
+        mc.setValue(device.get_pref('hyphenation_min_chars_after'))
+
+        self.limit_lines = mc = QSpinBox(self)
+        l.addRow(_('Maximum consecutive hyphenated lines') + ':', mc)
+        mc.setSuffix(_(' lines'))
+        mc.setSpecialValueText(_('Disabled'))
+        mc.setRange(0, 20)
+        mc.setValue(device.get_pref('hyphenation_limit_lines'))
+
+    def validate(self):
+        return True
+
+    @property
+    def affect_hyphenation(self):
+        return self.isChecked()
+
+    @property
+    def disable_hyphenation(self):
+        return self.disable_hyphenation_checkbox.isChecked()
+
+    @property
+    def hyphenation_min_chars(self):
+        return self.min_chars.value()
+
+    @property
+    def hyphenation_min_chars_before(self):
+        return self.min_chars_before.value()
+
+    @property
+    def hyphenation_min_chars_after(self):
+        return self.min_chars_after.value()
+
+    @property
+    def hyphenation_limit_lines(self):
+        return self.limit_lines.value()
+
 
 class CollectionsGroupBox(DeviceOptionsGroupBox):
 
     def __init__(self, parent, device):
         super().__init__(parent, device)
-        self.setTitle(_("Collections"))
+        self.setTitle(_('Collections'))
 
         self.options_layout = QGridLayout()
-        self.options_layout.setObjectName("options_layout")
+        self.options_layout.setObjectName('options_layout')
         self.setLayout(self.options_layout)
 
         self.setCheckable(True)
         self.setChecked(device.get_pref('manage_collections'))
-        self.setToolTip(wrap_msg(_('Create new bookshelves on the Kobo if they do not exist. This is only for firmware V2.0.0 or later.')))
+        self.setToolTip(wrap_msg(_('Create new collections on the Kobo if they do not exist. This is only for firmware V2.0.0 or later.')))
 
         self.use_collections_columns_checkbox = create_checkbox(
-                             _("Collections columns:"),
+                             _('Collections columns:'),
                              _('Use a column to generate collections.'),
                              device.get_pref('use_collections_columns')
                              )
         self.collections_columns_edit = QLineEdit(self)
-        self.collections_columns_edit.setToolTip(_('The Kobo from firmware V2.0.0 supports bookshelves.'
+        self.collections_columns_edit.setToolTip(_('The Kobo from firmware V2.0.0 supports collections.'
                 ' These are created on the Kobo. '
                 'Specify a tags type column for automatic management.'))
         self.collections_columns_edit.setText(device.get_pref('collections_columns'))
 
         self.use_collections_template_checkbox = create_checkbox(
-                             _("Collections template:"),
+                             _('Collections template:'),
                              _('Use a template to generate collections.'),
                              device.get_pref('use_collections_template')
                              )
@@ -272,13 +526,13 @@ class CollectionsGroupBox(DeviceOptionsGroupBox):
                             )
 
         self.create_collections_checkbox = create_checkbox(
-                         _("Create collections"),
-                         _('Create new bookshelves on the Kobo if they do not exist. This is only for firmware V2.0.0 or later.'),
+                         _('Create collections'),
+                         _('Create new collections on the Kobo if they do not exist. This is only for firmware V2.0.0 or later.'),
                          device.get_pref('create_collections')
                          )
         self.delete_empty_collections_checkbox = create_checkbox(
-                         _('Delete empty bookshelves'),
-                         _('Delete any empty bookshelves from the Kobo when syncing is finished. This is only for firmware V2.0.0 or later.'),
+                         _('Delete empty collections'),
+                         _('Delete any empty collections from the Kobo when syncing is finished. This is only for firmware V2.0.0 or later.'),
                          device.get_pref('delete_empty_collections')
                          )
 
@@ -287,6 +541,8 @@ class CollectionsGroupBox(DeviceOptionsGroupBox):
         self.ignore_collections_names_edit.setToolTip(_('List the names of collections to be ignored by '
                 'the collection management. The collections listed '
                 'will not be changed. Names are separated by commas.'))
+        self.ignore_collections_names_label.setToolTip(self.ignore_collections_names_edit.toolTip())
+        self.ignore_collections_names_label.setBuddy(self.ignore_collections_names_edit)
         self.ignore_collections_names_edit.setText(device.get_pref('ignore_collections_names'))
 
         self.options_layout.addWidget(self.use_collections_columns_checkbox,  1, 0, 1, 1)
@@ -346,10 +602,10 @@ class CoversGroupBox(DeviceOptionsGroupBox):
 
     def __init__(self, parent, device):
         super().__init__(parent, device)
-        self.setTitle(_("Upload covers"))
+        self.setTitle(_('Upload covers'))
 
         self.options_layout = QGridLayout()
-        self.options_layout.setObjectName("options_layout")
+        self.options_layout.setObjectName('options_layout')
         self.setLayout(self.options_layout)
 
         self.setCheckable(True)
@@ -465,14 +721,14 @@ class DeviceListGroupBox(DeviceOptionsGroupBox):
 
     def __init__(self, parent, device):
         super().__init__(parent, device)
-        self.setTitle(_("Show as on device"))
+        self.setTitle(_('Show as on device'))
 
         self.options_layout = QGridLayout()
-        self.options_layout.setObjectName("options_layout")
+        self.options_layout.setObjectName('options_layout')
         self.setLayout(self.options_layout)
 
         self.show_recommendations_checkbox = create_checkbox(
-                             _("Show recommendations"),
+                             _('Show recommendations'),
                              _('Kobo shows recommendations on the device.  In some cases these have '
                                'files but in other cases they are just pointers to the web site to buy. '
                                'Enable if you wish to see/delete them.'),
@@ -480,7 +736,7 @@ class DeviceListGroupBox(DeviceOptionsGroupBox):
                              )
 
         self.show_archived_books_checkbox = create_checkbox(
-                             _("Show archived books"),
+                             _('Show archived books'),
                              _('Archived books are listed on the device but need to be downloaded to read.'
                                ' Use this option to show these books and match them with books in the calibre library.'),
                              device.get_pref('show_archived_books')
@@ -514,15 +770,15 @@ class DeviceListGroupBox(DeviceOptionsGroupBox):
 class AdvancedGroupBox(DeviceOptionsGroupBox):
 
     def __init__(self, parent, device):
-        super().__init__(parent, device, _("Advanced options"))
-#         self.setTitle(_("Advanced Options"))
+        super().__init__(parent, device, _('Advanced options'))
+        # self.setTitle(_("Advanced Options"))
 
         self.options_layout = QGridLayout()
-        self.options_layout.setObjectName("options_layout")
+        self.options_layout.setObjectName('options_layout')
         self.setLayout(self.options_layout)
 
         self.support_newer_firmware_checkbox = create_checkbox(
-                            _("Attempt to support newer firmware"),
+                            _('Attempt to support newer firmware'),
                             _('Kobo routinely updates the firmware and the '
                               'database version. With this option calibre will attempt '
                               'to perform full read-write functionality - Here be Dragons!! '
@@ -533,7 +789,7 @@ class AdvancedGroupBox(DeviceOptionsGroupBox):
                              )
 
         self.debugging_title_checkbox = create_checkbox(
-                             _("Title to test when debugging"),
+                             _('Title to test when debugging'),
                              _('Part of title of a book that can be used when doing some tests for debugging. '
                                'The test is to see if the string is contained in the title of a book. '
                                'The better the match, the less extraneous output.'),
@@ -564,10 +820,10 @@ class MetadataGroupBox(DeviceOptionsGroupBox):
 
     def __init__(self, parent, device):
         super().__init__(parent, device)
-        self.setTitle(_("Update metadata on the device"))
+        self.setTitle(_('Update metadata on the device'))
 
         self.options_layout = QGridLayout()
-        self.options_layout.setObjectName("options_layout")
+        self.options_layout.setObjectName('options_layout')
         self.setLayout(self.options_layout)
 
         self.setCheckable(True)
@@ -576,17 +832,28 @@ class MetadataGroupBox(DeviceOptionsGroupBox):
                                'Be careful when doing this as it will take time and could make the initial connection take a long time.')))
 
         self.update_series_checkbox = create_checkbox(
-                             _("Set series information"),
+                             _('Set series information'),
                              _('The book lists on the Kobo devices can display series information. '
                                'This is not read by the device from the sideloaded books. '
-                               'Series information can only be added to the device after the book has been processed by the device. '
+                               'Series information can only be added to the device after the '
+                               'book has been processed by the device. '
                                'Enable if you wish to set series information.'),
                              device.get_pref('update_series')
                              )
+        self.force_series_id_checkbox = create_checkbox(
+                             _('Force series ID'),
+                             _('Kobo devices use a SeriesID to distinguish between different series. '
+                               'Purchased books have a SeriesID assigned by Kobo. Sideloaded books '
+                               'have a SeriesID assigned by calibre, which is usually different. '
+                               'This causes a series to be shown twice on the Kobo device. '
+                               'Enable if you wish to force all the SeriesID for books '
+                               'in a series to the same value.'),
+                             device.get_pref('force_series_id')
+                             )
         self.update_core_metadata_checkbox = create_checkbox(
-                             _("Update metadata on Book Details pages"),
+                             _('Update metadata on Book Details pages'),
                              _('This will update the metadata in the device database when the device is connected. '
-                               'The metadata updated is displayed on the device in the library and the book details page. '
+                               'The metadata updated is displayed on the device in the library and the Book details page. '
                                'This is the title, authors, comments/synopsis, series name and number, publisher and published Date, ISBN and language. '
                                'If a metadata plugboard exists for the device and book format, this will be used to set the metadata.'
                                ),
@@ -594,62 +861,67 @@ class MetadataGroupBox(DeviceOptionsGroupBox):
                              )
 
         self.update_purchased_kepubs_checkbox = create_checkbox(
-                             _("Update purchased books"),
+                             _('Update purchased books'),
                              _('Update books purchased from Kobo and downloaded to the device.'
                                ),
                              device.get_pref('update_purchased_kepubs')
                              )
         self.update_subtitle_checkbox = create_checkbox(
-                             _("Subtitle"),
+                             _('Subtitle'),
                              _('Update the subtitle on the device using a template.'),
                              device.get_pref('update_subtitle')
                              )
         self.subtitle_template_edit = TemplateConfig(
                             device.get_pref('subtitle_template'),
-                            tooltip=_("Enter a template to use to set the subtitle. "
-                                      "If the template is empty, the subtitle will be cleared."
+                            tooltip=_('Enter a template to use to set the subtitle. '
+                                      'If the template is empty, the subtitle will be cleared.'
                                       )
                             )
         self.update_bookstats_checkbox = create_checkbox(
-                             _("Book stats"),
+                             _('Book stats'),
                              _('Update the book stats '),
                              device.get_pref('update_bookstats')
                              )
         self.bookstats_wordcount_template_edit = TemplateConfig(
                             device.get_pref('bookstats_wordcount_template'),
-                            label=_("Words:"),
-                            tooltip=_("Enter a template to use to set the word count for the book. "
-                                      "If the template is empty, the word count will be cleared."
+                            label=_('Words:'),
+                            tooltip=_('Enter a template to use to set the word count for the book. '
+                                      'If the template is empty, the word count will be cleared.'
                                       )
                             )
         self.bookstats_pagecount_template_edit = TemplateConfig(
                             device.get_pref('bookstats_pagecount_template'),
-                            label=_("Pages:"),
-                            tooltip=_("Enter a template to use to set the page count for the book. "
-                                      "If the template is empty, the page count will be cleared."
+                            label=_('Pages:'),
+                            tooltip=_('Enter a template to use to set the page count for the book. '
+                                      'If the template is empty, the page count will be cleared.'
                                       )
                             )
 
         self.bookstats_timetoread_label = QLabel(_('Hours to read estimates:'))
         self.bookstats_timetoread_upper_template_edit = TemplateConfig(
                             device.get_pref('bookstats_timetoread_upper_template'),
-                            label=_("Upper:"),
-                            tooltip=_("Enter a template to use to set the upper estimate of the time to read for the book. "
-                                      "The estimate is in hours. "
-                                      "If the template is empty, the time will be cleared."
+                            label=_('Upper:'),
+                            tooltip=_('Enter a template to use to set the upper estimate of the time to read for the book. '
+                                      'The estimate is in hours. '
+                                      'If the template is empty, the time will be cleared.'
                                       )
                             )
         self.bookstats_timetoread_lower_template_edit = TemplateConfig(
                             device.get_pref('bookstats_timetoread_lower_template'),
-                            label=_("Lower:"),
-                            tooltip=_("Enter a template to use to set the lower estimate of the time to read for the book. "
-                                      "The estimate is in hours. "
-                                      "If the template is empty, the time will be cleared."
+                            label=_('Lower:'),
+                            tooltip=_('Enter a template to use to set the lower estimate of the time to read for the book. '
+                                      'The estimate is in hours. '
+                                      'If the template is empty, the time will be cleared.'
                                       )
                             )
 
         line = 0
-        self.options_layout.addWidget(self.update_series_checkbox,                   line, 0, 1, 4)
+        hbl = QHBoxLayout()
+        hbl.setContentsMargins(0, 0, 0, 0)
+        hbl.addWidget(self.update_series_checkbox)
+        hbl.addWidget(self.force_series_id_checkbox)
+        hbl.addStretch(1)
+        self.options_layout.addLayout(hbl,                                           line, 0, 1, 4)
         line += 1
         self.options_layout.addWidget(self.update_core_metadata_checkbox,            line, 0, 1, 4)
         line += 1
@@ -666,15 +938,23 @@ class MetadataGroupBox(DeviceOptionsGroupBox):
         line += 1
         self.options_layout.addWidget(self.update_purchased_kepubs_checkbox,         line, 0, 1, 4)
 
+        self.force_series_id_checkbox.setEnabled(self.update_series)
         self.update_core_metadata_checkbox.clicked.connect(self.update_core_metadata_checkbox_clicked)
+        self.update_series_checkbox.clicked.connect(self.update_series_checkbox_clicked)
         self.update_subtitle_checkbox.clicked.connect(self.update_subtitle_checkbox_clicked)
         self.update_bookstats_checkbox.clicked.connect(self.update_bookstats_checkbox_clicked)
         self.update_core_metadata_checkbox_clicked(device.get_pref('update_core_metadata'))
         self.update_subtitle_checkbox_clicked(device.get_pref('update_subtitle'))
         self.update_bookstats_checkbox_clicked(device.get_pref('update_bookstats'))
 
+    def update_series_checkbox_clicked(self, checked):
+        self.force_series_id_checkbox.setEnabled(checked)
+        if not checked:
+            self.force_series_id_checkbox.setChecked(False)
+
     def update_core_metadata_checkbox_clicked(self, checked):
         self.update_series_checkbox.setEnabled(not checked)
+        self.force_series_id_checkbox.setEnabled(self.update_series)
         self.subtitle_template_edit.setEnabled(checked)
         self.update_subtitle_checkbox.setEnabled(checked)
         self.update_bookstats_checkbox.setEnabled(checked)
@@ -715,6 +995,10 @@ class MetadataGroupBox(DeviceOptionsGroupBox):
     def update_series(self):
         return self.update_series_checkbox.isChecked()
 
+    @property
+    def force_series_id(self):
+        return self.update_series and self.force_series_id_checkbox.isChecked()
+
     @property
     def update_core_metadata(self):
         return self.update_core_metadata_checkbox.isChecked()
@@ -808,17 +1092,17 @@ class TemplateConfig(QWidget):  # {{{
 
 
 if __name__ == '__main__':
-    from calibre.gui2 import Application
     from calibre.devices.kobo.driver import KOBOTOUCH
     from calibre.devices.scanner import DeviceScanner
+    from calibre.gui2 import Application
     s = DeviceScanner()
     s.scan()
     app = Application([])
-    debug_print("KOBOTOUCH:", KOBOTOUCH)
+    debug_print('KOBOTOUCH:', KOBOTOUCH)
     dev = KOBOTOUCH(None)
-#     dev.startup()
-#     cd = dev.detect_managed_devices(s.devices)
-#     dev.open(cd, 'test')
+    # dev.startup()
+    # cd = dev.detect_managed_devices(s.devices)
+    # dev.open(cd, 'test')
     cw = dev.config_widget()
     d = QDialog()
     d.l = QVBoxLayout()
