@@ -1,42 +1,57 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import json, traceback, posixpath, importlib, os
+import datetime
+import importlib
+import json
+import os
+import posixpath
+import sys
+import traceback
+from collections.abc import Sequence
 from io import BytesIO
+from typing import NamedTuple
 
 from calibre import prints
 from calibre.constants import iswindows, numeric_version
 from calibre.devices.errors import PathError
 from calibre.devices.mtp.base import debug
 from calibre.devices.mtp.defaults import DeviceDefaults
-from calibre.ptempfile import SpooledTemporaryFile, PersistentTemporaryDirectory
+from calibre.devices.mtp.filesystem_cache import FileOrFolder, convert_timestamp
+from calibre.ptempfile import PersistentTemporaryDirectory, SpooledTemporaryFile
 from calibre.utils.filenames import shorten_components_to
-from polyglot.builtins import iteritems, itervalues, as_bytes
+from calibre.utils.icu import lower as icu_lower
+from polyglot.builtins import as_bytes, iteritems, itervalues
 
-BASE = importlib.import_module('calibre.devices.mtp.%s.driver'%(
-    'windows' if iswindows else 'unix')).MTP_DEVICE
+BASE = importlib.import_module('calibre.devices.mtp.{}.driver'.format('windows' if iswindows else 'unix')).MTP_DEVICE
+DEFAULT_THUMBNAIL_HEIGHT = 320
 
 
 class MTPInvalidSendPathError(PathError):
 
     def __init__(self, folder):
-        PathError.__init__(self, 'Trying to send to ignored folder: %s'%folder)
+        PathError.__init__(self, f'Trying to send to ignored folder: {folder}')
         self.folder = folder
 
 
+class ListEntry(NamedTuple):
+    name: str
+    is_folder: bool
+    size: int
+    mtime: datetime.datetime
+
+
 class MTP_DEVICE(BASE):
 
     METADATA_CACHE = 'metadata.calibre'
     DRIVEINFO = 'driveinfo.calibre'
-    CAN_SET_METADATA = []
     NEWS_IN_FOLDER = True
     MAX_PATH_LEN = 230
-    THUMBNAIL_HEIGHT = 160
-    THUMBNAIL_WIDTH = 120
+    THUMBNAIL_HEIGHT = DEFAULT_THUMBNAIL_HEIGHT
     CAN_SET_METADATA = []
     BACKLOADING_ERROR_MESSAGE = None
     MANAGES_DEVICE_PRESENCE = True
@@ -51,6 +66,7 @@ class MTP_DEVICE(BASE):
         self._prefs = None
         self.device_defaults = DeviceDefaults()
         self.current_device_defaults = {}
+        self.current_vid = self.current_pid = -1
         self.calibre_file_paths = {'metadata':self.METADATA_CACHE, 'driveinfo':self.DRIVEINFO}
         self.highlight_ignored_folders = False
 
@@ -63,7 +79,7 @@ class MTP_DEVICE(BASE):
             p.defaults['send_to'] = [
                 'Calibre_Companion', 'Books', 'eBooks/import', 'eBooks',
                 'wordplayer/calibretransfer', 'sdcard/ebooks',
-                'Android/data/com.amazon.kindle/files', 'kindle', 'NOOK'
+                'Android/data/com.amazon.kindle/files', 'kindle', 'NOOK', 'Documents',
             ]
             p.defaults['send_template'] = '{title} - {authors}'
             p.defaults['blacklist'] = []
@@ -73,11 +89,17 @@ class MTP_DEVICE(BASE):
 
         return self._prefs
 
+    @property
+    def is_kindle(self) -> bool:
+        return self.current_vid == 0x1949
+
     def is_folder_ignored(self, storage_or_storage_id, path,
                           ignored_folders=None):
         storage_id = str(getattr(storage_or_storage_id, 'object_id',
                              storage_or_storage_id))
         lpath = tuple(icu_lower(name) for name in path)
+        if self.is_kindle and lpath and lpath[-1].endswith('.sdr'):
+            return True
         if ignored_folders is None:
             ignored_folders = self.get_pref('ignored_folders')
         if storage_id in ignored_folders:
@@ -88,18 +110,28 @@ class MTP_DEVICE(BASE):
 
         # Top level ignores
         if lpath[0] in {
-            'alarms', 'dcim', 'movies', 'music', 'notifications',
-            'pictures', 'ringtones', 'samsung', 'sony', 'htc', 'bluetooth',
+            'alarms', 'dcim', 'movies', 'music', 'notifications', 'screenshots',
+            'pictures', 'ringtones', 'samsung', 'sony', 'htc', 'bluetooth', 'fonts',
             'games', 'lost.dir', 'video', 'whatsapp', 'image', 'com.zinio.mobile.android.reader'}:
             return True
+        if lpath[0].startswith('.') and lpath[0] not in ('.tolino', '.notebooks'):
+            # apparently the Tolino for some reason uses a hidden folder for its library, sigh.
+            # Kindle Scribe stores user notebooks in subfolders of '.notebooks'
+            return True
+        if lpath[0] == 'system' and not self.is_kindle:
+            # on Kindles we need the system/thumbnails folder for the amazon cover bug workaround
+            return True
 
-        if len(lpath) > 1 and lpath[0] == 'android':
-            # Ignore everything in Android apart from a few select folders
-            if lpath[1] != 'data':
-                return True
-            if len(lpath) > 2 and lpath[2] != 'com.amazon.kindle':
-                return True
-
+        if len(lpath) > 1:
+            if lpath[0] == 'android':
+                # Ignore everything in Android apart from a few select folders
+                if lpath[1] != 'data':
+                    return True
+                if len(lpath) > 2 and lpath[2] != 'com.amazon.kindle':
+                    return True
+            elif lpath[0] == 'system':
+                if lpath[1] != 'thumbnails':
+                    return True
         return False
 
     def configure_for_kindle_app(self):
@@ -131,9 +163,36 @@ class MTP_DEVICE(BASE):
                     isoformat(utcnow()))
             self.prefs['history'] = h
 
-        self.current_device_defaults = self.device_defaults(device, self)
+        self.current_device_defaults, self.current_vid, self.current_pid = self.device_defaults(device, self)
         self.calibre_file_paths = self.current_device_defaults.get(
             'calibre_file_paths', {'metadata':self.METADATA_CACHE, 'driveinfo':self.DRIVEINFO})
+        self.THUMBNAIL_HEIGHT = DEFAULT_THUMBNAIL_HEIGHT
+        if self.is_kindle:
+            self.THUMBNAIL_HEIGHT = 500  # see kindle/driver.py
+            try:
+                self.sync_kindle_thumbnails()
+            except Exception:
+                import traceback
+                traceback.print_exc()
+
+    def list(self, path, recurse=False):
+        if path.startswith('/'):
+            q = self._main_id
+            path = path[1:]
+        elif path.startswith('card:/'):
+            q = self._carda_id
+            path = path[6:]
+        for storage in self.filesystem_cache.entries:
+            if storage.storage_id == q:
+                if path:
+                    path = path.replace(os.sep, '/')
+                    parts = path.split('/')
+                    if parts:
+                        storage = storage.find_path(parts)
+                        if storage is None:
+                            return []
+                return list(storage.list(recurse))
+        return []
 
     def get_device_uid(self):
         return self.current_serial_num
@@ -153,9 +212,10 @@ class MTP_DEVICE(BASE):
 
     # Device information {{{
     def _update_drive_info(self, storage, location_code, name=None):
-        from calibre.utils.date import isoformat, now
-        from calibre.utils.config import from_json, to_json
         import uuid
+
+        from calibre.utils.config import from_json, to_json
+        from calibre.utils.date import isoformat, now
         f = storage.find_path(self.calibre_file_paths['driveinfo'].split('/'))
         dinfo = {}
         if f is not None:
@@ -174,7 +234,7 @@ class MTP_DEVICE(BASE):
             dinfo['device_name'] = name
         dinfo['location_code'] = location_code
         dinfo['last_library_uuid'] = getattr(self, 'current_library_uuid', None)
-        dinfo['calibre_version'] = '.'.join([str(i) for i in numeric_version])
+        dinfo['calibre_version'] = '.'.join(str(i) for i in numeric_version)
         dinfo['date_last_connected'] = isoformat(now())
         dinfo['mtp_prefix'] = storage.storage_prefix
         raw = as_bytes(json.dumps(dinfo, default=to_json))
@@ -213,8 +273,7 @@ class MTP_DEVICE(BASE):
         self.report_progress(0, msg)
 
     def books(self, oncard=None, end_session=True):
-        from calibre.devices.mtp.books import JSONCodec
-        from calibre.devices.mtp.books import BookList, Book
+        from calibre.devices.mtp.books import Book, BookList, JSONCodec
         self.report_progress(0, _('Listing files, this can take a while'))
         self.get_driveinfo()  # Ensure driveinfo is loaded
         sid = {'carda':self._carda_id, 'cardb':self._cardb_id}.get(oncard,
@@ -288,9 +347,24 @@ class MTP_DEVICE(BASE):
         return bl
 
     def read_file_metadata(self, mtp_file):
-        from calibre.ebooks.metadata.meta import get_metadata
         from calibre.customize.ui import quick_metadata
+        from calibre.ebooks.metadata.meta import get_metadata
         ext = mtp_file.name.rpartition('.')[-1].lower()
+        if self.is_kindle and ext == 'kfx' and mtp_file.name != 'metadata.kfx':
+            # locate the actual file containing KFX book metadata
+            stream = BytesIO()
+            try:
+                self.get_file_by_name(stream, mtp_file.parent, *(mtp_file.name[:-4] + '.sdr', 'assets', 'metadata.kfx'))
+            except Exception:
+                pass    # expect failure for sideloaded books and personal documents
+            else:
+                stream.name = 'metadata.kfx'
+                with quick_metadata:
+                    metadata = get_metadata(stream, stream_type=ext,
+                            force_read_metadata=True,
+                            pattern=self.build_template_regexp())
+                if metadata.title != 'metadata' and metadata.title != 'Unknown':
+                    return metadata
         stream = self.get_mtp_file(mtp_file)
         with quick_metadata:
             return get_metadata(stream, stream_type=ext,
@@ -329,6 +403,14 @@ class MTP_DEVICE(BASE):
         f = self.filesystem_cache.resolve_mtp_id_path(path)
         self.get_mtp_file(f, outfile)
 
+    def get_file_by_name(self, outfile, parent: FileOrFolder, *names: str) -> None:
+        ' Get the file parent/ + "/".join(names) and put it into outfile. Works with files not cached in FilesystemCache. '
+        self.get_mtp_file_by_name(parent, *names, stream=outfile)
+
+    def list_folder_by_name(self, parent: FileOrFolder, *names: str) ->tuple[ListEntry, ...]:
+        ' List the contents of the folder parent/ + "/".join(names). Works with folders not cached in FilesystemCache. '
+        return tuple(ListEntry(x['name'], x['is_folder'], x['size'], convert_timestamp(x['modified'])) for x in self.list_mtp_folder_by_name(parent, *names))
+
     def prepare_addable_books(self, paths):
         tdir = PersistentTemporaryDirectory('_prepare_mtp')
         ans = []
@@ -338,20 +420,39 @@ class MTP_DEVICE(BASE):
             except Exception as e:
                 ans.append((path, e, traceback.format_exc()))
                 continue
-            base = os.path.join(tdir, '%s'%f.object_id)
+            base = os.path.join(tdir, f'{f.object_id}')
             os.mkdir(base)
             name = f.name
             if iswindows:
                 plen = len(base)
                 name = ''.join(shorten_components_to(245-plen, [name]))
-            with lopen(os.path.join(base, name), 'wb') as out:
+            with open(os.path.join(base, name), 'wb') as out:
                 try:
                     self.get_mtp_file(f, out)
                 except Exception as e:
                     ans.append((path, e, traceback.format_exc()))
                 else:
                     ans.append(out.name)
+                    if self.is_kindle and path.endswith('.kfx'):
+                        # copy additional KFX files from the associated .sdr folder
+                        try:
+                            sdr_folder_name = f.name[:-4] + '.sdr'
+                            new_sdr_path = os.path.join(os.path.split(out.name)[0], sdr_folder_name)
+                            os.mkdir(new_sdr_path)
+                            self.scan_sdr_for_kfx_files(f.parent, (sdr_folder_name, 'assets'), new_sdr_path)
+                        except Exception:
+                            traceback.print_exc()
         return ans
+
+    def scan_sdr_for_kfx_files(self, parent, folders, new_sdr_path):
+        for ff in self.list_folder_by_name(parent, *folders):
+            if ff.is_folder:
+                self.scan_sdr_for_kfx_files(parent, folders + (ff.name,), new_sdr_path)
+            elif ff.name.endswith('.kfx') or ff.name == 'voucher':
+                name = ''.join(shorten_components_to(245 - len(new_sdr_path), [ff.name])) if iswindows else ff.name
+                with open(os.path.join(new_sdr_path, name), 'wb') as out:
+                    self.get_file_by_name(out, parent, *(folders + (ff.name,)))
+
     # }}}
 
     # Sending files to the device {{{
@@ -419,7 +520,7 @@ class MTP_DEVICE(BASE):
         self.report_progress(0, _('Transferring books to device...'))
         i, total = 0, len(files)
 
-        routing = {fmt:dest for fmt,dest in self.get_pref('rules')}
+        routing = dict(self.get_pref('rules'))
 
         for infile, fname, mi in zip(files, names, metadata):
             path = self.create_upload_path(prefix, mi, fname, routing)
@@ -435,10 +536,16 @@ class MTP_DEVICE(BASE):
                 close = False
             else:
                 sz = os.path.getsize(infile)
-                stream = lopen(infile, 'rb')
+                stream = open(infile, 'rb')
                 close = True
+            relpath = parent.mtp_relpath + (path[-1].lower(),)
             try:
                 mtp_file = self.put_file(parent, path[-1], stream, sz)
+                try:
+                    self.upload_cover(parent, relpath, storage, mi, stream)
+                except Exception:
+                    import traceback
+                    traceback.print_exc()
             finally:
                 if close:
                     stream.close()
@@ -450,6 +557,80 @@ class MTP_DEVICE(BASE):
         debug('upload_books() ended')
         return ans
 
+    def upload_cover(self, parent_folder: FileOrFolder, relpath_of_ebook_on_device: Sequence[str], storage: FileOrFolder, mi, ebook_file_as_stream):
+        if self.is_kindle:
+            self.upload_kindle_thumbnail(parent_folder, relpath_of_ebook_on_device, storage, mi, ebook_file_as_stream)
+
+    # Kindle cover thumbnail handling {{{
+
+    def upload_kindle_thumbnail(self, parent_folder: FileOrFolder, relpath_of_ebook_on_device: Sequence[str], storage: FileOrFolder, mi, ebook_file_as_stream):
+        coverdata = getattr(mi, 'thumbnail', None)
+        if not coverdata or not coverdata[2]:
+            return
+        from calibre.devices.kindle.driver import thumbnail_filename
+        tfname = thumbnail_filename(ebook_file_as_stream)
+        if not tfname:
+            return
+        thumbpath = 'system', 'thumbnails', tfname
+        cover_stream = BytesIO(coverdata[2])
+        sz = len(coverdata[2])
+        try:
+            parent = self.ensure_parent(storage, thumbpath)
+        except Exception as err:
+            print(f'Failed to upload cover thumbnail to system/thumbnails with error: {err}', file=sys.stderr)
+            return
+        self.put_file(parent, tfname, cover_stream, sz)
+        cover_stream.seek(0)
+        cache_path = 'amazon-cover-bug', tfname
+        parent = self.ensure_parent(storage, cache_path)
+        self.put_file(parent, tfname, cover_stream, sz)
+        # mapping from ebook relpath to thumbnail filename
+        from hashlib import sha1
+        index_name = sha1('/'.join(relpath_of_ebook_on_device).encode()).hexdigest()
+        data = tfname.encode()
+        self.put_file(parent, index_name, BytesIO(data), len(data))
+
+    def delete_kindle_cover_thumbnail_for(self, storage: FileOrFolder, mtp_relpath: Sequence[str]) -> None:
+        from hashlib import sha1
+        index_name = sha1('/'.join(mtp_relpath).encode()).hexdigest()
+        index = storage.find_path(('amazon-cover-bug', index_name))
+        if index is not None:
+            data = BytesIO()
+            self.get_mtp_file(index, data)
+            tfname = data.getvalue().decode().strip()
+            if tfname:
+                thumbnail = storage.find_path(('system', 'thumbnails', tfname))
+                if thumbnail is not None:
+                    self.delete_file_or_folder(thumbnail)
+                cache = storage.find_path(('amazon-cover-bug', tfname))
+                if cache is not None:
+                    self.delete_file_or_folder(cache)
+                self.delete_file_or_folder(index)
+
+    def sync_kindle_thumbnails(self):
+        for storage in self.filesystem_cache.entries:
+            self._sync_kindle_thumbnails(storage)
+
+    def _sync_kindle_thumbnails(self, storage):
+        system_thumbnails_dir = storage.find_path(('system', 'thumbnails'))
+        amazon_cover_bug_cache_dir = storage.find_path(('amazon-cover-bug',))
+        if system_thumbnails_dir is None or amazon_cover_bug_cache_dir is None:
+            return
+        debug('Syncing cover thumbnails to workaround amazon cover bug')
+        system_thumbnails = {x.name: x for x in system_thumbnails_dir.files}
+        count = 0
+        for f in amazon_cover_bug_cache_dir.files:
+            s = system_thumbnails.get(f.name)
+            if s is not None and s.size != f.size:
+                count += 1
+                data = BytesIO()
+                self.get_mtp_file(f, data)
+                data.seek(0)
+                sz = len(data.getvalue())
+                self.put_file(system_thumbnails_dir, f.name, data, sz)
+        debug(f'Restored {count} cover thumbnails that were destroyed by Amazon')
+    # }}}
+
     def add_books_to_metadata(self, mtp_files, metadata, booklists):
         debug('add_books_to_metadata() called')
         from calibre.devices.mtp.books import Book
@@ -481,15 +662,18 @@ class MTP_DEVICE(BASE):
             try:
                 self.recursive_delete(parent)
             except:
-                prints('Failed to delete parent: %s, ignoring'%(
-                    '/'.join(parent.full_path)))
+                prints('Failed to delete parent: {}, ignoring'.format('/'.join(parent.full_path)))
 
     def delete_books(self, paths, end_session=True):
         self.report_progress(0, _('Deleting books from device...'))
 
         for i, path in enumerate(paths):
             f = self.filesystem_cache.resolve_mtp_id_path(path)
+            fpath = f.mtp_relpath
+            storage = f.storage
             self.recursive_delete(f)
+            if self.is_kindle:
+                self.delete_kindle_cover_thumbnail_for(storage, fpath)
             self.report_progress((i+1) / float(len(paths)),
                     _('Deleted %s')%path)
         self.report_progress(1, _('All books deleted'))
@@ -522,7 +706,7 @@ class MTP_DEVICE(BASE):
         If that is not found looks for a device default and if that is not
         found uses the global default.'''
         dd = self.current_device_defaults if self.is_mtp_device_connected else {}
-        dev_settings = self.prefs.get('device-%s'%self.current_serial_num, {})
+        dev_settings = self.prefs.get(f'device-{self.current_serial_num}', {})
         default_value = dd.get(key, self.prefs[key])
         return dev_settings.get(key, default_value)
 
@@ -561,6 +745,8 @@ class MTP_DEVICE(BASE):
 
 def main():
     import io
+    from pprint import pprint
+    io
     dev = MTP_DEVICE(None)
     dev.startup()
     try:
@@ -574,17 +760,27 @@ def main():
         dev.set_progress_reporter(prints)
         dev.open(cd, None)
         dev.filesystem_cache.dump()
-        print('Prefix for main mem:', dev.prefix_for_location(None), flush=True)
-        raw = os.urandom(32 * 1024)
-        folder = dev.create_folder(dev.filesystem_cache.entries[0], 'developing-mtp-driver')
-        f = dev.put_file(folder, 'developing-mtp-driver.bin', io.BytesIO(raw), len(raw))
-        print('Put file:', f, flush=True)
-        buf = io.BytesIO()
-        dev.get_file(f.mtp_id_path, buf)
-        if buf.getvalue() != raw:
-            raise ValueError('Getting previously put file did not return expected data')
-        print('Successfully got previously put file', flush=True)
-        dev.recursive_delete(f)
+        print(dev.device_debug_info(), flush=True)
+        docs = dev.prefix_for_location(None)
+        print('Prefix for main mem:', docs, flush=True)
+        parent = dev.filesystem_cache.entries[0]
+        entries = dev.list_folder_by_name(parent, docs)
+        pprint(entries)
+        pprint(dev.get_mtp_metadata_by_name(parent, docs, entries[0].name))
+        files = [x for x in entries if not x.is_folder]
+        f = io.BytesIO()
+        dev.get_file_by_name(f, parent, docs, files[0].name)
+        data = f.getvalue()
+        print('Got', files[0].name, 'of size:', len(data))
+        parent = parent.folder_named(docs)
+        f.seek(0)
+        destname = 'mtp-driver-test.' + files[0].name.rpartition('.')[2]
+        m = dev.put_file(parent, destname, f, len(data))
+        print('Put', destname, 'in the', docs, 'folder')
+        pprint(m)
+    except Exception:
+        import traceback
+        traceback.print_exc()
     finally:
         dev.shutdown()
 
