@@ -1,16 +1,18 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import unittest, gc, io
+import gc
+import io
+import unittest
 
-from calibre.constants import iswindows, islinux
-from calibre.utils.icu import lower
+from calibre.constants import islinux, iswindows
 from calibre.devices.mtp.driver import MTP_DEVICE
 from calibre.devices.scanner import DeviceScanner
+from calibre.utils.icu import lower
 
 
 class ProgressCallback:
@@ -180,7 +182,7 @@ class TestDeviceInteraction(unittest.TestCase):
         return end_mem - start_mem
 
     def check_memory(self, once, many, msg, factor=2):
-        msg += ' for once: %g for many: %g'%(once, many)
+        msg += f' for once: {once:g} for many: {many:g}'
         if once > 0:
             self.assertTrue(many <= once*factor, msg=msg)
         else:
