@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
@@ -9,19 +9,21 @@ __docformat__ = 'restructuredtext en'
 Device driver for the SONY T1 devices
 '''
 
-import os, time, re
+import os
+import re
+import time
 from contextlib import closing
 from datetime import date
 
 from calibre import fsync
-from calibre.devices.mime import mime_type_ext
-from calibre.devices.errors import DeviceError
-from calibre.devices.usbms.driver import USBMS, debug_print
-from calibre.devices.usbms.device import USBDevice
-from calibre.devices.usbms.books import CollectionsBookList
-from calibre.devices.usbms.books import BookList
-from calibre.ebooks.metadata import authors_to_sort_string, authors_to_string
 from calibre.constants import islinux
+from calibre.devices.errors import DeviceError
+from calibre.devices.mime import mime_type_ext
+from calibre.devices.usbms.books import BookList, CollectionsBookList
+from calibre.devices.usbms.device import USBDevice
+from calibre.devices.usbms.driver import USBMS
+from calibre.ebooks.metadata import authors_to_sort_string, authors_to_string
+from calibre.prints import debug_print
 from polyglot.builtins import long_type
 
 DBPATH = 'Sony_Reader/database/books.db'
@@ -36,7 +38,7 @@ class ImageWrapper:
 
 class PRST1(USBMS):
     name           = 'SONY PRST1 and newer Device Interface'
-    gui_name       = 'SONY Reader'
+    gui_name       = 'SONY PRS-T1'
     description    = _('Communicate with the PRST1 and newer SONY e-book readers')
     author         = 'Kovid Goyal'
     supported_platforms = ['windows', 'osx', 'linux']
@@ -165,7 +167,7 @@ class PRST1(USBMS):
         bl = USBMS.books(self, oncard=oncard, end_session=end_session)
 
         dbpath = self.normalize_path(prefix + DBPATH)
-        debug_print("SQLite DB Path: " + dbpath)
+        debug_print('SQLite DB Path: ' + dbpath)
 
         with closing(sqlite.connect(dbpath)) as connection:
             # Replace undecodable characters in the db instead of erroring out
@@ -208,10 +210,10 @@ class PRST1(USBMS):
 
                 try:
                     device_offset = max(time_offsets, key=lambda a: time_offsets.get(a))
-                    debug_print("Device Offset: %d ms"%device_offset)
+                    debug_print(f'Device Offset: {device_offset} ms')
                     self.device_offset = device_offset
                 except ValueError:
-                    debug_print("No Books To Detect Device Offset.")
+                    debug_print('No Books To Detect Device Offset.')
 
             for idx, book in enumerate(bl):
                 query = 'SELECT _id, thumbnail FROM books WHERE file_path = ?'
@@ -261,7 +263,7 @@ class PRST1(USBMS):
         if self.plugboard_func:
             plugboard = self.plugboard_func(self.__class__.__name__,
                     'device_db', self.plugboards)
-            debug_print("PRST1: Using Plugboard", plugboard)
+            debug_print('PRST1: Using Plugboard', plugboard)
 
         prefix = self._card_a_prefix if oncard == 'carda' else self._main_prefix
         if prefix is None:
@@ -270,7 +272,7 @@ class PRST1(USBMS):
         source_id = 1 if oncard == 'carda' else 0
 
         dbpath = self.normalize_path(prefix + DBPATH)
-        debug_print("SQLite DB Path: " + dbpath)
+        debug_print('SQLite DB Path: ' + dbpath)
 
         collections = booklist.get_collections(collections_attributes)
 
@@ -288,7 +290,7 @@ class PRST1(USBMS):
         try:
             cursor = connection.cursor()
 
-            debug_print("Removing Orphaned Collection Records")
+            debug_print('Removing Orphaned Collection Records')
 
             # Purge any collections references that point into the abyss
             query = 'DELETE FROM collections WHERE content_id NOT IN (SELECT _id FROM books)'
@@ -296,7 +298,7 @@ class PRST1(USBMS):
             query = 'DELETE FROM collections WHERE collection_id NOT IN (SELECT _id FROM collection)'
             cursor.execute(query)
 
-            debug_print("Removing Orphaned Book Records")
+            debug_print('Removing Orphaned Book Records')
 
             # Purge any references to books not in this database
             # Idea is to prevent any spill-over where these wind up applying to some other book
@@ -314,11 +316,11 @@ class PRST1(USBMS):
             import traceback
             tb = traceback.format_exc()
             raise DeviceError((('The SONY database is corrupted. '
-                    ' Delete the file %s on your reader and then disconnect '
+                    f' Delete the file {dbpath} on your reader and then disconnect '
                     ' reconnect it. If you are using an SD card, you '
                     ' should delete the file on the card as well. Note that '
                     ' deleting this file will cause your reader to forget '
-                    ' any notes/highlights, etc.')%dbpath)+' Underlying error:'
+                    ' any notes/highlights, etc.'))+' Underlying error:'
                     '\n'+tb)
 
     def get_lastrowid(self, cursor):
@@ -360,7 +362,7 @@ class PRST1(USBMS):
         sequence_max = sequence_min
         sequence_dirty = 0
 
-        debug_print("Book Sequence Min: %d, Source Id: %d"%(sequence_min,source_id))
+        debug_print(f'Book Sequence Min: {sequence_min}, Source Id: {source_id}')
 
         try:
             cursor = connection.cursor()
@@ -372,11 +374,11 @@ class PRST1(USBMS):
             import traceback
             tb = traceback.format_exc()
             raise DeviceError((('The SONY database is corrupted. '
-                    ' Delete the file %s on your reader and then disconnect '
+                    f' Delete the file {dbpath} on your reader and then disconnect '
                     ' reconnect it. If you are using an SD card, you '
                     ' should delete the file on the card as well. Note that '
                     ' deleting this file will cause your reader to forget '
-                    ' any notes/highlights, etc.')%dbpath)+' Underlying error:'
+                    ' any notes/highlights, etc.'))+' Underlying error:'
                     '\n'+tb)
 
         # Get the books themselves, but keep track of any that are less than the minimum.
@@ -394,7 +396,7 @@ class PRST1(USBMS):
 
         # If the database is 'dirty', then we should fix up the Ids and the sequence number
         if sequence_dirty == 1:
-            debug_print("Book Sequence Dirty for Source Id: %d"%source_id)
+            debug_print(f'Book Sequence Dirty for Source Id: {source_id}')
             sequence_max = sequence_max + 1
             for book, bookId in db_books.items():
                 if bookId < sequence_min:
@@ -431,7 +433,7 @@ class PRST1(USBMS):
                     cursor.execute(query, t)
 
             self.set_database_sequence_id(connection, 'books', sequence_max)
-            debug_print("Book Sequence Max: %d, Source Id: %d"%(sequence_max,source_id))
+            debug_print(f'Book Sequence Max: {sequence_max}, Source Id: {source_id}')
 
         cursor.close()
         return db_books
@@ -493,7 +495,7 @@ class PRST1(USBMS):
                 book.bookId = self.get_lastrowid(cursor)
                 if upload_covers:
                     self.upload_book_cover(connection, book, source_id)
-                debug_print('Inserted New Book: (%u) '%book.bookId + book.title)
+                debug_print(f'Inserted New Book: ({book.bookId}) ' + book.title)
             else:
                 query = '''
                 UPDATE books
@@ -532,7 +534,7 @@ class PRST1(USBMS):
         sequence_max = sequence_min
         sequence_dirty = 0
 
-        debug_print("Collection Sequence Min: %d, Source Id: %d"%(sequence_min,source_id))
+        debug_print(f'Collection Sequence Min: {sequence_min}, Source Id: {source_id}')
 
         try:
             cursor = connection.cursor()
@@ -544,11 +546,11 @@ class PRST1(USBMS):
             import traceback
             tb = traceback.format_exc()
             raise DeviceError((('The SONY database is corrupted. '
-                    ' Delete the file %s on your reader and then disconnect '
+                    f' Delete the file {dbpath} on your reader and then disconnect '
                     ' reconnect it. If you are using an SD card, you '
                     ' should delete the file on the card as well. Note that '
                     ' deleting this file will cause your reader to forget '
-                    ' any notes/highlights, etc.')%dbpath)+' Underlying error:'
+                    ' any notes/highlights, etc.'))+' Underlying error:'
                     '\n'+tb)
 
         db_collections = {}
@@ -561,7 +563,7 @@ class PRST1(USBMS):
 
         # If the database is 'dirty', then we should fix up the Ids and the sequence number
         if sequence_dirty == 1:
-            debug_print("Collection Sequence Dirty for Source Id: %d"%source_id)
+            debug_print(f'Collection Sequence Dirty for Source Id: {source_id}')
             sequence_max = sequence_max + 1
             for collection, collectionId in db_collections.items():
                 if collectionId < sequence_min:
@@ -580,13 +582,13 @@ class PRST1(USBMS):
                     cursor.execute(query, t)
 
             self.set_database_sequence_id(connection, 'collection', sequence_max)
-            debug_print("Collection Sequence Max: %d, Source Id: %d"%(sequence_max,source_id))
+            debug_print(f'Collection Sequence Max: {sequence_max}, Source Id: {source_id}')
 
         # Fix up the collections table now...
         sequence_dirty = 0
         sequence_max = sequence_min
 
-        debug_print("Collections Sequence Min: %d, Source Id: %d"%(sequence_min,source_id))
+        debug_print(f'Collections Sequence Min: {sequence_min}, Source Id: {source_id}')
 
         query = 'SELECT _id FROM collections'
         cursor.execute(query)
@@ -600,7 +602,7 @@ class PRST1(USBMS):
                 sequence_max = max(sequence_max, row[0])
 
         if sequence_dirty == 1:
-            debug_print("Collections Sequence Dirty for Source Id: %d"%source_id)
+            debug_print(f'Collections Sequence Dirty for Source Id: {source_id}')
             sequence_max = sequence_max + 1
             for pairId in db_collection_pairs:
                 if pairId < sequence_min:
@@ -611,7 +613,7 @@ class PRST1(USBMS):
                     sequence_max = sequence_max + 1
 
             self.set_database_sequence_id(connection, 'collections', sequence_max)
-            debug_print("Collections Sequence Max: %d, Source Id: %d"%(sequence_max,source_id))
+            debug_print(f'Collections Sequence Max: {sequence_max}, Source Id: {source_id}')
 
         cursor.close()
         return db_collections
@@ -629,7 +631,7 @@ class PRST1(USBMS):
                     t = (collection, source_id)
                     cursor.execute(query, t)
                     db_collections[collection] = self.get_lastrowid(cursor)
-                    debug_print('Inserted New Collection: (%u) '%db_collections[collection] + collection)
+                    debug_print(f'Inserted New Collection: ({db_collections[collection]}) ' + collection)
 
                 # Get existing books in collection
                 query = '''
@@ -725,7 +727,7 @@ class PRST1(USBMS):
         metadata.lpath = filepath.partition(prefix)[2]
         metadata.lpath = metadata.lpath.replace('\\', '/')
         dbpath = self.normalize_path(prefix + DBPATH)
-        debug_print("SQLite DB Path: " + dbpath)
+        debug_print('SQLite DB Path: ' + dbpath)
 
         with closing(sqlite.connect(dbpath)) as connection:
             cursor = connection.cursor()
@@ -762,7 +764,7 @@ class PRST1(USBMS):
         if not os.path.exists(thumbnail_dir_path):
             os.makedirs(thumbnail_dir_path)
 
-        with lopen(thumbnail_file_path, 'wb') as f:
+        with open(thumbnail_file_path, 'wb') as f:
             f.write(book.thumbnail[-1])
             fsync(f)
 
