@@ -1,20 +1,18 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.13
 # License: GPLv3 Copyright: 2019, Kovid Goyal <kovid at kovidgoyal.net>
 
 
 import bs4
-from bs4 import (  # noqa
-    CData, Comment, Declaration, NavigableString, ProcessingInstruction,
-    SoupStrainer, Tag, __version__
-)
+from bs4 import CData, Comment, Declaration, NavigableString, ProcessingInstruction, SoupStrainer, Tag, __version__  # noqa: F401
 
 
 def parse_html(markup):
-    from calibre.ebooks.chardet import strip_encoding_declarations, xml_to_unicode, substitute_entites
+    from calibre import xml_replace_entities
+    from calibre.ebooks.chardet import strip_encoding_declarations, xml_to_unicode
     from calibre.utils.cleantext import clean_xml_chars
     if isinstance(markup, str):
         markup = strip_encoding_declarations(markup)
-        markup = substitute_entites(markup)
+        markup = xml_replace_entities(markup)
     else:
         markup = xml_to_unicode(markup, strip_encoding_pats=True, resolve_entities=True)[0]
     markup = clean_xml_chars(markup)
