@@ -6,7 +6,12 @@ Code for the conversion of ebook formats and the reading of metadata
 from various formats.
 '''
 
-import os, re, numbers, sys
+import numbers
+import os
+import re
+import sys
+from contextlib import suppress
+
 from calibre import prints
 from calibre.ebooks.chardet import xml_to_unicode
 
@@ -48,7 +53,7 @@ def return_raster_image(path):
 
 
 def extract_cover_from_embedded_svg(html, base, log):
-    from calibre.ebooks.oeb.base import XPath, SVG, XLINK
+    from calibre.ebooks.oeb.base import SVG, XLINK, XPath
     from calibre.utils.xml_parse import safe_xml_fromstring
     root = safe_xml_fromstring(html)
 
@@ -89,7 +94,7 @@ def extract_calibre_cover(raw, base, log):
                 return return_raster_image(img)
 
 
-def render_html_svg_workaround(path_to_html, log, width=590, height=750):
+def render_html_svg_workaround(path_to_html, log, width=1200, height=1600, root=''):
     from calibre.ebooks.oeb.base import SVG_NS
     with open(path_to_html, 'rb') as f:
         raw = f.read()
@@ -108,13 +113,13 @@ def render_html_svg_workaround(path_to_html, log, width=590, height=750):
             pass
 
     if data is None:
-        data = render_html_data(path_to_html, width, height)
+        data = render_html_data(path_to_html, width, height, root=root)
     return data
 
 
-def render_html_data(path_to_html, width, height):
+def render_html_data(path_to_html, width, height, root=''):
     from calibre.ptempfile import TemporaryDirectory
-    from calibre.utils.ipc.simple_worker import fork_job, WorkerError
+    from calibre.utils.ipc.simple_worker import WorkerError, fork_job
     result = {}
 
     def report_error(text=''):
@@ -127,7 +132,7 @@ def render_html_data(path_to_html, width, height):
 
     with TemporaryDirectory('-render-html') as tdir:
         try:
-            result = fork_job('calibre.ebooks.render_html', 'main', args=(path_to_html, tdir, 'jpeg'))
+            result = fork_job('calibre.ebooks.render_html', 'main', args=(path_to_html, tdir, 'jpeg', root))
         except WorkerError as e:
             report_error(e.orig_tb)
         else:
@@ -173,14 +178,15 @@ def unit_convert(value, base, font, dpi, body_font_size=12):
     ' Return value in pts'
     if isinstance(value, numbers.Number):
         return value
-    try:
+    with suppress(Exception):
         return float(value) * 72.0 / dpi
-    except:
-        pass
     result = value
     m = UNIT_RE.match(value)
     if m is not None and m.group(1):
-        value = float(m.group(1))
+        try:
+            value = float(m.group(1))
+        except ValueError:
+            value = 0
         unit = m.group(2)
         if unit == '%':
             result = (value / 100.0) * base
@@ -233,13 +239,13 @@ def generate_masthead(title, output_path=None, width=600, height=60):
 def escape_xpath_attr(value):
     if '"' in value:
         if "'" in value:
-            parts = re.split('("+)', value)
+            parts = re.split(r'("+)', value)
             ans = []
             for x in parts:
                 if x:
                     q = "'" if '"' in x else '"'
                     ans.append(q + x + q)
-            return 'concat(%s)' % ', '.join(ans)
+            return 'concat({})'.format(', '.join(ans))
         else:
-            return "'%s'" % value
-    return '"%s"' % value
+            return f"'{value}'"
+    return f'"{value}"'
