@@ -1,11 +1,15 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import re, codecs, sys
+import codecs
+import re
+import sys
+
+from calibre import xml_replace_entities
 
 _encoding_pats = (
     # XML declaration
@@ -15,6 +19,7 @@ _encoding_pats = (
     # HTML 4 Pragma directive
     r'''<meta\s+?[^<>]*?content\s*=\s*['"][^'"]*?charset=([-_a-z0-9]+)[^'"]*?['"][^<>]*>(?:\s*</meta>){0,1}''',
 )
+substitute_entities = substitute_entites = xml_replace_entities  # for plugins that might use this
 
 
 def compile_pats(binary):
@@ -36,7 +41,6 @@ class LazyEncodingPats:
 
 
 lazy_encoding_pats = LazyEncodingPats()
-ENTITY_PATTERN = re.compile(r'&(\S+?);')
 
 
 def strip_encoding_declarations(raw, limit=50*1024, preserve_newlines=False):
@@ -45,9 +49,11 @@ def strip_encoding_declarations(raw, limit=50*1024, preserve_newlines=False):
     is_binary = isinstance(raw, bytes)
     if preserve_newlines:
         if is_binary:
-            sub = lambda m: b'\n' * m.group().count(b'\n')
+            def sub(m):
+                return (b'\n' * m.group().count(b'\n'))
         else:
-            sub = lambda m: '\n' * m.group().count('\n')
+            def sub(m):
+                return ('\n' * m.group().count('\n'))
     else:
         sub = b'' if is_binary else ''
     for pat in lazy_encoding_pats(is_binary):
@@ -94,25 +100,16 @@ def find_declared_encoding(raw, limit=50*1024):
                 return ans
 
 
-def substitute_entites(raw):
-    from calibre import xml_entity_to_unicode
-    return ENTITY_PATTERN.sub(xml_entity_to_unicode, raw)
-
-
-_CHARSET_ALIASES = {"macintosh" : "mac-roman", "x-sjis" : "shift-jis"}
+_CHARSET_ALIASES = {'macintosh': 'mac-roman', 'x-sjis': 'shift-jis', 'mac-centraleurope': 'cp1250'}
 
 
 def detect(bytestring):
-    from cchardet import detect as implementation
-    ans = implementation(bytestring)
-    enc = ans.get('encoding')
-    if enc:
-        ans['encoding'] = enc.lower()
-    elif enc is None:
-        ans['encoding'] = ''
-    if ans.get('confidence') is None:
-        ans['confidence'] = 0
-    return ans
+    if isinstance(bytestring, str):
+        bytestring = bytestring.encode('utf-8', 'replace')
+    from calibre_extensions.uchardet import detect as implementation
+    enc = implementation(bytestring).lower()
+    enc = _CHARSET_ALIASES.get(enc, enc)
+    return {'encoding': enc, 'confidence': 1 if enc else 0}
 
 
 def force_encoding(raw, verbose, assume_utf8=False):
@@ -152,6 +149,11 @@ def detect_xml_encoding(raw, verbose=False, assume_utf8=False):
             encoding = encoding.decode('ascii', 'replace')
             break
     if encoding is None:
+        if assume_utf8:
+            try:
+                return raw.decode('utf-8'), 'utf-8'
+            except UnicodeDecodeError:
+                pass
         encoding = force_encoding(raw, verbose, assume_utf8=assume_utf8)
     if encoding.lower().strip() == 'macintosh':
         encoding = 'mac-roman'
@@ -187,6 +189,6 @@ def xml_to_unicode(raw, verbose=False, strip_encoding_pats=False,
     if strip_encoding_pats:
         raw = strip_encoding_declarations(raw)
     if resolve_entities:
-        raw = substitute_entites(raw)
+        raw = xml_replace_entities(raw)
 
     return raw, encoding
