@@ -5,14 +5,15 @@ __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import re, codecs
+import codecs
+import re
 
+from calibre import force_unicode
 from calibre.ebooks.BeautifulSoup import BeautifulSoup
 from calibre.ebooks.chardet import xml_to_unicode
-from calibre.ebooks.metadata import string_to_authors, MetaInformation
-from calibre.utils.logging import default_log
+from calibre.ebooks.metadata import MetaInformation, string_to_authors
 from calibre.ptempfile import TemporaryFile
-from calibre import force_unicode
+from calibre.utils.logging import default_log
 from polyglot.builtins import iterkeys
 
 
@@ -21,11 +22,11 @@ def _clean(s):
 
 
 def _detag(tag):
-    ans = ""
+    ans = ''
     if tag is None:
         return ans
     for elem in tag:
-        if hasattr(elem, "contents"):
+        if hasattr(elem, 'contents'):
             ans += _detag(elem)
         else:
             ans += _clean(elem)
@@ -41,7 +42,7 @@ def _metadata_from_table(soup, searchfor):
     # on the home page. cue some nasty special-case hacks...
     if re.match(r'^\s*'+searchfor+r'\s*$', td.decode_contents(), flags=re.I):
         meta = _detag(td.findNextSibling('td'))
-        return re.sub('^:', '', meta).strip()
+        return re.sub(r'^:', '', meta).strip()
     else:
         meta = _detag(td)
         return re.sub(r'^[^:]+:', '', meta).strip()
@@ -76,7 +77,7 @@ def _get_comments(soup):
     pages = (_metadata_from_span(soup, 'pages') or _metadata_from_table(soup, 'pages'))
     try:
         # date span can have copyright symbols in it...
-        date = date.replace('\u00a9', '').strip()
+        date = date.replace('©', '').strip()
         # and pages often comes as '(\d+ pages)'
         pages = re.search(r'\d+', pages).group(0)
         return f'Published {date}, {pages} pages.'
@@ -88,7 +89,7 @@ def _get_comments(soup):
 def _get_cover(soup, rdr):
     ans = None
     try:
-        ans = soup.find('img', alt=re.compile('cover', flags=re.I))['src']
+        ans = soup.find('img', alt=re.compile(r'cover', flags=re.I))['src']
     except TypeError:
         # meeehh, no handy alt-tag goodness, try some hackery
         # the basic idea behind this is that in general, the cover image
@@ -118,14 +119,15 @@ def _get_cover(soup, rdr):
         try:
             ans = rdr.GetFile(ans)
         except:
-            ans = rdr.root + "/" + ans
+            ans = rdr.root + '/' + ans
             try:
                 ans = rdr.GetFile(ans)
             except:
                 ans = None
         if ans is not None:
-            from PIL import Image
             import io
+
+            from PIL import Image
             buf = io.BytesIO()
             try:
                 Image.open(io.BytesIO(ans)).convert('RGB').save(buf, 'JPEG')
