@@ -1,12 +1,15 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, sys
+import os
+import sys
+
 from calibre.customize.conversion import InputFormatPlugin
+from calibre.utils.resources import get_path as P
 
 
 class LRFInput(InputFormatPlugin):
@@ -19,13 +22,13 @@ class LRFInput(InputFormatPlugin):
 
     def convert(self, stream, options, file_ext, log,
                 accelerators):
-        from calibre.ebooks.lrf.input import (MediaType, Styles, TextBlock,
-                Canvas, ImageBlock, RuledLine)
+        from calibre.ebooks.lrf.input import Canvas, ImageBlock, MediaType, RuledLine, Styles, TextBlock
         self.log = log
         self.log('Generating XML')
+        from lxml import etree
+
         from calibre.ebooks.lrf.lrfparser import LRFDocument
         from calibre.utils.xml_parse import safe_xml_fromstring
-        from lxml import etree
         d = LRFDocument(stream)
         d.parse()
         xml = d.to_xml(write_files=True)
@@ -36,19 +39,18 @@ class LRFInput(InputFormatPlugin):
         char_button_map = {}
         for x in doc.xpath('//CharButton[@refobj]'):
             ro = x.get('refobj')
-            jump_button = doc.xpath('//*[@objid="%s"]'%ro)
+            jump_button = doc.xpath(f'//*[@objid="{ro}"]')
             if jump_button:
                 jump_to = jump_button[0].xpath('descendant::JumpTo[@refpage and @refobj]')
                 if jump_to:
-                    char_button_map[ro] = '%s.xhtml#%s'%(jump_to[0].get('refpage'),
+                    char_button_map[ro] = '{}.xhtml#{}'.format(jump_to[0].get('refpage'),
                             jump_to[0].get('refobj'))
         plot_map = {}
         for x in doc.xpath('//Plot[@refobj]'):
             ro = x.get('refobj')
-            image = doc.xpath('//Image[@objid="%s" and @refstream]'%ro)
+            image = doc.xpath(f'//Image[@objid="{ro}" and @refstream]')
             if image:
-                imgstr = doc.xpath('//ImageStream[@objid="%s" and @file]'%
-                    image[0].get('refstream'))
+                imgstr = doc.xpath('//ImageStream[@objid="{}" and @file]'.format(image[0].get('refstream')))
                 if imgstr:
                     plot_map[ro] = imgstr[0].get('file')
 
