@@ -1,12 +1,11 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-from calibre.customize.conversion import (OutputFormatPlugin,
-        OptionRecommendation)
+from calibre.customize.conversion import OptionRecommendation, OutputFormatPlugin
 
 
 def remove_html_cover(oeb, log):
@@ -50,10 +49,10 @@ class MOBIOutput(OutputFormatPlugin):
         ),
         OptionRecommendation(name='no_inline_toc',
             recommended_value=False, level=OptionRecommendation.LOW,
-            help=_('Don\'t add Table of Contents to the book. Useful if '
+            help=_("Don't add Table of Contents to the book. Useful if "
                 'the book has its own table of contents.')),
         OptionRecommendation(name='toc_title', recommended_value=None,
-            help=_('Title for any generated in-line table of contents.')
+            help=_('Title for any generated inline table of contents.')
         ),
         OptionRecommendation(name='dont_compress',
             recommended_value=False, level=OptionRecommendation.LOW,
@@ -189,6 +188,7 @@ class MOBIOutput(OutputFormatPlugin):
             mobi_type = 'old'  # Amazon does not support KF8 periodicals
         create_kf8 = mobi_type in ('new', 'both')
 
+        self.oeb.set_page_progression_direction_if_needed()
         remove_html_cover(self.oeb, self.log)
         resources = Resources(oeb, opts, self.is_periodical,
                 add_fonts=create_kf8)
@@ -217,11 +217,11 @@ class MOBIOutput(OutputFormatPlugin):
                 for_joint=for_joint)
 
     def write_mobi(self, input_plugin, output_path, kf8, resources):
+        from calibre.customize.ui import plugin_for_input_format
         from calibre.ebooks.mobi.mobiml import MobiMLizer
+        from calibre.ebooks.oeb.transforms.htmltoc import HTMLTOCAdder
         from calibre.ebooks.oeb.transforms.manglecase import CaseMangler
         from calibre.ebooks.oeb.transforms.rasterize import SVGRasterizer, Unavailable
-        from calibre.ebooks.oeb.transforms.htmltoc import HTMLTOCAdder
-        from calibre.customize.ui import plugin_for_input_format
 
         opts, oeb = self.opts, self.oeb
         if not opts.no_inline_toc:
@@ -280,10 +280,10 @@ class AZW3Output(OutputFormatPlugin):
         ),
         OptionRecommendation(name='no_inline_toc',
             recommended_value=False, level=OptionRecommendation.LOW,
-            help=_('Don\'t add Table of Contents to the book. Useful if '
+            help=_("Don't add Table of Contents to the book. Useful if "
                 'the book has its own table of contents.')),
         OptionRecommendation(name='toc_title', recommended_value=None,
-            help=_('Title for any generated in-line table of contents.')
+            help=_('Title for any generated inline table of contents.')
         ),
         OptionRecommendation(name='dont_compress',
             recommended_value=False, level=OptionRecommendation.LOW,
@@ -308,12 +308,13 @@ class AZW3Output(OutputFormatPlugin):
 
     def convert(self, oeb, output_path, input_plugin, opts, log):
         from calibre.ebooks.mobi.writer2.resources import Resources
-        from calibre.ebooks.mobi.writer8.main import create_kf8_book
         from calibre.ebooks.mobi.writer8.cleanup import remove_duplicate_anchors
+        from calibre.ebooks.mobi.writer8.main import create_kf8_book
 
         self.oeb, self.opts, self.log = oeb, opts, log
         opts.mobi_periodical = self.is_periodical
         passthrough = getattr(opts, 'mobi_passthrough', False)
+        self.oeb.set_page_progression_direction_if_needed()
         remove_duplicate_anchors(oeb)
 
         resources = Resources(self.oeb, self.opts, self.is_periodical,
