@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
@@ -7,9 +7,9 @@ __docformat__ = 'restructuredtext en'
 
 import os
 
-from calibre.customize.conversion import InputFormatPlugin, OptionRecommendation
-from calibre.constants import numeric_version
 from calibre import walk
+from calibre.constants import numeric_version
+from calibre.customize.conversion import InputFormatPlugin, OptionRecommendation
 
 
 class RecipeDisabled(Exception):
@@ -47,6 +47,8 @@ class RecipeInput(InputFormatPlugin):
         OptionRecommendation(name='password', recommended_value=None,
             help=_('Password for sites that require a login to access '
                 'content.')),
+        OptionRecommendation(name='recipe_specific_option',
+            help=_('Recipe specific options.')),
         OptionRecommendation(name='dont_download_recipe',
             recommended_value=False,
             help=_('Do not download latest version of builtin recipes from the calibre server')),
@@ -56,21 +58,23 @@ class RecipeInput(InputFormatPlugin):
 
     def convert(self, recipe_or_file, opts, file_ext, log,
             accelerators):
+        listing_recipe_specific_options = 'list' in (opts.recipe_specific_option or ())
         from calibre.web.feeds.recipes import compile_recipe
         opts.output_profile.flow_size = 0
+        orig_no_inline_navbars = opts.no_inline_navbars
         if file_ext == 'downloaded_recipe':
             from calibre.utils.zipfile import ZipFile
             zf = ZipFile(recipe_or_file, 'r')
             zf.extractall()
             zf.close()
-            with lopen('download.recipe', 'rb') as f:
+            with open('download.recipe', 'rb') as f:
                 self.recipe_source = f.read()
             recipe = compile_recipe(self.recipe_source)
             recipe.needs_subscription = False
             self.recipe_object = recipe(opts, log, self.report_progress)
         else:
             if os.environ.get('CALIBRE_RECIPE_URN'):
-                from calibre.web.feeds.recipes.collection import get_custom_recipe, get_builtin_recipe_by_id
+                from calibre.web.feeds.recipes.collection import get_builtin_recipe_by_id, get_custom_recipe
                 urn = os.environ['CALIBRE_RECIPE_URN']
                 log('Downloading recipe urn: ' + urn)
                 rtype, recipe_id = urn.partition(':')[::2]
@@ -79,20 +83,19 @@ class RecipeInput(InputFormatPlugin):
                 if rtype == 'custom':
                     self.recipe_source = get_custom_recipe(recipe_id)
                 else:
-                    self.recipe_source = get_builtin_recipe_by_id(urn, log=log, download_recipe=True)
+                    self.recipe_source = get_builtin_recipe_by_id(urn, log=log, download_recipe=not listing_recipe_specific_options)
                 if not self.recipe_source:
                     raise ValueError('Could not find recipe with urn: ' + urn)
                 if not isinstance(self.recipe_source, bytes):
                     self.recipe_source = self.recipe_source.encode('utf-8')
                 recipe = compile_recipe(self.recipe_source)
             elif os.access(recipe_or_file, os.R_OK):
-                with lopen(recipe_or_file, 'rb') as f:
+                with open(recipe_or_file, 'rb') as f:
                     self.recipe_source = f.read()
                 recipe = compile_recipe(self.recipe_source)
                 log('Using custom recipe')
             else:
-                from calibre.web.feeds.recipes.collection import (
-                        get_builtin_recipe_by_title, get_builtin_recipe_titles)
+                from calibre.web.feeds.recipes.collection import get_builtin_recipe_by_title, get_builtin_recipe_titles
                 title = getattr(opts, 'original_recipe_input_arg', recipe_or_file)
                 title = os.path.basename(title).rpartition('.')[0]
                 titles = frozenset(get_builtin_recipe_titles())
@@ -101,15 +104,14 @@ class RecipeInput(InputFormatPlugin):
                     title = title.rpartition('.')[0]
 
                 raw = get_builtin_recipe_by_title(title, log=log,
-                        download_recipe=not opts.dont_download_recipe)
+                        download_recipe=not opts.dont_download_recipe and not listing_recipe_specific_options)
                 builtin = False
                 try:
                     recipe = compile_recipe(raw)
                     self.recipe_source = raw
                     if recipe.requires_version > numeric_version:
                         log.warn(
-                        'Downloaded recipe needs calibre version at least: %s' %
-                        ('.'.join(recipe.requires_version)))
+                        'Downloaded recipe needs calibre version at least: {}'.format('.'.join(recipe.requires_version)))
                         builtin = True
                 except:
                     log.exception('Failed to compile downloaded recipe. Falling '
@@ -127,18 +129,36 @@ class RecipeInput(InputFormatPlugin):
                     log('Using downloaded builtin recipe')
 
             if recipe is None:
-                raise ValueError('%r is not a valid recipe file or builtin recipe' %
-                        recipe_or_file)
+                raise ValueError(f'{recipe_or_file!r} is not a valid recipe file or builtin recipe')
 
             disabled = getattr(recipe, 'recipe_disabled', None)
             if disabled is not None:
                 raise RecipeDisabled(disabled)
-            ro = recipe(opts, log, self.report_progress)
-            ro.download()
+            if listing_recipe_specific_options:
+                rso = (getattr(recipe, 'recipe_specific_options', None) or {})
+                if rso:
+                    log(recipe.title, _('specific options:'))
+                    name_maxlen = max(map(len, rso))
+                    for name, meta in rso.items():
+                        log(' ', name.ljust(name_maxlen), '-', meta.get('short'))
+                        if 'long' in meta:
+                            from textwrap import wrap
+                            for line in wrap(meta['long'], 70 - name_maxlen + 5):
+                                log(' '*(name_maxlen + 4), line)
+                else:
+                    log(recipe.title, _('has no recipe specific options'))
+                raise SystemExit(0)
+            try:
+                ro = recipe(opts, log, self.report_progress)
+                ro.download()
+            finally:
+                from calibre.scraper.simple import cleanup_overseers
+                cleanup_overseers()
             self.recipe_object = ro
 
         for key, val in self.recipe_object.conversion_options.items():
             setattr(opts, key, val)
+        opts.no_inline_navbars = orig_no_inline_navbars
 
         for f in os.listdir('.'):
             if f.endswith('.opf'):
