@@ -1,21 +1,23 @@
-#!/usr/local/bin/python3.9
+#!/usr/local/bin/python3.13
 
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import sys, os, shutil
+import os
+import shutil
+import sys
 
 from lxml import etree
 
 from calibre import walk
-from calibre.utils.zipfile import ZipFile
 from calibre.utils.xml_parse import safe_xml_fromstring
+from calibre.utils.zipfile import ZipFile
 
 
 def pretty_all_xml_in_dir(path):
     for f in walk(path):
-        if f.endswith('.xml') or f.endswith('.rels'):
+        if f.endswith(('.xml', '.rels')):
             with open(f, 'r+b') as stream:
                 raw = stream.read()
                 if raw:
