@@ -5,7 +5,6 @@ __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
 from collections import OrderedDict
-from polyglot.builtins import iteritems
 
 
 class Note:
@@ -49,13 +48,13 @@ class Footnotes:
         note = notes.get(fid, None)
         if note is not None and note.type == 'normal':
             self.counter += 1
-            anchor = 'note_%d' % self.counter
+            anchor = f'note_{self.counter}'
             self.notes[anchor] = (str(self.counter), note)
             return anchor, str(self.counter)
         return None, None
 
     def __iter__(self):
-        for anchor, (counter, note) in iteritems(self.notes):
+        for anchor, (counter, note) in self.notes.items():
             yield anchor, counter, note
 
     @property
