@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.13
 
 
 __license__ = 'GPL v3'
@@ -6,11 +6,12 @@ __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import os
 import re
+
 from lxml.html.builder import HR, IMG
 
 from calibre import sanitize_file_name
 from calibre.constants import iswindows
-from calibre.ebooks.docx.names import barename
+from calibre.ebooks.docx.names import SVG_BLIP_URI, barename
 from calibre.utils.filenames import ascii_filename
 from calibre.utils.img import image_to_data, resize_to_fit
 from calibre.utils.imghdr import what
@@ -49,9 +50,9 @@ def get_image_properties(parent, XPath, get):
             pass
     ans = {}
     if width is not None:
-        ans['width'] = '%.3gpt' % width
+        ans['width'] = f'{width:.3g}pt'
     if height is not None:
-        ans['height'] = '%.3gpt' % height
+        ans['height'] = f'{height:.3g}pt'
 
     alt = None
     title = None
@@ -87,13 +88,13 @@ def get_image_properties(parent, XPath, get):
 def get_image_margins(elem):
     ans = {}
     for w, css in iteritems({'L':'left', 'T':'top', 'R':'right', 'B':'bottom'}):
-        val = elem.get('dist%s' % w, None)
+        val = elem.get(f'dist{w}', None)
         if val is not None:
             try:
                 val = emu_to_pt(val)
             except (TypeError, ValueError):
                 continue
-            ans['padding-%s' % css] = '%.3gpt' % val
+            ans[f'padding-{css}'] = f'{val:.3g}pt'
     return ans
 
 
@@ -162,7 +163,7 @@ class Images:
         ext = what(None, raw) or base.rpartition('.')[-1] or 'jpeg'
         if ext == 'emf':
             # For an example, see: https://bugs.launchpad.net/bugs/1224849
-            self.log('Found an EMF image: %s, trying to extract embedded raster image' % fname)
+            self.log(f'Found an EMF image: {fname}, trying to extract embedded raster image')
             from calibre.utils.wmf.emf import emf_unwrap
             try:
                 raw = emf_unwrap(raw)
@@ -182,7 +183,7 @@ class Images:
         name = base
         while name in exists:
             n, e = base.rpartition('.')[0::2]
-            name = '%s-%d.%s' % (n, c, e)
+            name = f'{n}-{c}.{e}'
             c += 1
         return name
 
@@ -190,7 +191,7 @@ class Images:
         resized, img = resize_to_fit(raw, max_width, max_height)
         if resized:
             base, ext = os.path.splitext(base)
-            base = base + '-%dx%d%s' % (max_width, max_height, ext)
+            base = base + f'-{max_width}x{max_height}{ext}'
             raw = image_to_data(img, fmt=ext[1:])
         return raw, base, resized
 
@@ -236,16 +237,19 @@ class Images:
                 name = image_filename(name)
             alt = pr.get('descr') or alt
             for a in XPath('descendant::a:blip[@r:embed or @r:link]')(pic):
-                rid = get(a, 'r:embed')
-                if not rid:
-                    rid = get(a, 'r:link')
+                rid = get(a, 'r:embed') or get(a, 'r:link')
+                for asvg in XPath(f'./a:extLst/a:ext[@uri="{SVG_BLIP_URI}"]/asvg:svgBlip[@r:embed or @r:link]')(a):
+                    svg_rid = get(asvg, 'r:embed') or get(asvg, 'r:link')
+                    if svg_rid and svg_rid in self.rid_map:
+                        rid = svg_rid
+                        break
                 if rid and rid in self.rid_map:
                     try:
                         src = self.generate_filename(rid, name)
                     except LinkedImageNotFound as err:
-                        self.log.warn('Linked image: %s not found, ignoring' % err.fname)
+                        self.log.warn(f'Linked image: {err.fname} not found, ignoring')
                         continue
-                    img = IMG(src='images/%s' % src)
+                    img = IMG(src=f'images/{src}')
                     img.set('alt', alt or 'Image')
                     if title:
                         img.set('title', title)
@@ -289,7 +293,7 @@ class Images:
                 pass
             else:
                 if pct > 0:
-                    style['width'] = '%.3g%%' % pct
+                    style['width'] = f'{pct:.3g}%'
             align = get(pict[0], 'o:hralign', 'center')
             if align in {'left', 'right'}:
                 style['margin-left'] = '0' if align == 'left' else 'auto'
@@ -304,10 +308,10 @@ class Images:
                 try:
                     src = self.generate_filename(rid)
                 except LinkedImageNotFound as err:
-                    self.log.warn('Linked image: %s not found, ignoring' % err.fname)
+                    self.log.warn(f'Linked image: {err.fname} not found, ignoring')
                     continue
                 style = get(imagedata.getparent(), 'style')
-                img = IMG(src='images/%s' % src)
+                img = IMG(src=f'images/{src}')
                 alt = get(imagedata, 'o:title')
                 img.set('alt', alt or 'Image')
                 if 'position:absolute' in style:
