@@ -1,35 +1,39 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.13
 
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import sys, os, re, math, errno, uuid, numbers
+import errno
+import math
+import numbers
+import os
+import re
+import sys
+import uuid
 from collections import OrderedDict, defaultdict
 
 from lxml import html
-from lxml.html.builder import (
-    HTML, HEAD, TITLE, BODY, LINK, META, P, SPAN, BR, DIV, A, DT, DL, DD, H1)
+from lxml.html.builder import BODY, BR, DD, DIV, DL, DT, H1, HEAD, HTML, LINK, META, SPAN, TITLE, A, P
 
 from calibre import guess_type
-from calibre.ebooks.docx.container import DOCX, fromstring
-from calibre.ebooks.docx.names import XML, generate_anchor
-from calibre.ebooks.docx.styles import Styles, inherit, PageProperties
-from calibre.ebooks.docx.numbering import Numbering
-from calibre.ebooks.docx.fonts import Fonts, is_symbol_font, map_symbol_text
-from calibre.ebooks.docx.images import Images
-from calibre.ebooks.docx.tables import Tables
-from calibre.ebooks.docx.footnotes import Footnotes
 from calibre.ebooks.docx.cleanup import cleanup_markup
+from calibre.ebooks.docx.container import DOCX, fromstring
+from calibre.ebooks.docx.fields import Fields
+from calibre.ebooks.docx.fonts import Fonts, is_symbol_font, map_symbol_text
+from calibre.ebooks.docx.footnotes import Footnotes
+from calibre.ebooks.docx.images import Images
+from calibre.ebooks.docx.names import XML, generate_anchor
+from calibre.ebooks.docx.numbering import Numbering
+from calibre.ebooks.docx.settings import Settings
+from calibre.ebooks.docx.styles import PageProperties, Styles, inherit
+from calibre.ebooks.docx.tables import Tables
 from calibre.ebooks.docx.theme import Theme
 from calibre.ebooks.docx.toc import create_toc
-from calibre.ebooks.docx.fields import Fields
-from calibre.ebooks.docx.settings import Settings
 from calibre.ebooks.metadata.opf2 import OPFCreator
 from calibre.utils.localization import canonicalize_lang, lang_as_iso639_1
 from polyglot.builtins import iteritems, itervalues
 
-
 NBSP = '\xa0'
 
 
@@ -71,6 +75,7 @@ class Convert:
         self.dest_dir = dest_dir or os.getcwd()
         self.mi = self.docx.metadata
         self.body = BODY()
+        self.uuid = uuid.uuid4().hex
         self.theme = Theme(self.namespace)
         self.settings = Settings(self.namespace)
         self.tables = Tables(self.namespace)
@@ -146,7 +151,7 @@ class Convert:
                 dl = DL(id=anchor)
                 dl.set('class', 'footnote')
                 self.body.append(dl)
-                dl.append(DT('[', A('←' + text, href='#back_%s' % anchor, title=text)))
+                dl.append(DT('[', A('←' + text, href=f'#back_{anchor}', title=text)))
                 dl[-1][0].tail = ']'
                 dl.append(DD())
                 paras = []
@@ -179,7 +184,7 @@ class Convert:
                 if style.text_indent is inherit or (hasattr(style.text_indent, 'endswith') and style.text_indent.endswith('pt')):
                     if style.text_indent is not inherit:
                         indent = float(style.text_indent[:-2]) + indent
-                    style.text_indent = '%.3gpt' % indent
+                    style.text_indent = f'{indent:.3g}pt'
                     parent.text = tabs[-1].tail or ''
                     for i in tabs:
                         parent.remove(i)
@@ -231,13 +236,13 @@ class Convert:
                 notes_header.tag = h.tag
                 cls = h.get('class', None)
                 if cls and cls != 'notes-header':
-                    notes_header.set('class', '%s notes-header' % cls)
+                    notes_header.set('class', f'{cls} notes-header')
                 break
 
         self.fields.polish_markup(self.object_map)
 
         self.log.debug('Cleaning up redundant markup generated by Word')
-        self.cover_image = cleanup_markup(self.log, self.html, self.styles, self.dest_dir, self.detect_cover, self.namespace.XPath)
+        self.cover_image = cleanup_markup(self.log, self.html, self.styles, self.dest_dir, self.detect_cover, self.namespace.XPath, self.uuid)
 
         return self.write(doc)
 
@@ -279,6 +284,13 @@ class Convert:
             if fallbacks:
                 for choice in choices:
                     ac.remove(choice)
+                if len(fallbacks) == 1 and ac.getparent().tag.endswith('}r') and len(fallbacks[0]) == 1:
+                    q = fallbacks[0][0]
+                    if q.tag and (q.tag.endswith('}drawing') or q.tag.endswith('}pict')):
+                        p = ac.getparent()
+                        idx = p.index(ac)
+                        p.insert(idx, q)
+                        p.remove(ac)
 
     def read_styles(self, relationships_by_type):
 
@@ -310,11 +322,11 @@ class Convert:
             try:
                 seraw = self.docx.read(sename)
             except KeyError:
-                self.log.warn('Settings %s do not exist' % sename)
+                self.log.warn(f'Settings {sename} do not exist')
             except OSError as e:
                 if e.errno != errno.ENOENT:
                     raise
-                self.log.warn('Settings %s file missing' % sename)
+                self.log.warn(f'Settings {sename} file missing')
             else:
                 self.settings(fromstring(seraw))
 
@@ -322,14 +334,14 @@ class Convert:
             try:
                 foraw = self.docx.read(foname)
             except KeyError:
-                self.log.warn('Footnotes %s do not exist' % foname)
+                self.log.warn(f'Footnotes {foname} do not exist')
             else:
                 forel = self.docx.get_relationships(foname)
         if enname is not None:
             try:
                 enraw = self.docx.read(enname)
             except KeyError:
-                self.log.warn('Endnotes %s do not exist' % enname)
+                self.log.warn(f'Endnotes {enname} do not exist')
             else:
                 enrel = self.docx.get_relationships(enname)
         footnotes(fromstring(foraw) if foraw else None, forel, fromstring(enraw) if enraw else None, enrel)
@@ -339,7 +351,7 @@ class Convert:
             try:
                 raw = self.docx.read(fname)
             except KeyError:
-                self.log.warn('Fonts table %s does not exist' % fname)
+                self.log.warn(f'Fonts table {fname} does not exist')
             else:
                 fonts(fromstring(raw), embed_relationships, self.docx, self.dest_dir)
 
@@ -347,7 +359,7 @@ class Convert:
             try:
                 raw = self.docx.read(tname)
             except KeyError:
-                self.log.warn('Styles %s do not exist' % sname)
+                self.log.warn(f'Styles {sname} do not exist')
             else:
                 self.theme(fromstring(raw))
 
@@ -356,7 +368,7 @@ class Convert:
             try:
                 raw = self.docx.read(sname)
             except KeyError:
-                self.log.warn('Styles %s do not exist' % sname)
+                self.log.warn(f'Styles {sname} do not exist')
             else:
                 self.styles(fromstring(raw), fonts, self.theme)
                 styles_loaded = True
@@ -367,7 +379,7 @@ class Convert:
             try:
                 raw = self.docx.read(nname)
             except KeyError:
-                self.log.warn('Numbering styles %s do not exist' % nname)
+                self.log.warn(f'Numbering styles {nname} do not exist')
             else:
                 numbering(fromstring(raw), self.styles, self.docx.get_relationships(nname)[0])
 
@@ -376,11 +388,11 @@ class Convert:
     def write(self, doc):
         toc = create_toc(doc, self.body, self.resolved_link_map, self.styles, self.object_map, self.log, self.namespace)
         raw = html.tostring(self.html, encoding='utf-8', doctype='<!DOCTYPE html>')
-        with lopen(os.path.join(self.dest_dir, 'index.html'), 'wb') as f:
+        with open(os.path.join(self.dest_dir, 'index.html'), 'wb') as f:
             f.write(raw)
         css = self.styles.generate_css(self.dest_dir, self.docx, self.notes_nopb, self.nosupsub)
         if css:
-            with lopen(os.path.join(self.dest_dir, 'docx.css'), 'wb') as f:
+            with open(os.path.join(self.dest_dir, 'docx.css'), 'wb') as f:
                 f.write(css.encode('utf-8'))
 
         opf = OPFCreator(self.dest_dir, self.mi)
@@ -398,7 +410,7 @@ class Convert:
                 guide.append(E.reference(
                     href='index.html#' + self.toc_anchor, title=_('Table of Contents'), type='toc'))
         toc_file = os.path.join(self.dest_dir, 'toc.ncx')
-        with lopen(os.path.join(self.dest_dir, 'metadata.opf'), 'wb') as of, open(toc_file, 'wb') as ncx:
+        with open(os.path.join(self.dest_dir, 'metadata.opf'), 'wb') as of, open(toc_file, 'wb') as ncx:
             opf.render(of, ncx, 'toc.ncx', process_guide=process_guide)
         if os.path.getsize(toc_file) == 0:
             os.remove(toc_file)
@@ -506,7 +518,7 @@ class Convert:
         m = re.match(r'heading\s+(\d+)$', style.style_name or '', re.IGNORECASE)
         if m is not None:
             n = min(6, max(1, int(m.group(1))))
-            dest.tag = 'h%d' % n
+            dest.tag = f'h{n}'
             dest.set('data-heading-level', str(n))
 
         if style.bidi is True:
@@ -592,8 +604,7 @@ class Convert:
             if anchor and anchor in self.anchor_map:
                 span.set('href', '#' + self.anchor_map[anchor])
                 continue
-            self.log.warn('Hyperlink with unknown target (rid=%s, anchor=%s), ignoring' %
-                          (rid, anchor))
+            self.log.warn(f'Hyperlink with unknown target (rid={rid}, anchor={anchor}), ignoring')
             # hrefs that point nowhere give epubcheck a hernia. The element
             # should be styled explicitly by Word anyway.
             # span.set('href', '#')
@@ -618,7 +629,7 @@ class Convert:
                 if anchor in self.anchor_map:
                     span.set('href', '#' + self.anchor_map[anchor])
                     continue
-                self.log.warn('Hyperlink field with unknown anchor: %s' % anchor)
+                self.log.warn(f'Hyperlink field with unknown anchor: {anchor}')
             else:
                 if url in self.anchor_map:
                     span.set('href', '#' + self.anchor_map[url])
@@ -660,15 +671,18 @@ class Convert:
                 ctext = child.text
                 if space != 'preserve':
                     # Remove leading and trailing whitespace. Word ignores
-                    # leading and trailing whitespace without preserve
-                    ctext = ctext.strip(' \n\r\t')
+                    # leading and trailing whitespace without preserve unless
+                    # the element is only whitespace.
+                    stripped = ctext.strip(' \n\r\t')
+                    if stripped:
+                        ctext = stripped
                 # Only use a <span> with white-space:pre-wrap if this element
                 # actually needs it, i.e. if it has more than one
                 # consecutive space or it has newlines or tabs.
                 multi_spaces = self.ms_pat.search(ctext) is not None
                 preserve = multi_spaces or self.ws_pat.search(ctext) is not None
                 if preserve:
-                    text.add_elem(SPAN(ctext, style="white-space:pre-wrap"))
+                    text.add_elem(SPAN(ctext, style='white-space:pre-wrap'))
                     ans.append(text.elem)
                 else:
                     text.buf.append(ctext)
@@ -682,7 +696,7 @@ class Convert:
                 else:
                     clear = child.get('clear', None)
                     if clear in {'all', 'left', 'right'}:
-                        br = BR(style='clear:%s'%('both' if clear == 'all' else clear))
+                        br = BR(style='clear:{}'.format('both' if clear == 'all' else clear))
                     else:
                         br = BR()
                 text.add_elem(br)
@@ -694,12 +708,14 @@ class Convert:
             elif self.namespace.is_tag(child, 'w:footnoteReference') or self.namespace.is_tag(child, 'w:endnoteReference'):
                 anchor, name = self.footnotes.get_ref(child)
                 if anchor and name:
-                    l = A(name, id='back_%s' % anchor, href='#' + anchor, title=name)
+                    l = A(name, id=f'back_{anchor}', href='#' + anchor, title=name)
                     l.set('class', 'noteref')
+                    l.set('role', 'doc-noteref')
                     text.add_elem(l)
                     ans.append(text.elem)
+                    ans.set('data-noteref-container', self.uuid)
             elif self.namespace.is_tag(child, 'w:tab'):
-                spaces = int(math.ceil((self.settings.default_tab_stop / 36) * 6))
+                spaces = math.ceil((self.settings.default_tab_stop / 36) * 6)
                 text.add_elem(SPAN(NBSP * spaces))
                 ans.append(text.elem)
                 ans[-1].set('class', 'tab')
@@ -840,6 +856,7 @@ class Convert:
 
 if __name__ == '__main__':
     import shutil
+
     from calibre.utils.logging import default_log
     default_log.filter_level = default_log.DEBUG
     dest_dir = os.path.join(os.getcwd(), 'docx_input')
