@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.13
 
 
 __license__ = 'GPL v3'
@@ -30,7 +30,7 @@ def from_headings(body, log, namespace, num_levels=3):
     def ensure_id(elem):
         ans = elem.get('id', None)
         if not ans:
-            ans = 'toc_id_%d' % (next(idcount) + 1)
+            ans = f'toc_id_{next(idcount) + 1}'
             elem.set('id', ans)
         return ans
 
@@ -123,7 +123,7 @@ def from_toc(docx, link_map, styles, object_map, log, namespace):
                 if txt and href and p is not None:
                     ps = styles.resolve_paragraph(p)
                     try:
-                        ml = int(ps.margin_left[:-2])
+                        ml = float(ps.margin_left[:-2])
                     except (TypeError, ValueError, AttributeError):
                         ml = 0
                     if ps.text_align in {'center', 'right'}:
