@@ -9,12 +9,12 @@ from collections import Counter, defaultdict
 from operator import attrgetter
 
 from lxml import etree
+from tinycss.css21 import CSS21Parser
 
 from calibre.ebooks import parse_css_length
 from calibre.ebooks.docx.writer.utils import convert_color, int_or_zero
 from calibre.utils.localization import lang_as_iso639_1
 from polyglot.builtins import iteritems
-from tinycss.css21 import CSS21Parser
 
 css_parser = CSS21Parser()
 
@@ -64,7 +64,8 @@ class CombinedStyle:
 
     def serialize(self, styles, normal_style):
         makeelement = self.namespace.makeelement
-        w = lambda x: '{{{}}}{}'.format(self.namespace.namespaces['w'], x)
+        def w(x):
+            return '{{{}}}{}'.format(self.namespace.namespaces['w'], x)
         block = makeelement(styles, 'w:style', w_styleId=self.id, w_type='paragraph')
         makeelement(block, 'w:name', w_val=self.name)
         makeelement(block, 'w:qFormat')
@@ -134,10 +135,10 @@ class FloatSpec:
         bdr = self.makeelement(parent, 'w:pBdr')
         for edge in border_edges:
             padding = getattr(self, 'padding_' + edge)
-            width = getattr(self, 'border_%s_width' % edge)
-            bstyle = getattr(self, 'border_%s_style' % edge)
+            width = getattr(self, f'border_{edge}_width')
+            bstyle = getattr(self, f'border_{edge}_style')
             self.makeelement(
-                bdr, 'w:'+edge, w_space=str(padding), w_val=bstyle, w_sz=str(width), w_color=getattr(self, 'border_%s_color' % edge))
+                bdr, 'w:'+edge, w_space=str(padding), w_val=bstyle, w_sz=str(width), w_color=getattr(self, f'border_{edge}_color'))
 
 
 class DOCXStyle:
@@ -199,6 +200,25 @@ LINE_STYLES = {
 }
 
 
+def convert_underline(items):
+    style = 'solid'
+    has_underline = False
+    color = 'auto'
+    for x in items:
+        if x in {'solid', 'double', 'dotted', 'dashed', 'wavy'}:
+            style = {'solid': 'single', 'wavy': 'wave', 'dashed': 'dash'}.get(x, x)
+        elif x in {'underline', 'overline', 'line-through', 'blink', 'none'}:
+            if x == 'underline':
+                has_underline = True
+            elif x == 'none':
+                has_underline = False
+        else:
+            color = convert_color(x)
+    if has_underline:
+        return style + ' ' + color
+    return ''
+
+
 class TextStyle(DOCXStyle):
 
     ALL_PROPS = ('font_family', 'font_size', 'bold', 'italic', 'color',
@@ -220,7 +240,7 @@ class TextStyle(DOCXStyle):
         self.color = convert_color(css['color'])
         self.background_color = None if is_parent_style else convert_color(css.backgroundColor)
         td = set((css.effective_text_decoration or '').split())
-        self.underline = 'underline' in td
+        self.underline = convert_underline(td)
         self.dstrike = 'line-through' in td and 'overline' in td
         self.strike = not self.dstrike and 'line-through' in td
         self.text_transform = css['text-transform']  # TODO: If lowercase or capitalize, transform the actual text
@@ -253,7 +273,7 @@ class TextStyle(DOCXStyle):
                     self.padding = padding
                 elif self.padding != padding:
                     self.padding = ignore
-                val = css['border-%s-width' % edge]
+                val = css[f'border-{edge}-width']
                 if not isinstance(val, numbers.Number):
                     val = {'thin':0.2, 'medium':1, 'thick':2}.get(val, 0)
                 val = min(96, max(2, int(val * 8)))
@@ -261,12 +281,12 @@ class TextStyle(DOCXStyle):
                     self.border_width = val
                 elif self.border_width != val:
                     self.border_width = ignore
-                color = convert_color(css['border-%s-color' % edge])
+                color = convert_color(css[f'border-{edge}-color'])
                 if self.border_color is None:
                     self.border_color = color
                 elif self.border_color != color:
                     self.border_color = ignore
-                style = LINE_STYLES.get(css['border-%s-style' % edge].lower(), 'none')
+                style = LINE_STYLES.get(css[f'border-{edge}-style'].lower(), 'none')
                 if self.border_style is None:
                     self.border_style = style
                 elif self.border_style != style:
@@ -329,7 +349,11 @@ class TextStyle(DOCXStyle):
         if check_attr('background_color'):
             rPr.append(makeelement(rPr, 'shd', fill=self.background_color or 'auto'))
         if check_attr('underline'):
-            rPr.append(makeelement(rPr, 'u', val='single' if self.underline else 'none'))
+            style, color = self.underline.partition(' ')[::2]
+            if color != 'auto':
+                rPr.append(makeelement(rPr, 'u', val=style, color=color))
+            else:
+                rPr.append(makeelement(rPr, 'u', val=style))
         if check_attr('dstrike'):
             rPr.append(makeelement(rPr, 'dstrike', val=bmap(self.dstrike)))
         if check_attr('strike'):
@@ -388,7 +412,11 @@ class DescendantTextStyle:
         if check('background_color'):
             add('shd', fill=child_style.background_color or 'auto')
         if check('underline'):
-            add('u', val='single' if child_style.underline else 'none')
+            if not child_style.underline:
+                add('u', val='none')
+            else:
+                style, color = child_style.underline.partition(' ')[::2]
+                add('u', val=style, color=color)
         if check('dstrike'):
             add('dstrike', val=bmap(child_style.dstrike))
         if check('strike'):
@@ -449,11 +477,11 @@ def read_css_block_borders(self, css, store_css_style=False):
             setattr(self, 'padding_' + edge, 0)
             setattr(self, 'margin_' + edge, 0)
             setattr(self, 'css_margin_' + edge, '')
-            setattr(self, 'border_%s_width' % edge, 2)
-            setattr(self, 'border_%s_color' % edge, None)
-            setattr(self, 'border_%s_style' %  edge, 'none')
+            setattr(self, f'border_{edge}_width', 2)
+            setattr(self, f'border_{edge}_color', None)
+            setattr(self, f'border_{edge}_style', 'none')
             if store_css_style:
-                setattr(self, 'border_%s_css_style' %  edge, 'none')
+                setattr(self, f'border_{edge}_css_style', 'none')
         else:
             # In DOCX padding can only be a positive integer
             try:
@@ -466,15 +494,15 @@ def read_css_block_borders(self, css, store_css_style=False):
             except ValueError:
                 setattr(self, 'margin_' + edge, 0)  # e.g.: margin: auto
             setattr(self, 'css_margin_' + edge, css._style.get('margin-' + edge, ''))
-            val = css['border-%s-width' % edge]
+            val = css[f'border-{edge}-width']
             if not isinstance(val, numbers.Number):
                 val = {'thin':0.2, 'medium':1, 'thick':2}.get(val, 0)
             val = min(96, max(2, int(val * 8)))
-            setattr(self, 'border_%s_width' % edge, val)
-            setattr(self, 'border_%s_color' % edge, convert_color(css['border-%s-color' % edge]) or 'auto')
-            setattr(self, 'border_%s_style' %  edge, LINE_STYLES.get(css['border-%s-style' % edge].lower(), 'none'))
+            setattr(self, f'border_{edge}_width', val)
+            setattr(self, f'border_{edge}_color', convert_color(css[f'border-{edge}-color']) or 'auto')
+            setattr(self, f'border_{edge}_style', LINE_STYLES.get(css[f'border-{edge}-style'].lower(), 'none'))
             if store_css_style:
-                setattr(self, 'border_%s_css_style' %  edge, css['border-%s-style' % edge].lower())
+                setattr(self, f'border_{edge}_css_style', css[f'border-{edge}-style'].lower())
 
 
 class BlockStyle(DOCXStyle):
@@ -490,8 +518,8 @@ class BlockStyle(DOCXStyle):
         read_css_block_borders(self, css)
         if is_table_cell:
             for edge in border_edges:
-                setattr(self, 'border_%s_style' % edge, 'none')
-                setattr(self, 'border_%s_width' % edge, 0)
+                setattr(self, f'border_{edge}_style', 'none')
+                setattr(self, f'border_{edge}_width', 0)
                 setattr(self, 'padding_' + edge, 0)
                 setattr(self, 'margin_' + edge, 0)
         if css is None:
@@ -537,14 +565,14 @@ class BlockStyle(DOCXStyle):
             padding = getattr(self, 'padding_' + edge)
             if (self is normal_style and padding > 0) or (padding != getattr(normal_style, 'padding_' + edge)):
                 e.set(w('space'), str(padding))
-            width = getattr(self, 'border_%s_width' % edge)
-            bstyle = getattr(self, 'border_%s_style' % edge)
+            width = getattr(self, f'border_{edge}_width')
+            bstyle = getattr(self, f'border_{edge}_style')
             if (self is normal_style and width > 0 and bstyle != 'none'
-                    ) or width != getattr(normal_style, 'border_%s_width' % edge
-                    ) or bstyle != getattr(normal_style, 'border_%s_style' % edge):
+                    ) or width != getattr(normal_style, f'border_{edge}_width'
+                    ) or bstyle != getattr(normal_style, f'border_{edge}_style'):
                 e.set(w('val'), bstyle)
                 e.set(w('sz'), str(width))
-                e.set(w('color'), getattr(self, 'border_%s_color' % edge))
+                e.set(w('color'), getattr(self, f'border_{edge}_color'))
             if e.attrib:
                 bdr.append(e)
         return bdr
@@ -716,7 +744,7 @@ class StylesManager:
                 if style.outline_level is None:
                     val = f'Para %0{snum}d' % i
                 else:
-                    val = 'Heading %d' % (style.outline_level + 1)
+                    val = f'Heading {style.outline_level + 1}'
                     heading_styles.append(style)
                 style.id = style.name = val
             style.seq = i
@@ -736,13 +764,12 @@ class StylesManager:
                         ds_counts[run.descendant_style] += run.style_weight
         rnum = len(str(max(1, len(ds_counts) - 1)))
         for i, (text_style, count) in enumerate(ds_counts.most_common()):
-            text_style.id = 'Text%d' % i
+            text_style.id = f'Text{i}'
             text_style.name = f'%0{rnum}d Text' % i
             text_style.seq = i
         self.descendant_text_styles = sorted(descendant_style_map, key=attrgetter('seq'))
 
-        self.log.debug('%d Text Styles %d Combined styles' % tuple(map(len, (
-            self.descendant_text_styles, self.combined_styles))))
+        self.log.debug(f'{len(self.descendant_text_styles)} Text Styles {len(self.combined_styles)} Combined styles')
 
         self.primary_heading_style = None
         if heading_styles:
