@@ -1,22 +1,23 @@
-#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal kovid@kovidgoyal.net'
 __docformat__ = 'restructuredtext en'
 
-"""
+'''
 Provides abstraction for metadata reading.writing from a variety of ebook formats.
-"""
-import os, sys, re
+'''
+import os
+import re
+import sys
 from contextlib import suppress
 
-from calibre import relpath, guess_type, prints, force_unicode
+from calibre import force_unicode, guess_type, prints, relpath
 from calibre.utils.config_base import tweaks
-from polyglot.builtins import codepoint_to_chr, iteritems, as_unicode
+from polyglot.builtins import as_unicode, iteritems
 from polyglot.urllib import quote, unquote, urlparse
 
-
 try:
     _author_pat = re.compile(tweaks['authors_split_regex'])
 except Exception:
@@ -165,19 +166,36 @@ def get_title_sort_pat(lang=None):
         ans = frozenset((r'A\s+', r'The\s+', r'An\s+'))
     if ans:
         ans = '|'.join(ans)
-        ans = '^(%s)'%ans
+        ans = f'^({ans})'
         try:
             ans = re.compile(ans, re.IGNORECASE)
         except:
             ans = re.compile(r'^(A|The|An)\s+', re.IGNORECASE)
     else:
-        ans = re.compile('^$')  # matches only the empty string
+        ans = re.compile(r'^$')  # matches only the empty string
     _title_pats[lang] = ans
     return ans
 
 
-_ignore_starts = '\'"'+''.join(codepoint_to_chr(x) for x in
-        list(range(0x2018, 0x201e))+[0x2032, 0x2033])
+quote_pairs = {
+    # https://en.wikipedia.org/wiki/Quotation_mark
+    '"': ('"',),
+    "'": ("'",),
+    '“': ('”','“'),
+    '”': ('”','”'),
+    '„': ('”','“'),
+    '‚': ('’','‘'),
+    '’': ('’','‘'),
+    '‘': ('’','‘'),
+    '‹': ('›',),
+    '›': ('‹',),
+    '《': ('》',),
+    '〈': ('〉',),
+    '»': ('«', '»'),
+    '«': ('«', '»'),
+    '「': ('」',),
+    '『': ('』',),
+}
 
 
 def title_sort(title, order=None, lang=None):
@@ -186,8 +204,11 @@ def title_sort(title, order=None, lang=None):
     title = title.strip()
     if order == 'strictly_alphabetic':
         return title
-    if title and title[0] in _ignore_starts:
+    if title and title[0] in quote_pairs:
+        q = title[0]
         title = title[1:]
+        if title and title[-1] in quote_pairs[q]:
+            title = title[:-1]
     match = get_title_sort_pat(lang).search(title)
     if match:
         try:
@@ -197,14 +218,17 @@ def title_sort(title, order=None, lang=None):
         else:
             if prep:
                 title = title[len(prep):] + ', ' + prep
-                if title[0] in _ignore_starts:
+                if title[0] in quote_pairs:
+                    q = title[0]
                     title = title[1:]
+                    if title and title[-1] in quote_pairs[q]:
+                        title = title[:-1]
     return title.strip()
 
 
 coding = list(zip(
 [1000,900,500,400,100,90,50,40,10,9,5,4,1],
-["M","CM","D","CD","C","XC","L","XL","X","IX","V","IV","I"]
+['M','CM','D','CD','C','XC','L','XL','X','IX','V','IV','I']
 ))
 
 
@@ -226,8 +250,8 @@ def fmt_sidx(i, fmt='%.2f', use_roman=False):
         i = float(i)
     except Exception:
         return str(i)
-    if int(i) == float(i):
-        return roman(int(i)) if use_roman else '%d'%int(i)
+    if int(i) == i:
+        return roman(int(i)) if use_roman else str(int(i))
     ans = fmt%i
     if '.' in ans:
         ans = ans.rstrip('0')
@@ -235,7 +259,6 @@ def fmt_sidx(i, fmt='%.2f', use_roman=False):
 
 
 class Resource:
-
     '''
     Represents a resource (usually a file on the filesystem or a URL pointing
     to the web. Such resources are commonly referred to in OPF files.
@@ -312,7 +335,7 @@ class Resource:
         return self._basedir
 
     def __repr__(self):
-        return 'Resource(%s, %s)'%(repr(self.path), repr(self.href()))
+        return f'Resource({self.path!r}, {self.href()!r})'
 
 
 class ResourceCollection:
@@ -334,7 +357,7 @@ class ResourceCollection:
 
     def __str__(self):
         resources = map(repr, self)
-        return '[%s]'%', '.join(resources)
+        return '[{}]'.format(', '.join(resources))
 
     def __repr__(self):
         return str(self)
