@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
@@ -85,8 +85,8 @@ CALIBRE_METADATA_FIELDS = frozenset((
     # a dict of user category names, where the value is a list of item names
     # from the book that are in that category
     'user_categories',
-    # a dict of author to an associated hyperlink
-    'author_link_map',
+    # a dict of items to associated hyperlink
+    'link_maps',
 ))
 
 ALL_METADATA_FIELDS =      SOCIAL_METADATA_FIELDS.union(
