@@ -4,14 +4,16 @@ Created on 4 Jun 2010
 @author: charles
 '''
 
-from base64 import b64encode, b64decode
-import json, traceback
+import json
+import traceback
 from datetime import datetime, time
 
-from calibre.ebooks.metadata.book import SERIALIZABLE_FIELDS
-from calibre.constants import filesystem_encoding, preferred_encoding
-from calibre.library.field_metadata import FieldMetadata
 from calibre import isbytestring
+from calibre.constants import filesystem_encoding, preferred_encoding
+from calibre.ebooks.metadata.book import SERIALIZABLE_FIELDS
+from calibre.library.field_metadata import FieldMetadata
+from polyglot.binary import as_base64_unicode, from_base64_bytes
+from polyglot.builtins import as_bytes, iteritems, itervalues
 
 # Translate datetimes to and from strings. The string form is the datetime in
 # UTC. The returned date is also UTC
@@ -19,7 +21,7 @@ from calibre import isbytestring
 
 def string_to_datetime(src):
     from calibre.utils.iso8601 import parse_iso8601
-    if src != "None":
+    if src != 'None':
         try:
             return parse_iso8601(src)
         except Exception:
@@ -28,15 +30,15 @@ def string_to_datetime(src):
 
 
 def datetime_to_string(dateval):
-    from calibre.utils.date import isoformat, UNDEFINED_DATE, local_tz
+    from calibre.utils.date import UNDEFINED_DATE, isoformat, local_tz
     if dateval is None:
-        return "None"
+        return 'None'
     if not isinstance(dateval, datetime):
         dateval = datetime.combine(dateval, time())
     if hasattr(dateval, 'tzinfo') and dateval.tzinfo is None:
         dateval = dateval.replace(tzinfo=local_tz)
     if dateval <= UNDEFINED_DATE:
-        return "None"
+        return 'None'
     return isoformat(dateval)
 
 
@@ -49,13 +51,13 @@ def encode_thumbnail(thumbnail):
         return None
     if not isinstance(thumbnail, (tuple, list)):
         try:
-            width, height = identify(bytes(thumbnail))[1:]
+            width, height = identify(as_bytes(thumbnail))[1:]
             if width < 0 or height < 0:
                 return None
             thumbnail = (width, height, thumbnail)
         except Exception:
             return None
-    return (thumbnail[0], thumbnail[1], b64encode(str(thumbnail[2])))
+    return (thumbnail[0], thumbnail[1], as_base64_unicode(thumbnail[2]))
 
 
 def decode_thumbnail(tup):
@@ -64,7 +66,7 @@ def decode_thumbnail(tup):
     '''
     if tup is None:
         return None
-    return (tup[0], tup[1], b64decode(tup[2]))
+    return (tup[0], tup[1], from_base64_bytes(tup[2]))
 
 
 def object_to_unicode(obj, enc=preferred_encoding):
@@ -94,20 +96,20 @@ def encode_is_multiple(fm):
         if dt == 'composite':
             fm['is_multiple'] = ','
         else:
-            fm['is_multiple'] =  '|'
+            fm['is_multiple'] = '|'
     else:
         fm['is_multiple'] = None
         fm['is_multiple2'] = {}
 
 
 def decode_is_multiple(fm):
-    im = fm.get('is_multiple2',  None)
+    im = fm.get('is_multiple2', None)
     if im:
         fm['is_multiple'] = im
         del fm['is_multiple2']
     else:
         # Must migrate the is_multiple from char to dict
-        im = fm.get('is_multiple',  {})
+        im = fm.get('is_multiple', {})
         if im:
             dt = fm.get('datatype', None)
             if dt == 'composite':
@@ -124,14 +126,16 @@ def decode_is_multiple(fm):
         fm['is_multiple'] = im
 
 
-class JsonCodec(object):
+class JsonCodec:
 
     def __init__(self, field_metadata=None):
         self.field_metadata = field_metadata or FieldMetadata()
 
     def encode_to_file(self, file_, booklist):
-        file_.write(json.dumps(self.encode_booklist_metadata(booklist),
-                              indent=2, encoding='utf-8'))
+        data = json.dumps(self.encode_booklist_metadata(booklist), indent=2)
+        if not isinstance(data, bytes):
+            data = data.encode('utf-8')
+        file_.write(data)
 
     def encode_booklist_metadata(self, booklist):
         result = []
@@ -148,7 +152,7 @@ class JsonCodec(object):
     def encode_metadata_attr(self, book, key):
         if key == 'user_metadata':
             meta = book.get_all_user_metadata(make_copy=True)
-            for fm in meta.itervalues():
+            for fm in itervalues(meta):
                 if fm['datatype'] == 'datetime':
                     fm['#value#'] = datetime_to_string(fm['#value#'])
                 encode_is_multiple(fm)
@@ -171,19 +175,19 @@ class JsonCodec(object):
     def decode_from_file(self, file_, booklist, book_class, prefix):
         js = []
         try:
-            js = json.load(file_, encoding='utf-8')
+            js = json.load(file_)
             for item in js:
                 entry = self.raw_to_book(item, book_class, prefix)
                 if entry is not None:
                     booklist.append(entry)
         except:
-            print 'exception during JSON decode_from_file'
+            print('exception during JSON decode_from_file')
             traceback.print_exc()
 
     def raw_to_book(self, json_book, book_class, prefix):
         try:
             book = book_class(prefix, json_book.get('lpath', None))
-            for key,val in json_book.iteritems():
+            for key,val in iteritems(json_book):
                 meta = self.decode_metadata(key, val)
                 if key == 'user_metadata':
                     book.set_all_user_metadata(meta)
@@ -193,14 +197,14 @@ class JsonCodec(object):
                     setattr(book, key, meta)
             return book
         except:
-            print 'exception during JSON decoding'
+            print('exception during JSON decoding')
             traceback.print_exc()
 
     def decode_metadata(self, key, value):
         if key == 'classifiers':
             key = 'identifiers'
         if key == 'user_metadata':
-            for fm in value.itervalues():
+            for fm in itervalues(value):
                 if fm['datatype'] == 'datetime':
                     fm['#value#'] = string_to_datetime(fm['#value#'])
                 decode_is_multiple(fm)
