@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
@@ -8,9 +8,9 @@ __docformat__ = 'restructuredtext en'
 from io import BytesIO
 
 from calibre.ebooks.docx.container import DOCX
-from calibre.utils.xml_parse import safe_xml_fromstring
 from calibre.ebooks.docx.writer.container import update_doc_props, xml2str
 from calibre.utils.imghdr import identify
+from calibre.utils.xml_parse import safe_xml_fromstring
 
 
 def get_cover(docx):
@@ -30,7 +30,7 @@ def get_cover(docx):
             if width < 0 or height < 0:
                 continue
             if 0.8 <= height/width <= 1.8 and height*width >= 160000:
-                return (fmt, raw)
+                return fmt, raw
 
 
 def get_metadata(stream):
@@ -64,7 +64,7 @@ def set_metadata(stream, mi):
     replacements = {}
     if ap_raw is not None:
         ap = safe_xml_fromstring(ap_raw)
-        comp = ap.makeelement('{%s}Company' % c.namespace.namespaces['ep'])
+        comp = ap.makeelement('{{{}}}Company'.format(c.namespace.namespaces['ep']))
         for child in tuple(ap):
             if child.tag == comp.tag:
                 ap.remove(child)
