@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
@@ -10,16 +10,16 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 import io
 import os
 import posixpath
-from contextlib import closing
+from contextlib import closing, contextmanager, suppress
 
 from calibre import CurrentDir
-from calibre.ebooks.metadata.opf import (
-    get_metadata as get_metadata_from_opf, set_metadata as set_metadata_opf
-)
+from calibre.ebooks.metadata.opf import get_metadata as get_metadata_from_opf
+from calibre.ebooks.metadata.opf import set_metadata as set_metadata_opf
 from calibre.ebooks.metadata.opf2 import OPF
-from calibre.utils.xml_parse import safe_xml_fromstring
 from calibre.ptempfile import TemporaryDirectory
+from calibre.utils.imghdr import what as what_image_type
 from calibre.utils.localunzip import LocalZipFile
+from calibre.utils.xml_parse import safe_xml_fromstring
 from calibre.utils.zipfile import BadZipfile, ZipFile, safe_replace
 
 
@@ -37,19 +37,25 @@ class ContainerException(OCFException):
 
 class Container(dict):
 
-    def __init__(self, stream=None):
+    def __init__(self, stream=None, file_exists=None):
         if not stream:
             return
         container = safe_xml_fromstring(stream.read())
         if container.get('version', None) != '1.0':
-            raise EPubException("unsupported version of OCF")
+            raise EPubException('unsupported version of OCF')
         rootfiles = container.xpath('./*[local-name()="rootfiles"]')
         if not rootfiles:
-            raise EPubException("<rootfiles/> element missing")
+            raise EPubException('<rootfiles/> element missing')
         for rootfile in rootfiles[0].xpath('./*[local-name()="rootfile"]'):
             mt, fp = rootfile.get('media-type'), rootfile.get('full-path')
             if not mt or not fp:
-                raise EPubException("<rootfile/> element malformed")
+                raise EPubException('<rootfile/> element malformed')
+
+            if file_exists and not file_exists(fp):
+                # Some Kobo epubs have multiple rootfile entries, but only one
+                # exists.  Ignore the ones that don't exist.
+                continue
+
             self[mt] = fp
 
 
@@ -92,16 +98,16 @@ class OCFReader(OCF):
             if mimetype != OCF.MIMETYPE:
                 print('WARNING: Invalid mimetype declaration', mimetype)
         except:
-            print('WARNING: Epub doesn\'t contain a valid mimetype declaration')
+            print("WARNING: Epub doesn't contain a valid mimetype declaration")
 
         try:
             with closing(self.open(OCF.CONTAINER_PATH)) as f:
-                self.container = Container(f)
+                self.container = Container(f, self.exists)
         except KeyError:
-            raise EPubException("missing OCF container.xml file")
+            raise EPubException('missing OCF container.xml file')
         self.opf_path = self.container[OPF.MIMETYPE]
         if not self.opf_path:
-            raise EPubException("missing OPF package file entry in container")
+            raise EPubException('missing OPF package file entry in container')
         self._opf_cached = self._encryption_meta_cached = None
 
     @property
@@ -111,7 +117,7 @@ class OCFReader(OCF):
                 with closing(self.open(self.opf_path)) as f:
                     self._opf_cached = OPF(f, self.root, populate_spine=False)
             except KeyError:
-                raise EPubException("missing OPF package file")
+                raise EPubException('missing OPF package file')
         return self._opf_cached
 
     @property
@@ -126,6 +132,13 @@ class OCFReader(OCF):
     def read_bytes(self, name):
         return self.open(name).read()
 
+    def exists(self, path):
+        try:
+            self.open(path).close()
+            return True
+        except OSError:
+            return False
+
 
 class OCFZipReader(OCFReader):
 
@@ -136,7 +149,7 @@ class OCFZipReader(OCFReader):
             try:
                 self.archive = ZipFile(stream, mode=mode)
             except BadZipfile:
-                raise EPubException("not a ZIP .epub OCF container")
+                raise EPubException('not a ZIP .epub OCF container')
         self.root = root
         if self.root is None:
             name = getattr(stream, 'name', False)
@@ -154,6 +167,13 @@ class OCFZipReader(OCFReader):
     def read_bytes(self, name):
         return self.archive.read(name)
 
+    def exists(self, path):
+        try:
+            self.archive.getinfo(path)
+            return True
+        except KeyError:
+            return False
+
 
 def get_zip_reader(stream, root=None):
     try:
@@ -171,7 +191,7 @@ class OCFDirReader(OCFReader):
         super().__init__()
 
     def open(self, path):
-        return lopen(os.path.join(self.root, path), 'rb')
+        return open(os.path.join(self.root, path), 'rb')
 
     def read_bytes(self, path):
         with self.open(path) as f:
@@ -193,7 +213,34 @@ def render_cover(cpage, zf, reader=None):
             cpage = os.path.join(tdir, cpage)
             if not os.path.exists(cpage):
                 return
-            return render_html_svg_workaround(cpage, default_log)
+
+            with suppress(Exception):
+                # In the case of manga, the first spine item may be an image
+                # already, so treat it as a raster cover.
+                file_format = what_image_type(cpage)
+                if file_format == 'jpeg':
+                    # Only JPEG is allowed since elsewhere we assume raster covers
+                    # are JPEG.  In principle we could convert other image formats
+                    # but this is already an out-of-spec case that happens to
+                    # arise in books from some stores.
+                    with open(cpage, 'rb') as source:
+                        return source.read()
+
+            return render_html_svg_workaround(cpage, default_log, root=tdir)
+
+
+epub_allow_rendered_cover = True
+
+
+@contextmanager
+def epub_metadata_settings(allow_rendered_cover=epub_allow_rendered_cover):
+    global epub_allow_rendered_cover
+    oarc = epub_allow_rendered_cover
+    epub_allow_rendered_cover = allow_rendered_cover
+    try:
+        yield
+    finally:
+        epub_allow_rendered_cover = oarc
 
 
 def get_cover(raster_cover, first_spine_item, reader):
@@ -207,15 +254,16 @@ def get_cover(raster_cover, first_spine_item, reader):
         except Exception:
             pass
 
-    return render_cover(first_spine_item, zf, reader=reader)
+    if epub_allow_rendered_cover:
+        return render_cover(first_spine_item, zf, reader=reader)
 
 
-def get_metadata(stream, extract_cover=True):
-    """ Return metadata as a :class:`Metadata` object """
+def get_metadata(stream, extract_cover=True, ftype='epub'):
+    ''' Return metadata as a :class:`Metadata` object '''
     stream.seek(0)
     reader = get_zip_reader(stream)
     opfbytes = reader.read_bytes(reader.opf_path)
-    mi, ver, raster_cover, first_spine_item = get_metadata_from_opf(opfbytes)
+    mi, ver, raster_cover, first_spine_item = get_metadata_from_opf(opfbytes, ftype)
     if extract_cover:
         base = posixpath.dirname(reader.opf_path)
         if raster_cover:
@@ -233,7 +281,7 @@ def get_metadata(stream, extract_cover=True):
     return mi
 
 
-def get_quick_metadata(stream):
+def get_quick_metadata(stream, ftype='epub'):
     return get_metadata(stream, False)
 
 
@@ -242,7 +290,7 @@ def serialize_cover_data(new_cdata, cpath):
     return save_cover_data_to(new_cdata, data_fmt=os.path.splitext(cpath)[1][1:])
 
 
-def set_metadata(stream, mi, apply_null=False, update_timestamp=False, force_identifiers=False, add_missing_cover=True):
+def set_metadata(stream, mi, apply_null=False, update_timestamp=False, force_identifiers=False, add_missing_cover=True, ftype='epub'):
     stream.seek(0)
     reader = get_zip_reader(stream, root=os.getcwd())
     new_cdata = None
@@ -252,7 +300,7 @@ def set_metadata(stream, mi, apply_null=False, update_timestamp=False, force_ide
             raise Exception('no cover')
     except Exception:
         try:
-            with lopen(mi.cover, 'rb') as f:
+            with open(mi.cover, 'rb') as f:
                 new_cdata = f.read()
         except Exception:
             pass
@@ -260,7 +308,7 @@ def set_metadata(stream, mi, apply_null=False, update_timestamp=False, force_ide
     opfbytes, ver, raster_cover = set_metadata_opf(
         reader.read_bytes(reader.opf_path), mi, cover_prefix=posixpath.dirname(reader.opf_path),
         cover_data=new_cdata, apply_null=apply_null, update_timestamp=update_timestamp,
-        force_identifiers=force_identifiers, add_missing_cover=add_missing_cover)
+        force_identifiers=force_identifiers, add_missing_cover=add_missing_cover, ftype=ftype)
     cpath = None
     replacements = {}
     if new_cdata and raster_cover:
