@@ -1,28 +1,27 @@
-#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
-__copyright__ = '2011, Roman Mukhin <ramses_ru at hotmail.com>, '\
-                '2008, Anatoly Shipitsin <norguhtar at gmail.com>'
+__copyright__ = ('2011, Roman Mukhin <ramses_ru at hotmail.com>, '
+                 '2008, Anatoly Shipitsin <norguhtar at gmail.com>')
 '''Read meta information from fb2 files'''
 
-import os, random
+import os
+import random
 from functools import partial
 from string import ascii_letters, digits
 
 from lxml import etree
 
-from calibre import strftime
+from calibre import force_unicode, guess_all_extensions, guess_type, prints, strftime
+from calibre.ebooks.chardet import xml_to_unicode
+from calibre.ebooks.metadata import MetaInformation, check_isbn
 from calibre.utils.date import parse_only_date
 from calibre.utils.img import save_cover_data_to
-from calibre.utils.xml_parse import safe_xml_fromstring
 from calibre.utils.imghdr import identify
-from calibre import guess_type, guess_all_extensions, prints, force_unicode
-from calibre.ebooks.metadata import MetaInformation, check_isbn
-from calibre.ebooks.chardet import xml_to_unicode
+from calibre.utils.xml_parse import safe_xml_fromstring
 from polyglot.binary import as_base64_unicode
 
-
 NAMESPACES = {
     'fb2'   :   'http://www.gribuser.ru/xml/fictionbook/2.0',
     'fb21'  :   'http://www.gribuser.ru/xml/fictionbook/2.1',
@@ -33,7 +32,7 @@ tostring = partial(etree.tostring, method='text', encoding='unicode')
 
 
 def XLINK(tag):
-    return '{%s}%s'%(NAMESPACES['xlink'], tag)
+    return '{{{}}}{}'.format(NAMESPACES['xlink'], tag)
 
 
 class Context:
@@ -74,7 +73,7 @@ class Context:
 
     def clear_meta_tags(self, doc, tag):
         for parent in ('title-info', 'src-title-info', 'publish-info'):
-            for x in self.XPath('//fb:%s/fb:%s'%(parent, tag))(doc):
+            for x in self.XPath(f'//fb:{parent}/fb:{tag}')(doc):
                 x.getparent().remove(x)
 
     def text2fb2(self, parent, text):
@@ -89,7 +88,7 @@ class Context:
 
 
 def get_fb2_data(stream):
-    from calibre.utils.zipfile import ZipFile, BadZipfile
+    from calibre.utils.zipfile import BadZipfile, ZipFile
     pos = stream.tell()
     try:
         zf = ZipFile(stream)
@@ -165,7 +164,7 @@ def _parse_authors(root, ctx):
     # Those are fallbacks: <src-title-info>, <document-info>
     author = None
     for author_sec in ['title-info', 'src-title-info', 'document-info']:
-        for au in ctx.XPath('//fb:%s/fb:author'%author_sec)(root):
+        for au in ctx.XPath(f'//fb:{author_sec}/fb:author')(root):
             author = _parse_author(au, ctx)
             if author:
                 authors.append(author)
@@ -180,7 +179,7 @@ def _parse_authors(root, ctx):
 
 
 def _parse_author(elm_author, ctx):
-    """ Returns a list of display author and sortable author"""
+    ''' Returns a list of display author and sortable author'''
 
     xp_templ = 'normalize-space(fb:%s/text())'
 
@@ -225,7 +224,7 @@ def _parse_cover(root, mi, ctx):
 
 def _parse_cover_data(root, imgid, mi, ctx):
     from calibre.ebooks.fb2 import base64_decode
-    elm_binary = ctx.XPath('//fb:binary[@id="%s"]'%imgid)(root)
+    elm_binary = ctx.XPath(f'//fb:binary[@id="{imgid}"]')(root)
     if elm_binary:
         mimetype = elm_binary[0].get('content-type', 'image/jpeg')
         mime_extensions = guess_all_extensions(mimetype)
@@ -250,7 +249,7 @@ def _parse_tags(root, mi, ctx):
     # Those are fallbacks: <src-title-info>
     for genre_sec in ['title-info', 'src-title-info']:
         # -- i18n Translations-- ?
-        tags = ctx.XPath('//fb:%s/fb:genre/text()' % genre_sec)(root)
+        tags = ctx.XPath(f'//fb:{genre_sec}/fb:genre/text()')(root)
         if tags:
             mi.tags = list(map(str, tags))
             break
@@ -287,7 +286,7 @@ def _parse_isbn(root, mi, ctx):
 def _parse_comments(root, mi, ctx):
     # pick up annotation but only from 1 section <title-info>;  fallback: <src-title-info>
     for annotation_sec in ['title-info', 'src-title-info']:
-        elms_annotation = ctx.XPath('//fb:%s/fb:annotation' % annotation_sec)(root)
+        elms_annotation = ctx.XPath(f'//fb:{annotation_sec}/fb:annotation')(root)
         if elms_annotation:
             mi.comments = tostring(elms_annotation[0])
             # TODO: tags i18n, xslt?
@@ -385,7 +384,7 @@ def _set_series(title_info, mi, ctx):
         seq = ctx.get_or_create(title_info, 'sequence')
         seq.set('name', mi.series)
         try:
-            seq.set('number', '%g'%mi.series_index)
+            seq.set('number', f'{mi.series_index:g}')
         except:
             seq.set('number', '1')
 
