@@ -1,11 +1,13 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.12
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>, John Howell <jhowell@acm.org>'
 
 
 # Based on work of John Howell reversing the KFX format
 # https://www.mobileread.com/forums/showpost.php?p=3176029&postcount=89
 
-import struct, sys, re
+import re
+import struct
+import sys
 from collections import defaultdict
 
 from calibre.ebooks.metadata.book.base import Metadata
@@ -13,8 +15,8 @@ from calibre.ebooks.mobi.utils import decint
 from calibre.utils.cleantext import clean_xml_chars
 from calibre.utils.config_base import tweaks
 from calibre.utils.date import parse_only_date
-from calibre.utils.localization import canonicalize_lang
 from calibre.utils.imghdr import identify
+from calibre.utils.localization import canonicalize_lang
 from polyglot.binary import as_base64_bytes, from_base64_bytes
 
 
@@ -48,14 +50,14 @@ PROP_METADATA_VALUE = b'P307'
 PROP_IMAGE = b'P417'
 
 METADATA_PROPERTIES = {
-    b'P10' : "languages",
-    b'P153': "title",
-    b'P154': "description",
-    b'P222': "author",
-    b'P232': "publisher",
+    b'P10' : 'languages',
+    b'P153': 'title',
+    b'P154': 'description',
+    b'P222': 'author',
+    b'P232': 'publisher',
 }
 
-COVER_KEY = "cover_image_base64"
+COVER_KEY = 'cover_image_base64'
 
 
 def hexs(string, sep=' '):
@@ -63,11 +65,10 @@ def hexs(string, sep=' '):
         string = bytearray(string)
     else:
         string = map(ord, string)
-    return sep.join('%02x' % b for b in string)
+    return sep.join(f'{b:02x}' for b in string)
 
 
 class PackedData:
-
     '''
     Simplify unpacking of packed binary data structures
     '''
@@ -99,7 +100,6 @@ class PackedData:
 
 
 class PackedBlock(PackedData):
-
     '''
     Common header structure of container and entity blocks
     '''
@@ -109,15 +109,13 @@ class PackedBlock(PackedData):
 
         self.magic = self.unpack_one('4s')
         if self.magic != magic:
-            raise InvalidKFX('%s magic number is incorrect (%s)' %
-                            (magic, hexs(self.magic)))
+            raise InvalidKFX(f'{magic} magic number is incorrect ({hexs(self.magic)})')
 
         self.version = self.unpack_one('<H')
         self.header_len = self.unpack_one('<L')
 
 
 class Container(PackedBlock):
-
     '''
     Container file containing data entities
     '''
@@ -141,7 +139,6 @@ class Container(PackedBlock):
 
 
 class Entity(PackedBlock):
-
     '''
     Data entity inside a container
     '''
@@ -162,7 +159,6 @@ class Entity(PackedBlock):
 
 
 class PackedIon(PackedData):
-
     '''
     Packed structured binary data format used by KFX
     '''
@@ -246,7 +242,7 @@ class PackedIon(PackedData):
 def property_name(property_number):
     # This should be changed to translate property numbers to the proper
     # strings using a symbol table
-    return b"P%d" % property_number
+    return b'P%d' % property_number
 
 
 def extract_metadata(container_data):
