@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.9
+#!/usr/local/bin/python3.13
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
 __license__   = 'GPL v3'
@@ -6,8 +6,8 @@ __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 import re
-import unicodedata
 import time
+import unicodedata
 from datetime import datetime
 from io import StringIO
 from operator import attrgetter
@@ -23,7 +23,7 @@ from calibre.utils.date import UNDEFINED_DATE, as_utc, utc_tz
 from calibre.utils.formatter import EvalFormatter
 from calibre.utils.html2text import html2text
 from calibre.utils.icu import lower, primary_sort_key
-from polyglot.builtins import iteritems, itervalues, as_unicode
+from polyglot.builtins import as_unicode, iteritems, itervalues
 from polyglot.queue import Empty, Queue
 from polyglot.urllib import quote, urlparse
 
@@ -62,8 +62,8 @@ def is_worker_alive(workers):
 
 # }}}
 
-# Merge results from different sources {{{
 
+# Merge results from different sources {{{
 
 class xISBN(Thread):
 
@@ -297,7 +297,7 @@ class ISBNMerge:
             if rating and rating > 0 and rating <= 5:
                 ratings.append(rating)
         if ratings:
-            ans.rating = int(round(sum(ratings)/len(ratings)))
+            ans.rating = round(sum(ratings)/len(ratings))
 
         # Smallest language is likely to be valid
         ans.language = self.length_merge('language', results,
@@ -501,7 +501,9 @@ def identify(log, abort,  # {{{
     log('We have %d merged results, merging took: %.2f seconds' %
             (len(results), time.time() - start_time))
     tm_rules = msprefs['tag_map_rules']
-    if tm_rules:
+    pm_rules = msprefs['publisher_map_rules']
+    s_rules = msprefs['series_map_rules']
+    if tm_rules or pm_rules or s_rules:
         from calibre.ebooks.metadata.tag_mapper import map_tags
     am_rules = msprefs['author_map_rules']
     if am_rules:
@@ -509,7 +511,8 @@ def identify(log, abort,  # {{{
         am_rules = compile_rules(am_rules)
 
     # normalize unicode strings
-    n = lambda x: unicodedata.normalize('NFC', as_unicode(x or '', errors='replace'))
+    def n(x):
+        return unicodedata.normalize('NFC', as_unicode(x or '', errors='replace'))
     for r in results:
         if r.tags:
             r.tags = list(map(n, r.tags))
@@ -530,6 +533,12 @@ def identify(log, abort,  # {{{
         r.tags = r.tags[:max_tags]
         if getattr(r.pubdate, 'year', 2000) <= UNDEFINED_DATE.year:
             r.pubdate = None
+        if pm_rules and r.publisher:
+            pubs = map_tags([r.publisher], pm_rules)
+            r.publisher = pubs[0] if pubs else ''
+        if s_rules and r.series:
+            ss = map_tags([r.series], s_rules)
+            r.series = ss[0] if ss else ''
 
     if msprefs['swap_author_names']:
         for r in results:
@@ -633,12 +642,9 @@ def urls_from_identifiers(identifiers, sort_results=False):  # {{{
 # }}}
 
 
-if __name__ == '__main__':  # tests {{{
-    # To run these test use: calibre-debug -e
-    # src/calibre/ebooks/metadata/sources/identify.py
-    from calibre.ebooks.metadata.sources.test import (
-        authors_test, test_identify, title_test
-    )
+def tests(start=0, limit=256):  # tests {{{
+    # To run these test use: calibre-debug -c "from calibre.ebooks.metadata.sources.identify import tests; tests()"
+    from calibre.ebooks.metadata.sources.test import authors_test, test_identify, title_test
     tests = [
             (
                 {'title':'Magykal Papers',
@@ -678,5 +684,5 @@ if __name__ == '__main__':  # tests {{{
 
         ]
     # test_identify(tests[1:2])
-    test_identify(tests)
+    test_identify(tests[start:limit])
 # }}}
