@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.12
 # vim:fileencoding=utf-8
 from __future__ import absolute_import, division, print_function, unicode_literals
 
@@ -18,8 +18,10 @@ msprefs.defaults['swap_author_names'] = False
 msprefs.defaults['fewer_tags'] = True
 msprefs.defaults['find_first_edition_date'] = False
 msprefs.defaults['append_comments'] = False
-msprefs.defaults['tag_map_rules'] = []
-msprefs.defaults['author_map_rules'] = []
+msprefs.defaults['tag_map_rules'] = ()
+msprefs.defaults['author_map_rules'] = ()
+msprefs.defaults['publisher_map_rules'] = ()
+msprefs.defaults['series_map_rules'] = ()
 msprefs.defaults['id_link_rules'] = {}
 msprefs.defaults['keep_dups'] = False
 
