@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.13
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2017, Kovid Goyal <kovid at kovidgoyal.net>
 
@@ -13,8 +13,9 @@ from threading import Thread
 
 import calibre.ebooks.metadata.sources.search_engines as builtin_search_engines
 from calibre import as_unicode, prints
-from calibre.constants import DEBUG, numeric_version
+from calibre.constants import numeric_version
 from calibre.ebooks.metadata.sources.base import Source
+from calibre.prints import debug_print
 from calibre.utils.config import JSONConfig
 from calibre.utils.https import get_https_resource_securely
 from polyglot.builtins import iteritems, itervalues
@@ -30,11 +31,6 @@ def search_engines_module():
     return current_search_engines
 
 
-def debug_print(*args, **k):
-    if DEBUG:
-        prints(*args, **k)
-
-
 def load_plugin(src):
     src = src.encode('utf-8')
     ns = {}
