@@ -1,17 +1,18 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import threading, re, json
+import json
+import re
+import threading
 
 from calibre import browser
 
 
 class xISBN:
-
     '''
     This class is used to find the ISBN numbers of "related" editions of a
     book, given its ISBN. Useful when querying services for metadata by ISBN,
@@ -101,7 +102,8 @@ class xISBN:
 xisbn = xISBN()
 
 if __name__ == '__main__':
-    import sys, pprint
+    import pprint
+    import sys
     isbn = sys.argv[-1]
     print(pprint.pprint(xisbn.get_data(isbn)))
     print()
