@@ -5,16 +5,18 @@ __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import struct, re, os
+import os
+import re
+import struct
 
 from calibre import replace_entities
-from calibre.utils.date import parse_date
-from calibre.ebooks.mobi import MobiError
 from calibre.ebooks.metadata import MetaInformation, check_isbn
-from calibre.ebooks.mobi.langcodes import main_language, sub_language, mobi2iana
+from calibre.ebooks.mobi import MobiError
+from calibre.ebooks.mobi.langcodes import main_language, mobi2iana, sub_language
 from calibre.utils.cleantext import clean_ascii_chars, clean_xml_chars
-from calibre.utils.localization import canonicalize_lang
 from calibre.utils.config_base import tweaks
+from calibre.utils.date import parse_date
+from calibre.utils.localization import canonicalize_lang
 
 NULL_INDEX = 0xffffffff
 
@@ -24,7 +26,7 @@ def uniq(vals):
     vals = vals or ()
     seen = set()
     seen_add = seen.add
-    return list(x for x in vals if x not in seen and not seen_add(x))
+    return [x for x in vals if x not in seen and not seen_add(x)]
 
 
 class EXTHHeader:  # {{{
@@ -43,7 +45,7 @@ class EXTHHeader:  # {{{
         self.page_progression_direction = None
         self.primary_writing_mode = None
 
-        self.decode = lambda x : clean_ascii_chars(x.decode(codec, 'replace'))
+        self.decode = lambda x: clean_ascii_chars(x.decode(codec, 'replace'))
 
         while left > 0:
             left -= 1
@@ -106,7 +108,7 @@ class EXTHHeader:  # {{{
                 except Exception:
                     pass
             # else:
-            #    print 'unknown record', idx, repr(content)
+            #     print('unknown record', idx, repr(content))
         if title:
             self.mi.title = replace_entities(clean_xml_chars(clean_ascii_chars(title)))
 
@@ -182,7 +184,7 @@ class EXTHHeader:  # {{{
             if self.kf8_header == NULL_INDEX:
                 self.kf8_header = None
         # else:
-        #    print 'unhandled metadata record', idx, repr(content)
+        #     print('unhandled metadata record', idx, repr(content))
 # }}}
 
 
@@ -218,8 +220,7 @@ class BookHeader:
                     }[self.codepage]
             except (IndexError, KeyError):
                 self.codec = 'cp1252' if not user_encoding else user_encoding
-                log.warn('Unknown codepage %d. Assuming %s' % (self.codepage,
-                    self.codec))
+                log.warn(f'Unknown codepage {self.codepage}. Assuming {self.codec}')
             # Some KF8 files have header length == 264 (generated by kindlegen
             # 2.9?). See https://bugs.launchpad.net/bugs/1179144
             max_header_length = 500  # We choose 500 for future versions of kindlegen
@@ -318,7 +319,7 @@ class MetadataHeader(BookHeader):
         self.stream.seek(60)
         ident = self.stream.read(8).upper()
         if ident not in (b'BOOKMOBI', b'TEXTREAD'):
-            raise MobiError('Unknown book type: %s' % ident)
+            raise MobiError(f'Unknown book type: {ident}')
         return ident
 
     def section_count(self):
