@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.9
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
@@ -8,8 +8,7 @@ __docformat__ = 'restructuredtext en'
 import struct
 from collections import OrderedDict, namedtuple
 
-from calibre.ebooks.mobi.utils import (decint, count_set_bits,
-        decode_string)
+from calibre.ebooks.mobi.utils import count_set_bits, decint, decode_string
 from polyglot.builtins import iteritems
 
 TagX = namedtuple('TagX', 'tag num_of_values bitmask eof')
@@ -17,7 +16,7 @@ PTagX = namedtuple('PTagX', 'tag value_count value_bytes num_of_values')
 INDEX_HEADER_FIELDS = (
             'len', 'nul1', 'type', 'gen', 'start', 'count', 'code',
             'lng', 'total', 'ordt', 'ligt', 'nligt', 'ncncx'
-    ) + tuple('unknown%d'%i for i in range(27)) + ('ocnt', 'oentries',
+    ) + tuple(f'unknown{i}' for i in range(27)) + ('ocnt', 'oentries',
             'ordt1', 'ordt2', 'tagx')
 
 
@@ -27,7 +26,7 @@ class InvalidFile(ValueError):
 
 def check_signature(data, signature):
     if data[:len(signature)] != signature:
-        raise InvalidFile('Not a valid %r section'%signature)
+        raise InvalidFile(f'Not a valid {signature!r} section')
 
 
 class NotAnINDXRecord(InvalidFile):
@@ -48,7 +47,7 @@ def parse_indx_header(data):
     check_signature(data, b'INDX')
     words = INDEX_HEADER_FIELDS
     num = len(words)
-    values = struct.unpack('>%dL' % num, data[4:4*(num+1)])
+    values = struct.unpack(f'>{num}L', data[4:4*(num+1)])
     ans = dict(zip(words, values))
     ans['idx_header_end_pos'] = 4 * (num+1)
     ordt1, ordt2 = ans['ordt1'], ans['ordt2']
@@ -104,8 +103,7 @@ class CNCX:  # {{{
                     except:
                         byts = raw[pos:]
                         r = format_bytes(byts)
-                        print('CNCX entry at offset %d has unknown format %s'%(
-                            pos+record_offset, r))
+                        print(f'CNCX entry at offset {pos + record_offset} has unknown format {r}')
                         self.records[pos+record_offset] = r
                         pos = len(raw)
                 pos += consumed+length
@@ -194,8 +192,7 @@ def get_tag_map(control_byte_count, tagx, data, strict=False):
                 total_consumed += consumed
                 values.append(byts)
             if total_consumed != x.value_bytes:
-                err = ("Error: Should consume %s bytes, but consumed %s" %
-                        (x.value_bytes, total_consumed))
+                err = (f'Error: Should consume {x.value_bytes} bytes, but consumed {total_consumed}')
                 if strict:
                     raise ValueError(err)
                 else:
@@ -203,8 +200,7 @@ def get_tag_map(control_byte_count, tagx, data, strict=False):
         ans[x.tag] = values
     # Test that all bytes have been processed
     if data.replace(b'\0', b''):
-        err = ("Warning: There are unprocessed index bytes left: %s" %
-                format_bytes(data))
+        err = (f'Warning: There are unprocessed index bytes left: {format_bytes(data)}')
         if strict:
             raise ValueError(err)
         else:
