@@ -1,23 +1,23 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, glob
+import glob
+import os
 
 from calibre import CurrentDir
-from calibre.ebooks.mobi import MobiError
-from calibre.ebooks.mobi.reader.mobi6 import MobiReader
-from calibre.ebooks.mobi.reader.headers import MetadataHeader
-from calibre.utils.logging import default_log
+from calibre.customize.ui import plugin_for_input_format, plugin_for_output_format
 from calibre.ebooks import DRMError
-from calibre.ebooks.mobi.reader.mobi8 import Mobi8Reader
 from calibre.ebooks.conversion.plumber import Plumber, create_oebbook
-from calibre.customize.ui import (plugin_for_input_format,
-        plugin_for_output_format)
+from calibre.ebooks.mobi import MobiError
+from calibre.ebooks.mobi.reader.headers import MetadataHeader
+from calibre.ebooks.mobi.reader.mobi6 import MobiReader
+from calibre.ebooks.mobi.reader.mobi8 import Mobi8Reader
 from calibre.utils.ipc.simple_worker import fork_job
+from calibre.utils.logging import default_log
 
 
 class BadFormat(ValueError):
@@ -98,11 +98,10 @@ def do_rebuild(opf, dest_path):
 def rebuild(src_dir, dest_path):
     opf = glob.glob(os.path.join(src_dir, '*.opf'))
     if not opf:
-        raise ValueError('No OPF file found in %s'%src_dir)
+        raise ValueError(f'No OPF file found in {src_dir}')
     opf = opf[0]
     # For debugging, uncomment the following two lines
     # def fork_job(a, b, args=None, no_output=True):
     #     do_rebuild(*args)
     fork_job('calibre.ebooks.mobi.tweak', 'do_rebuild', args=(opf, dest_path),
             no_output=True)
-
