@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
@@ -9,10 +9,10 @@ import re
 
 from lxml import etree, html
 
-from calibre import xml_replace_entities, force_unicode
-from calibre.utils.xml_parse import safe_xml_fromstring
+from calibre import force_unicode, xml_replace_entities
 from calibre.constants import filesystem_encoding
-from calibre.ebooks.chardet import xml_to_unicode, strip_encoding_declarations
+from calibre.ebooks.chardet import strip_encoding_declarations, xml_to_unicode
+from calibre.utils.xml_parse import safe_xml_fromstring
 from polyglot.builtins import iteritems, itervalues, string_or_bytes
 
 RECOVER_PARSER = etree.XMLParser(recover=True, no_network=True, resolve_entities=False)
@@ -27,12 +27,14 @@ class NotHTML(Exception):
         self.root_tag = root_tag
 
 
-def barename(name):
-    return name.rpartition('}')[-1]
+try:
+    from calibre_extensions.speedup import barename, namespace
+except ImportError:
+    def barename(name: str) -> str:
+        return name.rpartition('}')[-1]
 
-
-def namespace(name):
-    return name.rpartition('}')[0][1:]
+    def namespace(name: str) -> str:
+        return name.rpartition('}')[0][1:]
 
 
 def XHTML(name):
@@ -94,6 +96,7 @@ def node_depth(node):
 
 def html5_parse(data, max_nesting_depth=100):
     from html5_parser import parse
+
     from calibre.utils.cleantext import clean_xml_chars
     data = parse(clean_xml_chars(data), maybe_xhtml=True, keep_doctype=False, sanitize_names=True)
     # Check that the asinine HTML 5 algorithm did not result in a tree with
@@ -102,8 +105,7 @@ def html5_parse(data, max_nesting_depth=100):
         if isinstance(x.tag, string_or_bytes) and not len(x):  # Leaf node
             depth = node_depth(x)
             if depth > max_nesting_depth:
-                raise ValueError('HTML 5 parsing resulted in a tree with nesting'
-                        ' depth > %d'%max_nesting_depth)
+                raise ValueError(f'HTML 5 parsing resulted in a tree with nesting depth > {max_nesting_depth}')
     return data
 
 
@@ -129,11 +131,11 @@ def clean_word_doc(data, log):
         # but can become renderable HTML tags like <p/> if the
         # document is parsed by an HTML parser
         pat = re.compile(
-                r'<(%s):([a-zA-Z0-9]+)[^>/]*?></\1:\2>'%('|'.join(prefixes)),
+                r'<({}):([a-zA-Z0-9]+)[^>/]*?></\1:\2>'.format('|'.join(prefixes)),
                 re.DOTALL)
         data = pat.sub('', data)
         pat = re.compile(
-                r'<(%s):([a-zA-Z0-9]+)[^>/]*?/>'%('|'.join(prefixes)))
+                r'<({}):([a-zA-Z0-9]+)[^>/]*?/>'.format('|'.join(prefixes)))
         data = pat.sub('', data)
     return data
 
@@ -196,8 +198,8 @@ def parse_html(data, log=None, decoder=None, preprocessor=None,
                     val = val[1:-1]
                 user_entities[match.group(1)] = val
             if user_entities:
-                pat = re.compile(r'&(%s);'%('|'.join(list(user_entities.keys()))))
-                data = pat.sub(lambda m:user_entities[m.group(1)], data)
+                pat = re.compile(r'&({});'.format('|'.join(list(user_entities.keys()))))
+                data = pat.sub(lambda m: user_entities[m.group(1)], data)
 
     if preprocessor is not None:
         data = preprocessor(data)
@@ -218,7 +220,7 @@ def parse_html(data, log=None, decoder=None, preprocessor=None,
             data = safe_xml_fromstring(data, recover=False)
             check_for_html5(pre, data)
         except (HTML5Doc, etree.XMLSyntaxError):
-            log.debug('Parsing %s as HTML' % filename)
+            log.debug(f'Parsing {filename} as HTML')
             data = raw
             try:
                 data = html5_parse(data)
@@ -243,7 +245,7 @@ def parse_html(data, log=None, decoder=None, preprocessor=None,
     if barename(data.tag) != 'html':
         if barename(data.tag) in non_html_file_tags:
             raise NotHTML(data.tag)
-        log.warn('File %r does not appear to be (X)HTML'%filename)
+        log.warn(f'File {filename!r} does not appear to be (X)HTML')
         nroot = safe_xml_fromstring('<html></html>')
         has_body = False
         for child in list(data):
@@ -252,7 +254,7 @@ def parse_html(data, log=None, decoder=None, preprocessor=None,
                 break
         parent = nroot
         if not has_body:
-            log.warn('File %r appears to be a HTML fragment'%filename)
+            log.warn(f'File {filename!r} appears to be a HTML fragment')
             nroot = safe_xml_fromstring('<html><body/></html>')
             parent = nroot[0]
         for child in list(data.iter()):
@@ -276,8 +278,7 @@ def parse_html(data, log=None, decoder=None, preprocessor=None,
             try:
                 data = safe_xml_fromstring(data, recover=False)
             except etree.XMLSyntaxError:
-                log.warn('Stripping comments from %s'%
-                        filename)
+                log.warn(f'Stripping comments from {filename}')
                 data = re.compile(r'<!--.*?-->', re.DOTALL).sub('',
                         data)
                 data = data.replace(
@@ -287,7 +288,7 @@ def parse_html(data, log=None, decoder=None, preprocessor=None,
                 try:
                     data = safe_xml_fromstring(data)
                 except etree.XMLSyntaxError:
-                    log.warn('Stripping meta tags from %s'% filename)
+                    log.warn(f'Stripping meta tags from {filename}')
                     data = re.sub(r'<meta\s+[^>]+?>', '', data)
                     data = safe_xml_fromstring(data)
     elif namespace(data.tag) != XHTML_NS:
@@ -312,7 +313,7 @@ def parse_html(data, log=None, decoder=None, preprocessor=None,
     head = xpath(data, '/h:html/h:head')
     head = head[0] if head else None
     if head is None:
-        log.warn('File %s missing <head/> element' % filename)
+        log.warn(f'File {filename} missing <head/> element')
         head = etree.Element(XHTML('head'))
         data.insert(0, head)
         title = etree.SubElement(head, XHTML('title'))
@@ -339,7 +340,7 @@ def parse_html(data, log=None, decoder=None, preprocessor=None,
             body.getparent().remove(body)
             data.append(body)
         else:
-            log.warn('File %s missing <body/> element' % filename)
+            log.warn(f'File {filename} missing <body/> element')
             etree.SubElement(data, XHTML('body'))
 
     # Remove microsoft office markup
