@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.13
 
 
 __license__ = 'GPL v3'
@@ -10,14 +10,15 @@ from functools import partial
 from operator import itemgetter
 
 from css_parser.css import CSSRule, CSSStyleDeclaration
-from css_selectors import parse, SelectorSyntaxError
+from css_selectors import Select, SelectorError, SelectorSyntaxError, parse
 
 from calibre import force_unicode
-from calibre.ebooks.oeb.base import OEB_STYLES, OEB_DOCS, XHTML, css_text
+from calibre.ebooks.oeb.base import OEB_DOCS, OEB_STYLES, XHTML, css_text
 from calibre.ebooks.oeb.normalize_css import normalize_filter_css, normalizers
 from calibre.ebooks.oeb.polish.pretty import pretty_script_or_style, pretty_xml_tree, serialize
+from calibre.utils.icu import lower as icu_lower
 from calibre.utils.icu import numeric_sort_key
-from css_selectors import Select, SelectorError
+from calibre.utils.localization import ngettext
 from polyglot.builtins import iteritems, itervalues
 from polyglot.functools import lru_cache
 
