@@ -1,16 +1,16 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.12
 
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os, sys
+import os
+import sys
 
-from calibre.ptempfile import TemporaryDirectory
 from calibre.ebooks.conversion.plumber import Plumber
-from calibre.ebooks.oeb.polish.container import Container, OEB_DOCS, OEB_STYLES
 from calibre.ebooks.epub import initialize_container
-
+from calibre.ebooks.oeb.polish.container import OEB_DOCS, OEB_STYLES, Container
+from calibre.ptempfile import TemporaryDirectory
 from calibre.utils.logging import default_log
 from polyglot.builtins import iteritems
 
@@ -33,7 +33,7 @@ def auto_fill_manifest(container):
 
 def import_book_as_epub(srcpath, destpath, log=default_log):
     if not destpath.lower().endswith('.epub'):
-        raise ValueError('Can only import books into the EPUB format, not %s' % (os.path.basename(destpath)))
+        raise ValueError(f'Can only import books into the EPUB format, not {os.path.basename(destpath)}')
     with TemporaryDirectory('eei') as tdir:
         tdir = os.path.abspath(os.path.realpath(tdir))  # Needed to handle the multiple levels of symlinks for /tmp on OS X
         plumber = Plumber(srcpath, tdir, log)
