@@ -1,26 +1,29 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import re, sys, time
+import re
+import sys
+import time
 from collections import namedtuple
 from functools import partial
 
 from calibre.ebooks.oeb.polish.container import get_container
-from calibre.ebooks.oeb.polish.stats import StatsCollector
-from calibre.ebooks.oeb.polish.subset import subset_all_fonts, iter_subsettable_fonts
-from calibre.ebooks.oeb.polish.images import compress_images
-from calibre.ebooks.oeb.polish.upgrade import upgrade_book
-from calibre.ebooks.oeb.polish.embed import embed_all_fonts
 from calibre.ebooks.oeb.polish.cover import set_cover
-from calibre.ebooks.oeb.polish.replace import smarten_punctuation
-from calibre.ebooks.oeb.polish.jacket import (
-    replace_jacket, add_or_replace_jacket, find_existing_jacket, remove_jacket)
 from calibre.ebooks.oeb.polish.css import remove_unused_css
-from calibre.ebooks.oeb.polish.hyphenation import remove_soft_hyphens, add_soft_hyphens
+from calibre.ebooks.oeb.polish.download import download_external_resources, get_external_resources, replace_resources
+from calibre.ebooks.oeb.polish.embed import embed_all_fonts
+from calibre.ebooks.oeb.polish.hyphenation import add_soft_hyphens, remove_soft_hyphens
+from calibre.ebooks.oeb.polish.images import compress_images
+from calibre.ebooks.oeb.polish.jacket import add_or_replace_jacket, find_existing_jacket, remove_jacket, replace_jacket
+from calibre.ebooks.oeb.polish.replace import smarten_punctuation
+from calibre.ebooks.oeb.polish.stats import StatsCollector
+from calibre.ebooks.oeb.polish.subset import iter_subsettable_fonts, subset_all_fonts
+from calibre.ebooks.oeb.polish.upgrade import upgrade_book
+from calibre.utils.localization import ngettext
 from calibre.utils.logging import Log
 from polyglot.builtins import iteritems
 
@@ -37,6 +40,7 @@ ALL_OPTS = {
     'upgrade_book': False,
     'add_soft_hyphens': False,
     'remove_soft_hyphens': False,
+    'download_external_resources': False,
 }
 
 CUSTOMIZATION = {
@@ -47,7 +51,7 @@ CUSTOMIZATION = {
     'remove_ncx': True,
 }
 
-SUPPORTED = {'EPUB', 'AZW3'}
+SUPPORTED = {'EPUB', 'AZW3', 'KEPUB'}
 
 # Help {{{
 HELP = {'about': _(
@@ -63,7 +67,7 @@ changes needed for the desired effect.</p>
 <p>You should use this tool as the last step in your e-book creation process.</p>
 {0}
 <p>Note that polishing only works on files in the %s formats.</p>\
-''')%_(' or ').join(sorted('<b>%s</b>'%x for x in SUPPORTED)),
+''')%_(' or ').join(sorted(f'<b>{x}</b>' for x in SUPPORTED)),
 
 'embed': _('''\
 <p>Embed all fonts that are referenced in the document and are not already embedded.
@@ -132,6 +136,12 @@ better when the text is justified, in readers that do not support hyphenation.</
 'remove_soft_hyphens': _('''\
 <p>Remove soft hyphens from all text in the book.</p>
 '''),
+
+'download_external_resources': _('''\
+<p>Download external resources such as images, stylesheets, etc. that point to URLs instead of files in the book.
+All such resources will be downloaded and added to the book so that the book no longer references any external resources.
+</p>
+'''),
 }
 
 
@@ -145,7 +155,7 @@ def hfix(name, raw):
     return raw
 
 
-CLI_HELP = {x:hfix(x, re.sub('<.*?>', '', y)) for x, y in iteritems(HELP)}
+CLI_HELP = {x:hfix(x, re.sub(r'<.*?>', '', y)) for x, y in iteritems(HELP)}
 # }}}
 
 
@@ -160,8 +170,33 @@ def update_metadata(ebook, new_opf):
         stream.write(opfbytes)
 
 
+def download_resources(ebook, report) -> bool:
+    changed = False
+    url_to_referrer_map = get_external_resources(ebook)
+    if url_to_referrer_map:
+        n = len(url_to_referrer_map)
+        report(ngettext('Downloading one external resource', 'Downloading {} external resources', n).format(n))
+        replacements, failures = download_external_resources(ebook, url_to_referrer_map)
+        if not failures:
+            report(_('Successfully downloaded all resources'))
+        else:
+            tb = [f'{url}\n\t{err}\n' for url, err in iteritems(failures)]
+            if replacements:
+                report(_('Failed to download some resources, see details below:'))
+            else:
+                report(_('Failed to download all resources, see details below:'))
+            report(tb)
+        if replacements:
+            if replace_resources(ebook, url_to_referrer_map, replacements):
+                changed = True
+    else:
+        report(_('No external resources found in book'))
+    return changed
+
+
 def polish_one(ebook, opts, report, customization=None):
-    rt = lambda x: report('\n### ' + x)
+    def rt(x):
+        return report('\n### ' + x)
     jacket = None
     changed = False
     customization = customization or CUSTOMIZATION.copy()
@@ -265,6 +300,16 @@ def polish_one(ebook, opts, report, customization=None):
         add_soft_hyphens(ebook, report)
         changed = True
 
+    if opts.download_external_resources:
+        rt(_('Downloading external resources'))
+        try:
+            download_resources(ebook, report)
+        except Exception:
+            import traceback
+            report(_('Failed to download resources with error:'))
+            report(traceback.format_exc())
+        report('')
+
     return changed
 
 
@@ -335,6 +380,7 @@ def option_parser():
     o('--add-soft-hyphens', '-H', help=CLI_HELP['add_soft_hyphens'])
     o('--remove-soft-hyphens', help=CLI_HELP['remove_soft_hyphens'])
     o('--upgrade-book', '-U', help=CLI_HELP['upgrade_book'])
+    o('--download-external-resources', '-d', help=CLI_HELP['download_external_resources'])
 
     o('--verbose', help=_('Produce more verbose output, useful for debugging.'))
 
