@@ -1,20 +1,23 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import codecs, shutil, os, posixpath
-from polyglot.builtins import iteritems, itervalues
-from functools import partial
+import codecs
+import os
+import posixpath
+import shutil
 from collections import Counter, defaultdict
+from functools import partial
 
 from calibre import sanitize_file_name
 from calibre.ebooks.chardet import strip_encoding_declarations
 from calibre.ebooks.oeb.base import css_text
 from calibre.ebooks.oeb.polish.css import iter_declarations, remove_property_value
 from calibre.ebooks.oeb.polish.utils import extract
+from polyglot.builtins import iteritems, itervalues
 from polyglot.urllib import urlparse, urlunparse
 
 
@@ -45,7 +48,7 @@ class LinkReplacer:
         if purl.fragment:
             nfrag = self.frag_map(name, purl.fragment)
             if nfrag:
-                href += '#%s'%nfrag
+                href += f'#{nfrag}'
         if href != url:
             self.replaced = True
         return href
@@ -187,13 +190,13 @@ def rename_files(container, file_map):
     '''
     overlap = set(file_map).intersection(set(itervalues(file_map)))
     if overlap:
-        raise ValueError('Circular rename detected. The files %s are both rename targets and destinations' % ', '.join(overlap))
+        raise ValueError('Circular rename detected. The files {} are both rename targets and destinations'.format(', '.join(overlap)))
     for name, dest in iteritems(file_map):
         if container.exists(dest):
             if name != dest and name.lower() == dest.lower():
                 # A case change on an OS with a case insensitive file-system.
                 continue
-            raise ValueError('Cannot rename {0} to {1} as {1} already exists'.format(name, dest))
+            raise ValueError(f'Cannot rename {name} to {dest} as {dest} already exists')
     if len(tuple(itervalues(file_map))) != len(set(itervalues(file_map))):
         raise ValueError('Cannot rename, the set of destination files contains duplicates')
     link_map = {}
@@ -215,13 +218,13 @@ def replace_file(container, name, path, basename, force_mt=None):
             b, e = nname.rpartition('.')[0::2]
             while container.exists(nname):
                 count += 1
-                nname = b + ('_%d.%s' % (count, e))
+                nname = b + f'_{count}.{e}'
             rename_files(container, {name:nname})
             mt = force_mt or container.guess_type(nname)
             container.mime_map[nname] = mt
             for itemid, q in iteritems(container.manifest_id_map):
                 if q == nname:
-                    for item in container.opf_xpath('//opf:manifest/opf:item[@href and @id="%s"]' % itemid):
+                    for item in container.opf_xpath(f'//opf:manifest/opf:item[@href and @id="{itemid}"]'):
                         item.set('media-type', mt)
         container.dirty(container.opf_name)
         with container.open(nname, 'wb') as dest:
@@ -229,8 +232,8 @@ def replace_file(container, name, path, basename, force_mt=None):
 
 
 def mt_to_category(container, mt):
-    from calibre.ebooks.oeb.polish.utils import guess_type, OEB_FONTS
     from calibre.ebooks.oeb.base import OEB_DOCS, OEB_STYLES
+    from calibre.ebooks.oeb.polish.utils import OEB_FONTS, guess_type
     if mt in OEB_DOCS:
         category = 'text'
     elif mt in OEB_STYLES:
@@ -305,7 +308,7 @@ def rationalize_folders(container, folder_type_map):
                 while new_name in all_names or new_name in new_names:
                     c += 1
                     n, ext = bn.rpartition('.')[0::2]
-                    new_name = posixpath.join(folder, '%s_%d.%s' % (n, c, ext))
+                    new_name = posixpath.join(folder, f'{n}_{c}.{ext}')
                 name_map[name] = new_name
                 new_names.add(new_name)
     return name_map
@@ -345,7 +348,7 @@ def remove_links_in_declaration(href_to_name, style, predicate):
 def remove_links_to(container, predicate):
     ''' predicate must be a function that takes the arguments (name, href,
     fragment=None) and returns True iff the link should be removed '''
-    from calibre.ebooks.oeb.base import iterlinks, OEB_DOCS, OEB_STYLES, XPath, XHTML
+    from calibre.ebooks.oeb.base import OEB_DOCS, OEB_STYLES, XHTML, XPath, iterlinks
     stylepath = XPath('//h:style')
     styleattrpath = XPath('//*[@style]')
     changed = set()
