@@ -5,15 +5,15 @@ __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import sys
-from collections import defaultdict, Counter
+from collections import Counter, defaultdict
 
 from calibre import replace_entities
-from calibre.spell.break_iterator import split_into_words, index_of
-from calibre.spell.dictionary import parse_lang_code
 from calibre.ebooks.oeb.base import barename
 from calibre.ebooks.oeb.polish.container import OPF_NAMESPACES, get_container
 from calibre.ebooks.oeb.polish.parsing import parse
-from calibre.ebooks.oeb.polish.toc import find_existing_ncx_toc, find_existing_nav_toc
+from calibre.ebooks.oeb.polish.toc import find_existing_nav_toc, find_existing_ncx_toc
+from calibre.spell.break_iterator import index_of, split_into_words
+from calibre.spell.dictionary import parse_lang_code
 from calibre.utils.icu import ord_string
 from polyglot.builtins import iteritems
 
@@ -22,7 +22,7 @@ _patterns = None
 
 class Patterns:
 
-    __slots__ = ('sanitize_invisible_pat', 'split_pat', 'digit_pat', 'fr_elision_pat')
+    __slots__ = ('digit_pat', 'fr_elision_pat', 'sanitize_invisible_pat', 'split_pat')
 
     def __init__(self):
         import regex
@@ -36,7 +36,7 @@ class Patterns:
         # French words with prefixes are reduced to the stem word, so that the
         # words appear only once in the word list
         self.fr_elision_pat = regex.compile(
-            "^(?:l|d|m|t|s|j|c|ç|lorsqu|puisqu|quoiqu|qu)['’]", flags=regex.UNICODE | regex.VERSION1 | regex.IGNORECASE)
+            r"^(?:l|d|m|t|s|j|c|ç|lorsqu|puisqu|quoiqu|qu)['’]", flags=regex.UNICODE | regex.VERSION1 | regex.IGNORECASE)
 
 
 def patterns():
@@ -56,7 +56,7 @@ class CharCounter:
 
 class Location:
 
-    __slots__ = ('file_name', 'sourceline', 'original_word', 'location_node', 'node_item', 'elided_prefix')
+    __slots__ = ('elided_prefix', 'file_name', 'location_node', 'node_item', 'original_word', 'sourceline')
 
     def __init__(self, file_name=None, elided_prefix='', original_word=None, location_node=None, node_item=(None, None)):
         self.file_name, self.elided_prefix, self.original_word = file_name, elided_prefix, original_word
@@ -70,6 +70,9 @@ class Location:
         self.original_word = self.elided_prefix + new_word
 
 
+file_word_count = 0
+
+
 def filter_words(word):
     if not word:
         return False
@@ -80,10 +83,12 @@ def filter_words(word):
 
 
 def get_words(text, lang):
+    global file_word_count
     try:
         ans = split_into_words(str(text), lang)
     except (TypeError, ValueError):
         return ()
+    file_word_count += len(ans)
     return list(filter(filter_words, ans))
 
 
@@ -137,7 +142,7 @@ def count_chars_in_text(node, attr, counter, file_name, locale):
 
 def add_words_from_escaped_html(text, words, file_name, node, attr, locale):
     text = replace_entities(text)
-    root = parse('<html><body><div>%s</div></body></html>' % text, decoder=lambda x:x.decode('utf-8'))
+    root = parse(f'<html><body><div>{text}</div></body></html>', decoder=lambda x:x.decode('utf-8'))
     ewords = defaultdict(list)
     ewords[None] = 0
     read_words_from_html(root, ewords, file_name, locale)
@@ -150,17 +155,17 @@ def add_words_from_escaped_html(text, words, file_name, node, attr, locale):
 
 def count_chars_in_escaped_html(text, counter, file_name, node, attr, locale):
     text = replace_entities(text)
-    root = parse('<html><body><div>%s</div></body></html>' % text, decoder=lambda x:x.decode('utf-8'))
+    root = parse(f'<html><body><div>{text}</div></body></html>', decoder=lambda x:x.decode('utf-8'))
     count_chars_in_html(root, counter, file_name, locale)
 
 
-_opf_file_as = '{%s}file-as' % OPF_NAMESPACES['opf']
+_opf_file_as = '{{{}}}file-as'.format(OPF_NAMESPACES['opf'])
 opf_spell_tags = {'title', 'creator', 'subject', 'description', 'publisher'}
 
+
 # We can only use barename() for tag names and simple attribute checks so that
 # this code matches up with the syntax highlighter base spell checking
 
-
 def read_words_from_opf(root, words, file_name, book_locale):
     for tag in root.iterdescendants('*'):
         if barename(tag.tag) in opf_spell_tags:
@@ -275,7 +280,7 @@ def group_sort(locations):
     for loc in locations:
         if loc.file_name not in order:
             order[loc.file_name] = len(order)
-    return sorted(locations, key=lambda l:(order[l.file_name], l.sourceline))
+    return sorted(locations, key=lambda l:(order[l.file_name], l.sourceline or 0))
 
 
 def get_checkable_file_names(container):
@@ -299,7 +304,10 @@ def root_is_excluded_from_spell_check(root):
     return False
 
 
-def get_all_words(container, book_locale, get_word_count=False, excluded_files=()):
+def get_all_words(container, book_locale, get_word_count=False, excluded_files=(), file_words_counts=None):
+    global file_word_count
+    if file_words_counts is None:
+        file_words_counts = {}
     words = defaultdict(list)
     words[None] = 0
     file_names, ncx_toc = get_checkable_file_names(container)
@@ -309,12 +317,15 @@ def get_all_words(container, book_locale, get_word_count=False, excluded_files=(
         root = container.parsed(file_name)
         if root_is_excluded_from_spell_check(root):
             continue
+        file_word_count = 0
         if file_name == container.opf_name:
             read_words_from_opf(root, words, file_name, book_locale)
         elif file_name == ncx_toc:
             read_words_from_ncx(root, words, file_name, book_locale)
         elif hasattr(root, 'xpath'):
             read_words_from_html(root, words, file_name, book_locale)
+        file_words_counts[file_name] = file_word_count
+        file_word_count = 0
     count = words.pop(None)
     ans = {k:group_sort(v) for k, v in iteritems(words)}
     if get_word_count:
@@ -393,7 +404,8 @@ def undo_replace_word(container, undo_cache):
 
 if __name__ == '__main__':
     import pprint
-    from calibre.gui2.tweak_book import set_book_locale, dictionaries
+
+    from calibre.gui2.tweak_book import dictionaries, set_book_locale
     container = get_container(sys.argv[-1], tweak_mode=True)
     set_book_locale(container.mi.language)
     pprint.pprint(get_all_words(container, dictionaries.default_locale))
