@@ -1,16 +1,19 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.13
 
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os, unittest, shutil
+import os
+import shutil
+import unittest
 
-from calibre import CurrentDir
-from calibre.ptempfile import TemporaryDirectory
-from calibre.ptempfile import PersistentTemporaryDirectory
-from calibre.utils.logging import DevNull
 import calibre.ebooks.oeb.polish.container as pc
+from calibre import CurrentDir
+from calibre.ptempfile import PersistentTemporaryDirectory, TemporaryDirectory
+from calibre.utils.logging import DevNull
+from calibre.utils.resources import get_image_path as I
+from calibre.utils.resources import get_path as P
 from polyglot.builtins import iteritems
 
 
@@ -22,7 +25,14 @@ def get_cache():
     return cache
 
 
+once_per_run = set()
+
+
 def needs_recompile(obj, srcs):
+    is_ci = os.environ.get('CI', '').lower() == 'true'
+    if is_ci and obj not in once_per_run:
+        once_per_run.add(obj)
+        return True
     if isinstance(srcs, str):
         srcs = [srcs]
     try:
@@ -37,7 +47,7 @@ def needs_recompile(obj, srcs):
 
 def build_book(src, dest, args=()):
     from calibre.ebooks.conversion.cli import main
-    main(['ebook-convert', src, dest] + list(args))
+    main(['ebook-convert', src, dest, '-vv'] + list(args))
 
 
 def add_resources(raw, rmap):
@@ -48,27 +58,31 @@ def add_resources(raw, rmap):
     return raw
 
 
+def setup_simple_book(src):
+    with open(src, 'rb') as sf:
+        raw = sf.read().decode('utf-8')
+    raw = add_resources(raw, {
+        'LMONOI': P('fonts/liberation/LiberationMono-Italic.ttf'),
+        'LMONOR': P('fonts/liberation/LiberationMono-Regular.ttf'),
+        'IMAGE1': I('marked.png'),
+        'IMAGE2': I('textures/light_wood.png'),
+    })
+    shutil.copy2(I('lt.png'), '.')
+    x = 'index.html'
+    with open(x, 'wb') as f:
+        f.write(raw.encode('utf-8'))
+    return x
+
+
 def get_simple_book(fmt='epub'):
     cache = get_cache()
     ans = os.path.join(cache, 'simple.'+fmt)
     src = os.path.join(os.path.dirname(__file__), 'simple.html')
     if needs_recompile(ans, src):
-        with TemporaryDirectory('bpt') as tdir:
-            with CurrentDir(tdir):
-                with lopen(src, 'rb') as sf:
-                    raw = sf.read().decode('utf-8')
-                raw = add_resources(raw, {
-                    'LMONOI': P('fonts/liberation/LiberationMono-Italic.ttf'),
-                    'LMONOR': P('fonts/liberation/LiberationMono-Regular.ttf'),
-                    'IMAGE1': I('marked.png'),
-                    'IMAGE2': I('textures/light_wood.png'),
-                })
-                shutil.copy2(I('lt.png'), '.')
-                x = 'index.html'
-                with lopen(x, 'wb') as f:
-                    f.write(raw.encode('utf-8'))
-                build_book(x, ans, args=[
-                    '--level1-toc=//h:h2', '--language=en', '--authors=Kovid Goyal', '--cover=lt.png'])
+        with TemporaryDirectory('bpt') as tdir, CurrentDir(tdir):
+            x = setup_simple_book(src)
+            build_book(x, ans, args=[
+                '--level1-toc=//h:h2', '--language=en', '--authors=Kovid Goyal', '--cover=lt.png'])
     return ans
 
 
@@ -78,9 +92,10 @@ def get_split_book(fmt='epub'):
     src = os.path.join(os.path.dirname(__file__), 'split.html')
     if needs_recompile(ans, src):
         x = src.replace('split.html', 'index.html')
-        raw = lopen(src, 'rb').read().decode('utf-8')
+        with open(src, 'rb') as sf:
+            raw = sf.read().decode('utf-8')
         try:
-            with lopen(x, 'wb') as f:
+            with open(x, 'wb') as f:
                 f.write(raw.encode('utf-8'))
             build_book(x, ans, args=['--level1-toc=//h:h2', '--language=en', '--authors=Kovid Goyal',
                                         '--cover=' + I('lt.png')])
@@ -89,6 +104,29 @@ def get_split_book(fmt='epub'):
     return ans
 
 
+def get_book_for_kepubify(has_cover=True, epub_version='3'):
+    cache = get_cache()
+    ans = os.path.join(cache, f'kepubify-{has_cover}-{epub_version}.epub')
+    src = os.path.join(os.path.dirname(__file__), 'simple.html')
+    if needs_recompile(ans, src):
+        with TemporaryDirectory('bpt') as tdir, CurrentDir(tdir):
+            index_html = setup_simple_book(src)
+            args = ['--level1-toc=//h:h2', '--language=en', '--authors=Kovid Goyal', f'--epub-version={epub_version}']
+            if has_cover:
+                args.append('--cover=lt.png')
+            else:
+                args.append('--no-default-epub-cover')
+            build_book(index_html, ans, args=args)
+    c = pc.get_container(ans)
+    with c.open('page_styles.css', 'r+') as f:
+        css = f.read()
+        css += '\n\ndiv { widows: 13; orphans: 12; color: red; }'
+        f.seek(0), f.truncate(), f.write(css)
+    c.commit()
+
+    return ans
+
+
 devnull = DevNull()
 
 
