@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
@@ -17,13 +17,13 @@ from calibre.constants import iswindows
 from calibre.ebooks.chardet import strip_encoding_declarations
 from calibre.ebooks.metadata import fmt_sidx, rating_to_stars
 from calibre.ebooks.metadata.sources.identify import urls_from_identifiers
-from calibre.ebooks.oeb.base import (
-    XHTML, XHTML_NS, XPath, urldefrag, urlnormalize, xml2text
-)
+from calibre.ebooks.oeb.base import XHTML, XHTML_NS, XPath, urldefrag, urlnormalize, xml2text
 from calibre.library.comments import comments_to_html, markdown
 from calibre.utils.config import tweaks
 from calibre.utils.date import as_local_time, format_date, is_date_undefined
 from calibre.utils.icu import sort_key
+from calibre.utils.localization import ngettext
+from calibre.utils.resources import get_path as P
 
 JACKET_XPATH = '//h:meta[@name="calibre-content" and @content="jacket"]'
 
@@ -73,7 +73,7 @@ class RemoveFirstImage(Base):
                     raw = xml2text(body[0]).strip()
                     imgs = XPath('//h:img|//svg:svg')(item.data)
                     if not raw and not imgs:
-                        self.log('Removing %s as it has no content'%item.href)
+                        self.log(f'Removing {item.href} as it has no content')
                         self.oeb.manifest.remove(item)
                         deleted_item = item
                 break
@@ -134,7 +134,7 @@ class Jacket(Base):
         self.oeb.spine.insert(0, jacket, True)
         self.oeb.inserted_metadata_jacket = jacket
         for img, path in referenced_images(root):
-            self.oeb.log('Embedding referenced image %s into jacket' % path)
+            self.oeb.log(f'Embedding referenced image {path} into jacket')
             ext = path.rpartition('.')[-1].lower()
             item_id, href = self.oeb.manifest.generate('jacket_image', 'jacket_img.'+ext)
             with open(path, 'rb') as f:
@@ -160,8 +160,8 @@ class Jacket(Base):
         if opts.insert_metadata:
             self.insert_metadata(metadata)
 
-# Render Jacket {{{
 
+# Render Jacket {{{
 
 def get_rating(rating, rchar, e_rchar):
     ans = ''
@@ -174,7 +174,7 @@ def get_rating(rating, rchar, e_rchar):
     if num < 1:
         return ans
 
-    ans = ("%s%s") % (rchar * int(num), e_rchar * (5 - int(num)))
+    ans = (f'{rchar * int(num)}{e_rchar * (5 - int(num))}')
     return ans
 
 
@@ -293,7 +293,7 @@ def render_jacket(mi, output_profile,
     except:
         title_str = _('Unknown')
     title_str = escape(title_str)
-    title = '<span class="title">%s</span>' % title_str
+    title = f'<span class="title">{title_str}</span>'
 
     series = Series(mi.series, mi.series_index)
     try:
@@ -332,23 +332,24 @@ def render_jacket(mi, output_profile,
 
     def generate_html(comments):
         display = Attributes()
-        args = dict(xmlns=XHTML_NS,
-            title_str=title_str,
-            identifiers=Identifiers(mi.identifiers),
-            css=css,
-            title=title,
-            author=author,
-            publisher=publisher, publisher_label=_('Publisher'),
-            pubdate_label=_('Published'), pubdate=Timestamp(pubdate, tweaks['gui_pubdate_display_format']),
-            series_label=ngettext('Series', 'Series', 1), series=series,
-            rating_label=_('Rating'), rating=rating,
-            tags_label=_('Tags'), tags=tags,
-            timestamp=Timestamp(timestamp, tweaks['gui_timestamp_display_format']), timestamp_label=_('Date'),
-            comments=comments,
-            footer='',
-            display=display,
-            searchable_tags=' '.join(escape(t)+'ttt' for t in tags.tags_list),
-        )
+        args = {
+            'xmlns': XHTML_NS,
+            'title_str': title_str,
+            'identifiers': Identifiers(mi.identifiers),
+            'css': css,
+            'title': title,
+            'author': author,
+            'publisher': publisher, 'publisher_label': _('Publisher'),
+            'pubdate_label': _('Published'), 'pubdate': Timestamp(pubdate, tweaks['gui_pubdate_display_format']),
+            'series_label': ngettext('Series', 'Series', 1), 'series': series,
+            'rating_label': _('Rating'), 'rating': rating,
+            'tags_label': _('Tags'), 'tags': tags,
+            'timestamp': Timestamp(timestamp, tweaks['gui_timestamp_display_format']), 'timestamp_label': _('Date'),
+            'comments': comments,
+            'footer': '',
+            'display': display,
+            'searchable_tags': ' '.join(escape(t)+'ttt' for t in tags.tags_list),
+        }
         for key in mi.custom_field_keys():
             m = mi.get_user_metadata(key, False) or {}
             try:
@@ -365,14 +366,21 @@ def render_jacket(mi, output_profile,
                     val = val or ''
                     ctype = m.get('display', {}).get('interpret_as') or 'html'
                     if ctype == 'long-text':
-                        val = '<pre style="white-space:pre-wrap">%s</pre>' % escape(val)
+                        val = f'<pre style="white-space:pre-wrap">{escape(val)}</pre>'
                     elif ctype == 'short-text':
-                        val = '<span>%s</span>' % escape(val)
+                        val = f'<span>{escape(val)}</span>'
                     elif ctype == 'markdown':
                         val = markdown(val)
                     else:
                         val = comments_to_html(val)
                     args[dkey] = val
+                elif dt == 'composite':
+                    val = val or ''
+                    # if the column is marked as containing html, use it
+                    # unchanged. Otherwise treat it as a comment.
+                    if not m.get('display', {}).get('contains_html', False):
+                        val = comments_to_html(val)
+                    args[dkey] = val
                 else:
                     args[dkey] = escape(val)
                 args[dkey+'_label'] = escape(display_name)
@@ -382,10 +390,10 @@ def render_jacket(mi, output_profile,
                 pass
 
         if False:
-            print("Custom column values available in jacket template:")
+            print('Custom column values available in jacket template:')
             for key in args.keys():
                 if key.startswith('_') and not key.endswith('_label'):
-                    print(" {}: {}".format('#' + key[1:], args[key]))
+                    print(' {}: {}'.format('#' + key[1:], args[key]))
 
         # Used in the comment describing use of custom columns in templates
         # Don't change this unless you also change it in template.xhtml
