@@ -1,11 +1,11 @@
-#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-from calibre.ebooks.oeb.base import OEB_DOCS, XPath, XHTML
+from calibre.ebooks.oeb.base import OEB_DOCS, XHTML, XPath
 
 
 class LinearizeTables:
