@@ -1,17 +1,18 @@
-#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import re, uuid
+import re
+import uuid
+from collections import Counter, OrderedDict
 
 from lxml import etree
-from collections import OrderedDict, Counter
 
-from calibre.ebooks.oeb.base import XPNSMAP, TOC, XHTML, xml2text, barename
 from calibre.ebooks import ConversionError
+from calibre.ebooks.oeb.base import TOC, XHTML, XPNSMAP, barename, xml2text
 from polyglot.builtins import itervalues
 from polyglot.urllib import urlparse
 
@@ -21,7 +22,7 @@ def XPath(x):
         return etree.XPath(x, namespaces=XPNSMAP)
     except etree.XPathSyntaxError:
         raise ConversionError(
-        'The syntax of the XPath expression %s is invalid.' % repr(x))
+        f'The syntax of the XPath expression {x!r} is invalid.')
 
 
 def isspace(x):
@@ -68,8 +69,7 @@ class DetectStructure:
                 self.oeb.toc = orig_toc
             else:
                 self.oeb.auto_generated_toc = True
-                self.log('Auto generated TOC with %d entries.' %
-                        self.oeb.toc.count())
+                self.log(f'Auto generated TOC with {self.oeb.toc.count()} entries.')
 
         if opts.toc_filter is not None:
             regexp = re.compile(opts.toc_filter)
@@ -113,7 +113,7 @@ class DetectStructure:
             expr = XPath(expr)
         except:
             self.log.warn(
-                'Invalid start reading at XPath expression, ignoring: %s'%expr)
+                f'Invalid start reading at XPath expression, ignoring: {expr}')
             return
         for item in self.oeb.spine:
             if not hasattr(item.data, 'xpath'):
@@ -128,11 +128,9 @@ class DetectStructure:
                 if 'text' in self.oeb.guide:
                     self.oeb.guide.remove('text')
                 self.oeb.guide.add('text', 'Start', item.href+'#'+eid)
-                self.log('Setting start reading at position to %s in %s'%(
-                    self.opts.start_reading_at, item.href))
+                self.log(f'Setting start reading at position to {self.opts.start_reading_at} in {item.href}')
                 return
-        self.log.warn("Failed to find start reading at position: %s"%
-                self.opts.start_reading_at)
+        self.log.warn(f'Failed to find start reading at position: {self.opts.start_reading_at}')
 
     def get_toc_parts_for_xpath(self, expr):
         # if an attribute is selected by the xpath expr then truncate it
@@ -154,7 +152,7 @@ class DetectStructure:
                 len(ans)
                 return ans
             except:
-                self.log.warn('Invalid chapter expression, ignoring: %s'%expr)
+                self.log.warn(f'Invalid chapter expression, ignoring: {expr}')
                 return []
 
         if self.opts.chapter:
@@ -231,7 +229,7 @@ class DetectStructure:
                                 play_order=self.oeb.toc.next_play_order())
                             num += 1
                         except ValueError:
-                            self.oeb.log.exception('Failed to process link: %r' % href)
+                            self.oeb.log.exception(f'Failed to process link: {href!r}')
                             continue  # Most likely an incorrectly URL encoded link
                         if self.opts.max_toc_links > 0 and \
                                 num >= self.opts.max_toc_links:
@@ -250,7 +248,7 @@ class DetectStructure:
             text = elem.get('alt', '')
         text = re.sub(r'\s+', ' ', text.strip())
         text = text[:1000].strip()
-        id = elem.get('id', 'calibre_toc_%d'%counter)
+        id = elem.get('id', f'calibre_toc_{counter}')
         elem.set('id', id)
         href = '#'.join((item.href, id))
         return text, href
@@ -266,7 +264,7 @@ class DetectStructure:
                 len(ans)
                 return ans
             except:
-                self.log.warn('Invalid ToC expression, ignoring: %s'%expr)
+                self.log.warn(f'Invalid ToC expression, ignoring: {expr}')
                 return []
 
         for document in self.oeb.spine:
