@@ -1,17 +1,22 @@
-#!/usr/local/bin/python3.9
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
+import os
 from collections import defaultdict
+from io import BytesIO
 
-from calibre.ebooks.oeb.base import urlnormalize, css_text
-from calibre.utils.fonts.sfnt.subset import subset, NoGlyphs, UnsupportedFont
-from polyglot.builtins import iteritems, itervalues
 from tinycss.fonts3 import parse_font_family
 
+from calibre.ebooks.oeb.base import css_text, urlnormalize
+from calibre.utils.fonts.subset import subset
+from polyglot.builtins import iteritems
+
+font_properties = ('font-family', 'src', 'font-weight', 'font-stretch', 'font-style', 'text-transform')
+
 
 def get_font_properties(rule, default=None):
     '''
@@ -21,8 +26,7 @@ def get_font_properties(rule, default=None):
     '''
     props = {}
     s = rule.style
-    for q in ('font-family', 'src', 'font-weight', 'font-stretch',
-            'font-style'):
+    for q in font_properties:
         g = 'uri' if q == 'src' else 'value'
         try:
             val = s.getProperty(q).propertyValue[0]
@@ -119,7 +123,6 @@ def elem_style(style_rules, cls, inherited_style):
 
 
 class SubsetFonts:
-
     '''
     Subset all embedded fonts. Must be run after CSS flattening, as it requires
     CSS normalization and flattening to work.
@@ -150,37 +153,33 @@ class SubsetFonts:
             else:
                 fonts[item.href] = font
 
-        for font in itervalues(fonts):
+        for font in fonts.values():
             if not font['chars']:
-                self.log('The font %s is unused. Removing it.'%font['src'])
+                self.log('The font {} is unused. Removing it.'.format(font['src']))
                 remove(font)
                 continue
+            old_raw = font['item'].data
+            output = BytesIO()
+            font_type = os.path.splitext(font['item'].href)[1][1:].lower()
             try:
-                raw, old_stats, new_stats = subset(font['item'].data, font['chars'])
-            except NoGlyphs:
-                self.log('The font %s has no used glyphs. Removing it.'%font['src'])
-                remove(font)
-                continue
-            except UnsupportedFont as e:
-                self.log.warn('The font %s is unsupported for subsetting. %s'%(
-                    font['src'], e))
+                subset(BytesIO(old_raw), output, font_type, font['chars'])
+            except Exception as e:
+                self.log.warn('The font {} is unsupported for subsetting. {}'.format(font['src'], e))
                 sz = len(font['item'].data)
                 totals[0] += sz
                 totals[1] += sz
             else:
-                font['item'].data = raw
-                nlen = sum(itervalues(new_stats))
-                olen = sum(itervalues(old_stats))
-                self.log('Decreased the font %s to %.1f%% of its original size'%
-                        (font['src'], nlen/olen *100))
+                font['item'].data = output.getvalue()
+                nlen = len(font['item'].data)
+                olen = len(old_raw)
+                self.log('Decreased the font {} to {:.1f}% of its original size'.format(font['src'], nlen/olen *100))
                 totals[0] += nlen
                 totals[1] += olen
 
             font['item'].unload_data_from_memory()
 
         if totals[0]:
-            self.log('Reduced total font size to %.1f%% of original'%
-                    (totals[0]/totals[1] * 100))
+            self.log(f'Reduced total font size to {totals[0]/totals[1]*100:.1f}% of original')
 
     def find_embedded_fonts(self):
         '''
@@ -188,7 +187,10 @@ class SubsetFonts:
         '''
         self.embedded_fonts = []
         for item in self.oeb.manifest:
-            if not hasattr(item.data, 'cssRules'):
+            try:
+                if not hasattr(item.data, 'cssRules'):
+                    continue
+            except FileNotFoundError:
                 continue
             self.embedded_fonts.extend(find_font_face_rules(item, self.oeb))
 
@@ -295,13 +297,20 @@ class SubsetFonts:
             if matches:
                 return matches[0]
 
-    def find_chars(self, elem):
+    def find_chars(self, elem, style):
         ans = set()
+        transform = lambda x: x  # noqa: E731
+        tt = style.get('text-transform')
+        if tt:
+            if tt in ('uppercase', 'capitalize'):
+                transform = str.upper
+            elif tt == 'lowercase':
+                transform = str.lower
         if elem.text:
-            ans |= set(elem.text)
+            ans |= set(transform(elem.text))
         for child in elem:
             if child.tail:
-                ans |= set(child.tail)
+                ans |= set(transform(child.tail))
         return ans
 
     def find_usage_in(self, elem, inherited_style):
@@ -310,6 +319,6 @@ class SubsetFonts:
             self.find_usage_in(child, style)
         font = self.used_font(style)
         if font:
-            chars = self.find_chars(elem)
+            chars = self.find_chars(elem, style)
             if chars:
                 font['chars'] |= chars
