@@ -1,10 +1,10 @@
-#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.13
 # License: GPL v3 Copyright: 2019, Kovid Goyal <kovid at kovidgoyal.net>
 
 
 import sys
 
-from qt.core import QApplication, QUrl, QPageLayout, QPageSize, QMarginsF
+from qt.core import QApplication, QMarginsF, QPageLayout, QPageSize, QUrl
 from qt.webengine import QWebEnginePage
 
 from calibre.gui2 import load_builtin_fonts, must_use_qt
