@@ -2,36 +2,35 @@
 # License: GPL v3 Copyright: 2019, Kovid Goyal <kovid at kovidgoyal.net>
 
 # Imports {{{
-
-
 import copy
 import json
 import os
 import signal
 import sys
 from collections import namedtuple
-from html5_parser import parse
-from io import BytesIO
+from functools import lru_cache
 from itertools import count, repeat
-from qt.core import (
-    QApplication, QMarginsF, QObject, QPageLayout, Qt, QTimer, QUrl, pyqtSignal, sip
-)
+
+from html5_parser import parse
+from qt.core import QApplication, QByteArray, QMarginsF, QObject, QPageLayout, Qt, QTimer, QUrl, pyqtSignal, sip
 from qt.webengine import (
-    QWebEnginePage, QWebEngineProfile, QWebEngineUrlRequestInterceptor
+    QWebEnginePage,
+    QWebEngineProfile,
+    QWebEngineSettings,
+    QWebEngineUrlRequestInterceptor,
+    QWebEngineUrlRequestJob,
+    QWebEngineUrlSchemeHandler,
 )
 
 from calibre import detect_ncpus, human_readable, prepare_string_for_xml
-from calibre.constants import __version__, iswindows, ismacos
+from calibre.constants import FAKE_HOST, FAKE_PROTOCOL, __appname__, __version__, ismacos, iswindows
 from calibre.ebooks.metadata.xmp import metadata_to_xmp_packet
 from calibre.ebooks.oeb.base import XHTML, XPath
 from calibre.ebooks.oeb.polish.container import Container as ContainerBase
 from calibre.ebooks.oeb.polish.toc import get_toc
-from calibre.ebooks.pdf.image_writer import (
-    Image, PDFMetadata, draw_image_page, get_page_layout
-)
-from calibre.ebooks.pdf.render.serialize import PDFStream
+from calibre.ebooks.oeb.polish.utils import guess_type
+from calibre.ebooks.pdf.image_writer import PDFMetadata, get_page_layout
 from calibre.gui2 import setup_unix_signals
-from calibre.gui2.webengine import secure_webengine
 from calibre.srv.render_book import check_for_maths
 from calibre.utils.fonts.sfnt.container import Sfnt, UnsupportedFont
 from calibre.utils.fonts.sfnt.errors import NoGlyphs
@@ -39,17 +38,18 @@ from calibre.utils.fonts.sfnt.merge import merge_truetype_fonts_for_pdf
 from calibre.utils.fonts.sfnt.subset import pdf_subset
 from calibre.utils.logging import default_log
 from calibre.utils.monotonic import monotonic
-from calibre.utils.podofo import (
-    dedup_type3_fonts, get_podofo, remove_unused_fonts, set_metadata_implementation
-)
+from calibre.utils.podofo import add_image_page, dedup_type3_fonts, get_podofo, remove_unused_fonts, set_metadata_implementation
+from calibre.utils.resources import get_path as P
 from calibre.utils.short_uuid import uuid4
-from polyglot.builtins import iteritems
+from calibre.utils.webengine import secure_webengine, send_reply, setup_profile
+from polyglot.builtins import as_bytes, iteritems
 from polyglot.urllib import urlparse
 
-OK, KILL_SIGNAL = range(0, 2)
-HANG_TIME = 60  # seconds
 # }}}
 
+OK, KILL_SIGNAL = range(2)
+HANG_TIME = 60  # seconds
+
 
 # Utils {{{
 def data_as_pdf_doc(data):
@@ -74,7 +74,15 @@ def create_skeleton(container):
     spine_name = tuple(container.spine_names)[-1][0]
     root = container.parsed(spine_name)
     root = copy.deepcopy(root)
-    body = last_tag(root)
+    body = None
+    for child in tuple(root.iterchildren('*')):
+        if body is None:
+            if child.tag == XHTML('body') or child.tag == 'body':
+                body = child
+        else:
+            root.remove(child)
+    if body is None:
+        body = last_tag(root)
     body.text = body.tail = None
     del body[:]
     name = container.add_file(spine_name, b'', modify_name_if_needed=True)
@@ -124,6 +132,7 @@ def fix_fullscreen_images(container):
                 svg.set('width', '100vw')
                 svg.set('height', '100vh')
                 container.dirty(file_name)
+
 # }}}
 
 
@@ -137,6 +146,87 @@ class Container(ContainerBase):
         ContainerBase.__init__(self, root_dir or os.path.dirname(opf_path), opf_path, log)
 
 
+class UrlSchemeHandler(QWebEngineUrlSchemeHandler):
+
+    def __init__(self, container, parent=None):
+        QWebEngineUrlSchemeHandler.__init__(self, parent)
+        self.allowed_hosts = (FAKE_HOST,)
+        self.container = container
+
+    def requestStarted(self, rq):
+        if bytes(rq.requestMethod()) != b'GET':
+            return self.fail_request(rq, QWebEngineUrlRequestJob.Error.RequestDenied)
+        url = rq.requestUrl()
+        host = url.host()
+        if host not in self.allowed_hosts or url.scheme() != FAKE_PROTOCOL:
+            return self.fail_request(rq)
+        path = url.path()
+        if path.startswith('/book/'):
+            name = path[len('/book/'):]
+            try:
+                mime_type = self.container.mime_map.get(name) or guess_type(name)
+                try:
+                    with self.container.open(name) as f:
+                        q = os.path.abspath(f.name)
+                        if not q.startswith(self.container.root):
+                            raise FileNotFoundError('Attempt to leave sandbox')
+                        data = f.read()
+                except FileNotFoundError:
+                    print(f'Could not find file {name} in book', file=sys.stderr)
+                    rq.fail(QWebEngineUrlRequestJob.Error.UrlNotFound)
+                    return
+                data = as_bytes(data)
+                mime_type = {
+                    # Prevent warning in console about mimetype of fonts
+                    'application/vnd.ms-opentype':'application/x-font-ttf',
+                    'application/x-font-truetype':'application/x-font-ttf',
+                    'application/font-sfnt': 'application/x-font-ttf',
+                }.get(mime_type, mime_type)
+                send_reply(rq, mime_type, data)
+            except Exception:
+                import traceback
+                traceback.print_exc()
+                return self.fail_request(rq, QWebEngineUrlRequestJob.Error.RequestFailed)
+        elif path.startswith('/mathjax/'):
+            try:
+                ignore, ignore, base, rest = path.split('/', 3)
+            except ValueError:
+                print(f'Could not find file {path} in mathjax', file=sys.stderr)
+                rq.fail(QWebEngineUrlRequestJob.Error.UrlNotFound)
+                return
+            try:
+                mime_type = guess_type(rest)
+                if base == 'loader' and '/' not in rest and '\\' not in rest:
+                    data = P(rest, allow_user_override=False, data=True)
+                elif base == 'data':
+                    q = os.path.abspath(os.path.join(mathjax_dir(), rest))
+                    if not q.startswith(mathjax_dir()):
+                        raise FileNotFoundError('')
+                    with open(q, 'rb') as f:
+                        data = f.read()
+                else:
+                    raise FileNotFoundError('')
+                send_reply(rq, mime_type, data)
+            except FileNotFoundError:
+                print(f'Could not find file {path} in mathjax', file=sys.stderr)
+                rq.fail(QWebEngineUrlRequestJob.Error.UrlNotFound)
+                return
+            except Exception:
+                import traceback
+                traceback.print_exc()
+                return self.fail_request(rq, QWebEngineUrlRequestJob.Error.RequestFailed)
+        else:
+            return self.fail_request(rq)
+
+    def fail_request(self, rq, fail_code=None):
+        if fail_code is None:
+            fail_code = QWebEngineUrlRequestJob.Error.UrlNotFound
+        rq.fail(fail_code)
+        print(f'Blocking FAKE_PROTOCOL request: {rq.requestUrl().toString()} with code: {fail_code}', file=sys.stderr)
+
+# }}}
+
+
 class Renderer(QWebEnginePage):
 
     work_done = pyqtSignal(object, object)
@@ -150,24 +240,24 @@ class Renderer(QWebEnginePage):
         self.settle_time = 0
         self.wait_for_title = None
         s = self.settings()
-        s.setAttribute(s.JavascriptEnabled, True)
-        s.setFontSize(s.DefaultFontSize, int(opts.pdf_default_font_size))
-        s.setFontSize(s.DefaultFixedFontSize, int(opts.pdf_mono_font_size))
-        s.setFontSize(s.MinimumLogicalFontSize, 8)
-        s.setFontSize(s.MinimumFontSize, 8)
+        s.setAttribute(QWebEngineSettings.WebAttribute.JavascriptEnabled, True)
+        s.setFontSize(QWebEngineSettings.FontSize.DefaultFontSize, int(opts.pdf_default_font_size))
+        s.setFontSize(QWebEngineSettings.FontSize.DefaultFixedFontSize, int(opts.pdf_mono_font_size))
+        s.setFontSize(QWebEngineSettings.FontSize.MinimumLogicalFontSize, 8)
+        s.setFontSize(QWebEngineSettings.FontSize.MinimumFontSize, 8)
         std = {
             'serif': opts.pdf_serif_family,
             'sans' : opts.pdf_sans_family,
             'mono' : opts.pdf_mono_family
         }.get(opts.pdf_standard_font, opts.pdf_serif_family)
         if std:
-            s.setFontFamily(s.StandardFont, std)
+            s.setFontFamily(QWebEngineSettings.FontFamily.StandardFont, std)
         if opts.pdf_serif_family:
-            s.setFontFamily(s.SerifFont, opts.pdf_serif_family)
+            s.setFontFamily(QWebEngineSettings.FontFamily.SerifFont, opts.pdf_serif_family)
         if opts.pdf_sans_family:
-            s.setFontFamily(s.SansSerifFont, opts.pdf_sans_family)
+            s.setFontFamily(QWebEngineSettings.FontFamily.SansSerifFont, opts.pdf_sans_family)
         if opts.pdf_mono_family:
-            s.setFontFamily(s.FixedFont, opts.pdf_mono_family)
+            s.setFontFamily(QWebEngineSettings.FontFamily.FixedFont, opts.pdf_mono_family)
 
         self.titleChanged.connect(self.title_changed)
         self.loadStarted.connect(self.load_started)
@@ -236,7 +326,10 @@ class Renderer(QWebEnginePage):
 
         self.settle_time = settle_time
         self.page_layout = page_layout
-        self.setUrl(QUrl.fromLocalFile(path))
+        url = QUrl(f'{FAKE_PROTOCOL}://{FAKE_HOST}/')
+        url.setPath(path)
+        self.setUrl(url)
+        self.job_started_at = monotonic()
 
 
 class RequestInterceptor(QWebEngineUrlRequestInterceptor):
@@ -248,28 +341,23 @@ class RequestInterceptor(QWebEngineUrlRequestInterceptor):
             request_info.block(True)
             return
         qurl = request_info.requestUrl()
-        if qurl.scheme() != 'file':
-            self.log.warn(f'Blocking URL request {qurl.toString()} as it is not for a local file')
-            request_info.block(True)
-            return
-        path = qurl.toLocalFile()
-        path = os.path.normcase(os.path.abspath(path))
-        if not path.startswith(self.container_root) and not path.startswith(self.resources_root):
-            self.log.warn(f'Blocking URL request with path: {path}')
+        if qurl.scheme() not in (FAKE_PROTOCOL,):
+            self.log.warn(f'Blocking URL request {qurl.toString()} as it is not for a resource in the book')
             request_info.block(True)
             return
 
 
 class RenderManager(QObject):
 
-    def __init__(self, opts, log, container_root):
+    def __init__(self, opts, log, container):
         QObject.__init__(self)
         self.interceptor = RequestInterceptor(self)
         self.has_maths = {}
         self.interceptor.log = self.log = log
-        self.interceptor.container_root = os.path.normcase(os.path.abspath(container_root))
-        self.interceptor.resources_root = os.path.normcase(os.path.abspath(os.path.dirname(mathjax_dir())))
         ans = QWebEngineProfile(QApplication.instance())
+        setup_profile(ans)
+        self.url_handler = UrlSchemeHandler(container, parent=ans)
+        ans.installUrlSchemeHandler(QByteArray(FAKE_PROTOCOL.encode('ascii')), self.url_handler)
         ua = 'calibre-pdf-output ' + __version__
         ans.setHttpUserAgent(ua)
         s = ans.settings()
@@ -314,9 +402,12 @@ class RenderManager(QObject):
 
     def convert_html_files(self, jobs, settle_time=0, wait_for_title=None, has_maths=None):
         self.has_maths = has_maths or {}
+        self.render_count = 0
+        self.total_count = len(jobs)
         while len(self.workers) < min(len(jobs), self.max_workers):
             self.create_worker()
         self.pending = list(jobs)
+        self.log(f'Rendering {len(self.pending)} HTML files')
         self.results = {}
         self.settle_time = settle_time
         self.wait_for_title = wait_for_title
@@ -357,6 +448,12 @@ class RenderManager(QObject):
 
     def work_done(self, worker, result):
         self.results[worker.result_key] = result
+        for w in self.workers:
+            if not w.working and w.job_started_at > 0:
+                time_taken = monotonic() - w.job_started_at
+                self.render_count += 1
+                self.log.debug(f'Rendered: {worker.result_key} in {time_taken:.1f} seconds ({self.render_count}/{self.total_count})')
+                w.job_started_at = 0
         if self.pending:
             self.assign_work()
         else:
@@ -378,13 +475,14 @@ def resolve_margins(margins, page_layout):
 
 
 def job_for_name(container, name, margins, page_layout):
-    index_file = container.name_to_abspath(name)
+    index_file = '/book/' + name
     if margins:
         page_layout = QPageLayout(page_layout)
         page_layout.setUnits(QPageLayout.Unit.Point)
         new_margins = QMarginsF(*resolve_margins(margins, page_layout))
         page_layout.setMargins(new_margins)
     return index_file, page_layout, name
+
 # }}}
 
 
@@ -398,15 +496,8 @@ def update_metadata(pdf_doc, pdf_metadata):
 
 
 def add_cover(pdf_doc, cover_data, page_layout, opts):
-    buf = BytesIO()
-    page_size = page_layout.fullRectPoints().size()
-    img = Image(cover_data)
-    writer = PDFStream(buf, (page_size.width(), page_size.height()), compress=True)
-    writer.apply_fill(color=(1, 1, 1))
-    draw_image_page(writer, img, preserve_aspect_ratio=opts.preserve_cover_aspect_ratio)
-    writer.end()
-    cover_pdf_doc = data_as_pdf_doc(buf.getvalue())
-    pdf_doc.insert_existing_page(cover_pdf_doc)
+    r = page_layout.fullRect(QPageLayout.Unit.Point)
+    add_image_page(pdf_doc, cover_data, page_size=(r.left(), r.top(), r.width(), r.height()), preserve_aspect_ratio=opts.preserve_cover_aspect_ratio)
 # }}}
 
 
@@ -427,10 +518,11 @@ def create_margin_files(container):
         if margins:
             margins = dict_to_margins(json.loads(margins))
         yield MarginFile(name, margins)
+
 # }}}
 
 
-# Link handling  {{{
+# Link handling {{{
 def add_anchors_markup(root, uuid, anchors):
     body = last_tag(root)
     div = body.makeelement(
@@ -536,7 +628,7 @@ def make_anchors_unique(container, log):
 
 class AnchorLocation:
 
-    __slots__ = ('pagenum', 'left', 'top', 'zoom')
+    __slots__ = ('left', 'pagenum', 'top', 'zoom')
 
     def __init__(self, pagenum=1, left=0, top=0, zoom=0):
         self.pagenum, self.left, self.top, self.zoom = pagenum, left, top, zoom
@@ -592,6 +684,7 @@ def fix_links(pdf_doc, anchor_locations, name_anchor_map, mark_links, log):
         return loc.as_tuple
 
     pdf_doc.alter_links(replace_link, mark_links)
+
 # }}}
 
 
@@ -657,7 +750,7 @@ def get_page_number_display_map(render_manager, opts, num_pages, log):
             if not isinstance(result, dict):
                 raise ValueError('Not a dict')
         except Exception:
-            log.warn(f'Could not do page number mapping, got unexpected result: {repr(result)}')
+            log.warn(f'Could not do page number mapping, got unexpected result: {result!r}')
         else:
             default_map = {int(k): int(v) for k, v in iteritems(result)}
     return default_map
@@ -684,7 +777,7 @@ def add_pagenum_toc(root, toc, opts, page_number_display_map):
     .calibre-pdf-toc .level-%d td:first-of-type { padding-left: %.1gem }
     .calibre-pdf-toc .level-%d td:first-of-type { padding-left: %.1gem }
     .calibre-pdf-toc .level-%d td:first-of-type { padding-left: %.1gem }
-    ''' % tuple(indents) + (opts.extra_css or '')
+    ''' % tuple(indents) + (opts.extra_css or '')  # noqa: UP031
     style = body.makeelement(XHTML('style'), type='text/css')
     style.text = css
     body.append(style)
@@ -702,7 +795,7 @@ def add_pagenum_toc(root, toc, opts, page_number_display_map):
     E('h2', text=(opts.toc_title or _('Table of Contents')), parent=body)
     table = E('table', parent=body)
     for level, node in toc.iterdescendants(level=0):
-        tr = E('tr', cls='level-%d' % level, parent=table)
+        tr = E('tr', cls=f'level-{level}', parent=table)
         E('td', text=node.title or _('Unknown'), parent=tr)
         num = node.pdf_loc.pagenum
         num = page_number_display_map.get(num, num)
@@ -713,7 +806,6 @@ def add_pagenum_toc(root, toc, opts, page_number_display_map):
 
 # Fonts {{{
 
-
 def all_glyph_ids_in_w_arrays(arrays, as_set=False):
     ans = set()
     for w in arrays:
@@ -755,14 +847,18 @@ def merge_font_files(fonts, log):
     descendant_fonts = [f for f in fonts if f['Subtype'] != 'Type0']
     total_size = sum(len(f['Data']) for f in descendant_fonts)
     merged_sfnt = merge_truetype_fonts_for_pdf(tuple(f['sfnt'] for f in descendant_fonts), log)
-    w_arrays = tuple(filter(None, (f['W'] for f in descendant_fonts)))
-    glyph_ids = all_glyph_ids_in_w_arrays(w_arrays, as_set=True)
-    h_arrays = tuple(filter(None, (f['W2'] for f in descendant_fonts)))
-    glyph_ids |= all_glyph_ids_in_w_arrays(h_arrays, as_set=True)
-    try:
-        pdf_subset(merged_sfnt, glyph_ids)
-    except NoGlyphs:
-        log.warn(f'Subsetting of {fonts[0]["BaseFont"]} failed with no glyphs found, ignoring')
+    if False:
+        # As of Qt 6.7.2 webengine produces W arrays that do not contain all
+        # used glyph ids, so we cannot subset. Can be tested by
+        # echo 'this is a test boulder sentence' > test.txt; ebook-convert test.txt .pdf
+        w_arrays = tuple(filter(None, (f['W'] for f in descendant_fonts)))
+        glyph_ids = all_glyph_ids_in_w_arrays(w_arrays, as_set=True)
+        h_arrays = tuple(filter(None, (f['W2'] for f in descendant_fonts)))
+        glyph_ids |= all_glyph_ids_in_w_arrays(h_arrays, as_set=True)
+        try:
+            pdf_subset(merged_sfnt, glyph_ids)
+        except NoGlyphs:
+            log.warn(f'Subsetting of {fonts[0]["BaseFont"]} failed with no glyphs found, ignoring')
     font_data = merged_sfnt()[0]
     log(f'Merged {len(fonts)} instances of {fonts[0]["BaseFont"]} reducing size from {human_readable(total_size)} to {human_readable(len(font_data))}')
     return font_data, tuple(f['Reference'] for f in descendant_fonts)
@@ -817,7 +913,7 @@ def test_merge_fonts():
 PAGE_NUMBER_TEMPLATE = '<footer><div style="margin: auto">_PAGENUM_</div></footer>'
 
 
-def add_header_footer(manager, opts, pdf_doc, container, page_number_display_map, page_layout, page_margins_map, pdf_metadata, report_progress, toc=None):
+def add_header_footer(manager, opts, pdf_doc, container, page_number_display_map, page_layout, page_margins_map, pdf_metadata, report_progress, toc, log):
     header_template, footer_template = opts.pdf_header_template, opts.pdf_footer_template
     if not footer_template and opts.pdf_page_numbers:
         footer_template = PAGE_NUMBER_TEMPLATE
@@ -826,12 +922,13 @@ def add_header_footer(manager, opts, pdf_doc, container, page_number_display_map
     report_progress(0.8, _('Adding headers and footers'))
     name = create_skeleton(container)
     root = container.parsed(name)
-    reset_css = 'margin: 0; padding: 0; border-width: 0; background-color: unset;'
+    reset_css = 'margin: 0; padding: 0; border-width: 0; background-color: unset; column-count: unset; column-width: unset;'
     root.set('style', reset_css)
     body = last_tag(root)
     body.attrib.pop('id', None)
     body.set('style', reset_css)
     job = job_for_name(container, name, Margins(0, 0, 0, 0), page_layout)
+    page_layout = job[1]
 
     def m(tag_name, text=None, style=None, **attrs):
         ans = root.makeelement(XHTML(tag_name), **attrs)
@@ -916,6 +1013,9 @@ def add_header_footer(manager, opts, pdf_doc, container, page_number_display_map
         toplevel_toc_map = stack_to_map(create_toc_stack(tc()))
         toplevel_pagenum_map, toplevel_pages_map = page_counts_map(tc())
 
+    dpi = 96  # dont know how to query Qt for this, seems to be the same on all platforms
+    def pt_to_px(pt): return int(pt * dpi / 72)
+
     def create_container(page_num, margins):
         style = {
             'page-break-inside': 'avoid',
@@ -933,11 +1033,11 @@ def add_header_footer(manager, opts, pdf_doc, container, page_number_display_map
             'overflow': 'hidden',
             'background-color': 'unset',
         }
-
         ans = m('div', style=style, id=f'p{page_num}')
         return ans
 
-    def format_template(template, page_num, height):
+    def format_template(template, page_num, height, margins):
+        div_width_px = pt_to_px(page_layout.paintRectPoints().width() - margins.left - margins.right)
         template = template.replace('_TOP_LEVEL_SECTION_PAGES_', str(toplevel_pagenum_map[page_num - 1]))
         template = template.replace('_TOP_LEVEL_SECTION_PAGENUM_', str(toplevel_pages_map[page_num - 1]))
         template = template.replace('_TOTAL_PAGES_', str(pages_in_doc))
@@ -946,12 +1046,14 @@ def add_header_footer(manager, opts, pdf_doc, container, page_number_display_map
         template = template.replace('_AUTHOR_', prepare_string_for_xml(pdf_metadata.author, True))
         template = template.replace('_TOP_LEVEL_SECTION_', prepare_string_for_xml(toplevel_toc_map[page_num - 1]))
         template = template.replace('_SECTION_', prepare_string_for_xml(page_toc_map[page_num - 1]))
+        template = template.replace('_WIDTH_PIXELS_', str(div_width_px))
+        template = template.replace('_HEIGHT_PIXELS_', str(pt_to_px(height)))
         troot = parse(template, namespace_elements=True)
         ans = last_tag(troot)[0]
         style = ans.get('style') or ''
         style = (
-            'margin: 0; padding: 0; height: {height}pt; border-width: 0;'
-            'display: flex; align-items: center; overflow: hidden; background-color: unset;').format(height=height) + style
+            f'margin: 0; padding: 0; height: {height}pt; border-width: 0;'
+            'display: flex; align-items: center; overflow: hidden; background-color: unset;') + style
         ans.set('style', style)
         for child in ans.xpath('descendant-or-self::*[@class]'):
             cls = frozenset(child.get('class').split())
@@ -968,12 +1070,12 @@ def add_header_footer(manager, opts, pdf_doc, container, page_number_display_map
         div = create_container(page_num, margins)
         body.append(div)
         if header_template:
-            div.append(format_template(header_template, page_num, margins.top))
+            div.append(format_template(header_template, page_num, margins.top, margins))
         if footer_template:
-            div.append(format_template(footer_template, page_num, margins.bottom))
+            div.append(format_template(footer_template, page_num, margins.bottom, margins))
 
     container.commit()
-    # print(open(job[0]).read())
+    # print(container.raw_data(name))
     results = manager.convert_html_files([job], settle_time=1)
     data = results[name]
     if not isinstance(data, bytes):
@@ -983,8 +1085,7 @@ def add_header_footer(manager, opts, pdf_doc, container, page_number_display_map
     first_page_num = pdf_doc.page_count()
     num_pages = doc.page_count()
     if first_page_num != num_pages:
-        raise ValueError('The number of header/footers pages ({}) != number of document pages ({})'.format(
-            num_pages, first_page_num))
+        raise ValueError(f'The number of header/footers pages ({num_pages}) < number of document pages ({first_page_num})')
     pdf_doc.append(doc)
     pdf_doc.impose(1, first_page_num + 1, num_pages)
     report_progress(0.9, _('Headers and footers added'))
@@ -994,14 +1095,11 @@ def add_header_footer(manager, opts, pdf_doc, container, page_number_display_map
 
 # Maths {{{
 
+@lru_cache(maxsize=2)
 def mathjax_dir():
     return P('mathjax', allow_user_override=False)
 
 
-def path_to_url(path):
-    return QUrl.fromLocalFile(path).toString()
-
-
 def add_maths_script(container):
     has_maths = {}
     for name, is_linear in container.spine_names:
@@ -1009,10 +1107,9 @@ def add_maths_script(container):
         has_maths[name] = hm = check_for_maths(root)
         if not hm:
             continue
-        script = root.makeelement(XHTML('script'), type="text/javascript", src=path_to_url(
-            P('pdf-mathjax-loader.js', allow_user_override=False)))
+        script = root.makeelement(XHTML('script'), type='text/javascript', src=f'{FAKE_PROTOCOL}://{FAKE_HOST}/mathjax/loader/pdf-mathjax-loader.js')
         script.set('async', 'async')
-        script.set('data-mathjax-path', path_to_url(mathjax_dir()))
+        script.set('data-mathjax-path', f'{FAKE_PROTOCOL}://{FAKE_HOST}/mathjax/data/')
         last_tag(root).append(script)
     return has_maths
 # }}}
@@ -1046,7 +1143,7 @@ def convert(opf_path, opts, metadata=None, output_path=None, log=default_log, co
     container.commit()
     report_progress(0.1, _('Completed markup transformation'))
 
-    manager = RenderManager(opts, log, container.root)
+    manager = RenderManager(opts, log, container)
     page_layout = get_page_layout(opts)
     pdf_doc = None
     anchor_locations = {}
@@ -1056,7 +1153,8 @@ def convert(opf_path, opts, metadata=None, output_path=None, log=default_log, co
     results = manager.convert_html_files(jobs, settle_time=1, has_maths=has_maths)
     num_pages = 0
     page_margins_map = []
-    for margin_file in margin_files:
+    all_docs = []
+    for i, margin_file in enumerate(margin_files):
         name = margin_file.name
         data = results[name]
         if not isinstance(data, bytes):
@@ -1066,11 +1164,10 @@ def convert(opf_path, opts, metadata=None, output_path=None, log=default_log, co
         doc_pages = doc.page_count()
         page_margins_map.extend(repeat(resolve_margins(margin_file.margins, page_layout), doc_pages))
         num_pages += doc_pages
+        all_docs.append(doc)
 
-        if pdf_doc is None:
-            pdf_doc = doc
-        else:
-            pdf_doc.append(doc)
+    pdf_doc = all_docs[0]
+    pdf_doc.append(*all_docs[1:])
 
     page_number_display_map = get_page_number_display_map(manager, opts, num_pages, log)
 
@@ -1098,7 +1195,7 @@ def convert(opf_path, opts, metadata=None, output_path=None, log=default_log, co
     add_header_footer(
         manager, opts, pdf_doc, container,
         page_number_display_map, page_layout, page_margins_map,
-        pdf_metadata, report_progress, toc if has_toc else None)
+        pdf_metadata, report_progress, toc if has_toc else None, log)
 
     num_removed = remove_unused_fonts(pdf_doc)
     if num_removed:
@@ -1119,15 +1216,16 @@ def convert(opf_path, opts, metadata=None, output_path=None, log=default_log, co
             mult = -1 if i % 2 else 1
             val = opts.pdf_odd_even_offset
             if abs(val) < min(margins.left, margins.right):
-                box = list(pdf_doc.get_page_box("CropBox", i))
+                box = list(pdf_doc.get_page_box('CropBox', i))
                 box[0] += val * mult
-                pdf_doc.set_page_box("CropBox", i, *box)
+                pdf_doc.set_page_box('CropBox', i, *box)
 
     if cover_data:
         add_cover(pdf_doc, cover_data, page_layout, opts)
 
     if metadata is not None:
         update_metadata(pdf_doc, pdf_metadata)
+    pdf_doc.creator = pdf_doc.producer = __appname__ + ' ' + __version__
     report_progress(1, _('Updated metadata in PDF'))
 
     if opts.uncompressed_pdf:
