@@ -1,18 +1,20 @@
-#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import codecs, zlib, numbers
-from io import BytesIO
+import codecs
+import numbers
+import zlib
 from datetime import datetime
+from io import BytesIO
 
 from calibre.utils.logging import default_log
-from polyglot.builtins import iteritems, codepoint_to_chr
-from polyglot.binary import as_hex_bytes
 from calibre_extensions.speedup import pdf_float
+from polyglot.binary import as_hex_bytes
+from polyglot.builtins import codepoint_to_chr, iteritems
 
 EOL = b'\n'
 
@@ -72,12 +74,12 @@ def serialize(o, stream):
     elif o is None:
         stream.write_raw(b'null')
     elif isinstance(o, datetime):
-        val = o.strftime("D:%Y%m%d%H%M%%02d%z")%min(59, o.second)
+        val = o.strftime('D:%Y%m%d%H%M%%02d%z')%min(59, o.second)
         if datetime.tzinfo is not None:
-            val = "(%s'%s')"%(val[:-2], val[-2:])
+            val = f"({val[:-2]}'{val[-2:]}')"
         stream.write(val.encode('ascii'))
     else:
-        raise ValueError('Unknown object: %r'%o)
+        raise ValueError(f'Unknown object: {o!r}')
 
 
 class Name(str):
@@ -85,7 +87,7 @@ class Name(str):
     def pdf_serialize(self, stream):
         raw = self.encode('ascii')
         if len(raw) > 126:
-            raise ValueError('Name too long: %r'%self)
+            raise ValueError(f'Name too long: {self!r}')
         raw = bytearray(raw)
         sharp = ord(b'#')
         buf = (
@@ -224,11 +226,11 @@ class Reference:
         self.num, self.obj = num, obj
 
     def pdf_serialize(self, stream):
-        raw = '%d 0 R'%self.num
+        raw = f'{self.num} 0 R'
         stream.write(raw.encode('ascii'))
 
     def __repr__(self):
-        return '%d 0 R'%self.num
+        return f'{self.num} 0 R'
 
     def __str__(self):
         return repr(self)
