@@ -1,16 +1,17 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import sys, copy
+import copy
+import sys
 from collections import namedtuple
 
 from qt.core import QLinearGradient, QPointF, sip
 
-from calibre.ebooks.pdf.render.common import Name, Array, Dictionary
+from calibre.ebooks.pdf.render.common import Array, Dictionary, Name
 
 Stop = namedtuple('Stop', 't color')
 
@@ -78,7 +79,7 @@ class LinearGradientPattern(Dictionary):
                         matrix):
         start = gradient.start()
         stop = gradient.finalStop()
-        stops = list(map(lambda x: [x[0], x[1].getRgbF()], gradient.stops()))
+        stops = [[x[0], x[1].getRgbF()] for x in gradient.stops()]
         spread = gradient.spread()
         if spread != gradient.PadSpread:
             inv = matrix.inverted()[0]
