@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 '''
 Convert pml markup to and from html
 '''
@@ -8,16 +6,16 @@ __license__   = 'GPL v3'
 __copyright__ = '2009, John Schember <john@nachtimwald.com>'
 __docformat__ = 'restructuredtext en'
 
+import io
 import os
 import re
-import StringIO
 from copy import deepcopy
 
 from calibre import my_unichr, prepare_string_for_xml
 from calibre.ebooks.metadata.toc import TOC
 
 
-class PML_HTMLizer(object):
+class PML_HTMLizer:
 
     STATES = [
         'i',
@@ -159,10 +157,10 @@ class PML_HTMLizer(object):
     def prepare_pml(self, pml):
         # Give Chapters the form \\*='text'text\\*. This is used for generating
         # the TOC later.
-        pml = re.sub(r'(?msu)(?P<c>\\x)(?P<text>.*?)(?P=c)', lambda match: '%s="%s"%s%s' %
-                     (match.group('c'), self.strip_pml(match.group('text')), match.group('text'), match.group('c')), pml)
-        pml = re.sub(r'(?msu)(?P<c>\\X[0-4])(?P<text>.*?)(?P=c)', lambda match: '%s="%s"%s%s' %
-                     (match.group('c'), self.strip_pml(match.group('text')), match.group('text'), match.group('c')), pml)
+        pml = re.sub(r'(?msu)(?P<c>\\x)(?P<text>.*?)(?P=c)', lambda match: '{}="{}"{}{}'.format(
+            match.group('c'), self.strip_pml(match.group('text')), match.group('text'), match.group('c')), pml)
+        pml = re.sub(r'(?msu)(?P<c>\\X[0-4])(?P<text>.*?)(?P=c)', lambda match: '{}="{}"{}{}'.format(
+            match.group('c'), self.strip_pml(match.group('text')), match.group('text'), match.group('c')), pml)
 
         # Remove comments
         pml = re.sub(r'(?mus)\\v(?P<text>.*?)\\v', '', pml)
@@ -174,18 +172,18 @@ class PML_HTMLizer(object):
         pml = re.sub(r'(?mus)^[ ]*$', '', pml)
 
         # Footnotes and Sidebars.
-        pml = re.sub(r'(?mus)<footnote\s+id="(?P<target>.+?)">\s*(?P<text>.*?)\s*</footnote>', lambda match: '\\FN="%s"%s\\FN' %
-                     (match.group('target'), match.group('text')) if match.group('text') else '', pml)
-        pml = re.sub(r'(?mus)<sidebar\s+id="(?P<target>.+?)">\s*(?P<text>.*?)\s*</sidebar>', lambda match: '\\SB="%s"%s\\SB' %
-                     (match.group('target'), match.group('text')) if match.group('text') else '', pml)
+        pml = re.sub(r'(?mus)<footnote\s+id="(?P<target>.+?)">\s*(?P<text>.*?)\s*</footnote>', lambda match: '\\FN="{}"{}\\FN'.format(
+            match.group('target'), match.group('text')) if match.group('text') else '', pml)
+        pml = re.sub(r'(?mus)<sidebar\s+id="(?P<target>.+?)">\s*(?P<text>.*?)\s*</sidebar>', lambda match: '\\SB="{}"{}\\SB'.format(
+            match.group('target'), match.group('text')) if match.group('text') else '', pml)
 
         # Convert &'s into entities so &amp; in the text doesn't get turned into
         # &. It will display as &amp;
         pml = pml.replace('&', '&amp;')
 
         # Replace \\a and \\U with either the unicode character or the entity.
-        pml = re.sub(r'\\a(?P<num>\d{3})', lambda match: '&#%s;' % match.group('num'), pml)
-        pml = re.sub(r'\\U(?P<num>[0-9a-f]{4})', lambda match: '%s' % my_unichr(int(match.group('num'), 16)), pml)
+        pml = re.sub(r'\\a(?P<num>\d{3})', lambda match: '&#{};'.format(match.group('num')), pml)
+        pml = re.sub(r'\\U(?P<num>[0-9a-f]{4})', lambda match: '{}'.format(my_unichr(int(match.group('num'), 16))), pml)
 
         pml = prepare_string_for_xml(pml)
 
@@ -219,17 +217,17 @@ class PML_HTMLizer(object):
         return html
 
     def cleanup_html_remove_redundant(self, html):
-        for key in self.STATES_TAGS.keys():
+        for key in self.STATES_TAGS:
             open, close = self.STATES_TAGS[key]
             if key in self.STATES_VALUE_REQ:
-                html = re.sub(r'(?u)%s\s*%s' % (open % '.*?', close), '', html)
+                html = re.sub(r'(?u){}\s*{}'.format(open % '.*?', close), '', html)
             else:
-                html = re.sub(r'(?u)%s\s*%s' % (open, close), '', html)
+                html = re.sub(fr'(?u){open}\s*{close}', '', html)
         html = re.sub(r'(?imu)<p>\s*</p>', '', html)
         return html
 
     def start_line(self):
-        start = u''
+        start = ''
 
         state = deepcopy(self.state)
         div = []
@@ -258,10 +256,10 @@ class PML_HTMLizer(object):
             else:
                 start += self.STATES_TAGS[key][0]
 
-        return u'<p>%s' % start
+        return f'<p>{start}'
 
     def end_line(self):
-        end = u''
+        end = ''
 
         div = []
         span = []
@@ -281,10 +279,10 @@ class PML_HTMLizer(object):
             else:
                 end += self.STATES_TAGS[key][1]
 
-        return u'%s</p>' % end
+        return f'{end}</p>'
 
     def process_code(self, code, stream, pre=''):
-        text = u''
+        text = ''
 
         code = self.CODE_STATES.get(code, None)
         if not code:
@@ -309,7 +307,7 @@ class PML_HTMLizer(object):
         return text
 
     def process_code_simple(self, code, stream):
-        text = u''
+        text = ''
 
         if self.state[code][0]:
             if code in self.STATES_CLOSE_VALUE_REQ:
@@ -330,7 +328,7 @@ class PML_HTMLizer(object):
         return text
 
     def process_code_div(self, code, stream):
-        text = u''
+        text = ''
 
         # Close code.
         if self.state[code][0]:
@@ -384,7 +382,7 @@ class PML_HTMLizer(object):
         return text
 
     def process_code_span(self, code, stream):
-        text = u''
+        text = ''
 
         # Close code.
         if self.state[code][0]:
@@ -422,7 +420,7 @@ class PML_HTMLizer(object):
         return text
 
     def process_code_block(self, code, stream, pre=''):
-        text = u''
+        text = ''
 
         # Close all spans
         for c in self.SPAN_STATES:
@@ -445,7 +443,7 @@ class PML_HTMLizer(object):
                 if code in self.LINK_STATES:
                     val = val.lstrip('#')
                 if pre:
-                    val = '%s-%s' % (pre, val)
+                    val = f'{pre}-{val}'
                 if code in self.STATES_VALUE_REQ:
                     text += self.STATES_TAGS[code][0] % val
                 else:
@@ -467,7 +465,7 @@ class PML_HTMLizer(object):
         return text
 
     def code_value(self, stream):
-        value = u''
+        value = ''
         # state 0 is before =
         # state 1 is before the first "
         # state 2 is before the second "
@@ -506,7 +504,7 @@ class PML_HTMLizer(object):
             # Unable to complete the sequence to reterieve the value. Reset
             # the stream to the location it started.
             stream.seek(loc)
-            value = u''
+            value = ''
 
         return value.strip()
 
@@ -558,13 +556,14 @@ class PML_HTMLizer(object):
             else:
                 indent_state['et'] = False
 
-            # Must use StringIO, cStringIO does not support unicode
-            line = StringIO.StringIO(line)
+            if isinstance(line, bytes):
+                line = line.decode('utf-8')
+            line = io.StringIO(line)
             parsed.append(self.start_line())
 
             c = line.read(1)
             while c != '':
-                text = u''
+                text = ''
 
                 if c == '\\':
                     c = line.read(1)
@@ -573,13 +572,13 @@ class PML_HTMLizer(object):
                         text = self.process_code(c, line)
                     elif c in 'FS':
                         l = line.read(1)
-                        if '%s%s' % (c, l) == 'Fn':
+                        if f'{c}{l}' == 'Fn':
                             text = self.process_code('Fn', line, 'fn')
-                        elif '%s%s' % (c, l) == 'FN':
+                        elif f'{c}{l}' == 'FN':
                             text = self.process_code('FN', line)
-                        elif '%s%s' % (c, l) == 'SB':
+                        elif f'{c}{l}' == 'SB':
                             text = self.process_code('SB', line)
-                        elif '%s%s' % (c, l) == 'Sd':
+                        elif f'{c}{l}' == 'Sd':
                             text = self.process_code('Sd', line, 'sb')
                     elif c in 'xXC':
                         empty = False
@@ -590,25 +589,25 @@ class PML_HTMLizer(object):
                         level = 0
                         if c in 'XC':
                             level = line.read(1)
-                        id = 'pml_toc-%s' % len(self.toc)
+                        id = f'pml_toc-{len(self.toc)}'
                         value = self.code_value(line)
                         if c == 'x':
                             t = self.process_code(c, line)
                         elif c == 'X':
-                            t = self.process_code('%s%s' % (c, level), line)
+                            t = self.process_code(f'{c}{level}', line)
                         if not value or value == '':
                             text = t
                         else:
                             self.toc.append((level, (os.path.basename(self.file_name), id, value)))
-                            text = '%s<span id="%s"></span>' % (t, id)
+                            text = f'{t}<span id="{id}"></span>'
                     elif c == 'm':
                         empty = False
                         src = self.code_value(line)
-                        text = '<img src="images/%s" />' % src
+                        text = f'<img src="images/{src}" />'
                     elif c == 'Q':
                         empty = False
                         id = self.code_value(line)
-                        text = '<span id="%s"></span>' % id
+                        text = f'<span id="{id}"></span>'
                     elif c == 'p':
                         empty = False
                         text = '<br /><br style="page-break-after: always;" />'
@@ -616,7 +615,7 @@ class PML_HTMLizer(object):
                         pass
                     elif c == 'w':
                         empty = False
-                        text = '<hr width="%s" />' % self.code_value(line)
+                        text = f'<hr style="width: {self.code_value(line)}" />'
                     elif c == 't':
                         indent_state['t'] = not indent_state['t']
                     elif c == 'T':
@@ -672,10 +671,10 @@ class PML_HTMLizer(object):
                     indent_state['T'] = False
                     adv_indent_val = ''
 
-                output.append(u''.join(parsed))
+                output.append(''.join(parsed))
             line.close()
 
-        output = self.cleanup_html(u'\n'.join(output))
+        output = self.cleanup_html('\n'.join(output))
 
         return output
 
@@ -699,18 +698,18 @@ class PML_HTMLizer(object):
         t_l3 = None
 
         for level, (href, id, text) in self.toc:
-            if level == u'0':
+            if level == '0':
                 t_l0 = n_toc.add_item(href, id, text)
                 t_l1 = None
                 t_l2 = None
                 t_l3 = None
-            elif level == u'1':
+            elif level == '1':
                 if t_l0 is None:
                     t_l0 = n_toc
                 t_l1 = t_l0.add_item(href, id, text)
                 t_l2 = None
                 t_l3 = None
-            elif level == u'2':
+            elif level == '2':
                 if t_l1 is None:
                     if t_l0 is None:
                         t_l1 = n_toc
@@ -718,7 +717,7 @@ class PML_HTMLizer(object):
                         t_l1 = t_l0
                 t_l2 = t_l1.add_item(href, id, text)
                 t_l3 = None
-            elif level == u'3':
+            elif level == '3':
                 if t_l2 is None:
                     if t_l1 is None:
                         if t_l0 is None:
@@ -756,10 +755,11 @@ def pml_to_html(pml):
 def footnote_sidebar_to_html(pre_id, id, pml):
     id = id.strip('\x01')
     if id.strip():
-        html = '<br /><br style="page-break-after: always;" /><div id="%s-%s">%s<small><a href="#r%s-%s">return</a></small></div>' % (
-            pre_id, id, pml_to_html(pml), pre_id, id)
+        html = (
+            f'<br /><br style="page-break-after: always;" /><div id="{pre_id}-{id}">{pml_to_html(pml)}'
+            f'<small><a href="#r{pre_id}-{id}">return</a></small></div>')
     else:
-        html = '<br /><br style="page-break-after: always;" /><div>%s</div>' % pml_to_html(pml)
+        html = f'<br /><br style="page-break-after: always;" /><div>{pml_to_html(pml)}</div>'
     return html
 
 
