@@ -10,15 +10,18 @@
 #                                                                       #
 #                                                                       #
 #########################################################################
-import os, re
+import os
+import re
+
 from calibre.ebooks.rtf2xml import copy
 from calibre.ptempfile import better_mktemp
+
 from . import open_for_read, open_for_write
 
 
 class HeadingsToSections:
-    """
-    """
+    '''
+    '''
 
     def __init__(self,
             in_file,
@@ -26,7 +29,7 @@ class HeadingsToSections:
             copy=None,
             run_level=1,
             ):
-        """
+        '''
         Required:
             'file'
         Optional:
@@ -35,14 +38,14 @@ class HeadingsToSections:
             directory from which the script is run.)
         Returns:
             nothing
-            """
+        '''
         self.__file = in_file
         self.__bug_handler = bug_handler
         self.__copy = copy
         self.__write_to = better_mktemp()
 
     def __initiate_values(self):
-        """
+        '''
         Required:
             Nothing
         Return:
@@ -50,15 +53,15 @@ class HeadingsToSections:
         Logic:
             The self.__end_list is a list of tokens that will force a list to end.
             Likewise, the self.__end_lines is a list of lines that forces a list to end.
-        """
-        self.__state = "default"
+        '''
+        self.__state = 'default'
         self.__all_sections = []
         self.__chunk = ''
         self.__state_dict={
-        'default'           :   self.__default_func,
-        'in_table'          :   self.__in_table_func,
-        'in_list'           :   self.__in_list_func,
-        'after_body'        :   self.__after_body_func,
+        'default'   : self.__default_func,
+        'in_table'  : self.__in_table_func,
+        'in_list'   : self.__in_list_func,
+        'after_body': self.__after_body_func,
         }
         self.__list_depth = 0
         self.__end_list = [
@@ -85,7 +88,7 @@ class HeadingsToSections:
         self.__id_regex = re.compile(r'\<list-id\>(\d+)')
 
     def __close_lists(self):
-        """
+        '''
         Required:
             Nothing
         Return:
@@ -97,7 +100,7 @@ class HeadingsToSections:
             Keep track of how many levels you close. Reduce the list by that
             many levels.
             Reverse the list again.
-        """
+        '''
         current_indent = self.__left_indent
         self.__all_lists.reverse()
         num_levels_closed = 0
@@ -123,7 +126,7 @@ class HeadingsToSections:
     def __write_start_section(self, current_level, name):
         section_num = ''
         for the_num in self.__section_num:
-            section_num += '%s.' % the_num
+            section_num += f'{the_num}.'
         section_num = section_num[:-1]
         num_in_level = len(self.__all_sections)
         num_in_level = self.__section_num[num_in_level]
@@ -132,9 +135,8 @@ class HeadingsToSections:
             'mi<mk<sect-start\n'
                 )
         self.__write_obj.write(
-                'mi<tg<open-att__<section<num>%s<num-in-level>%s<level>%s'
-                '<type>%s\n'
-                % (section_num, num_in_level, level, name)
+                f'mi<tg<open-att__<section<num>{section_num}<num-in-level>{num_in_level}<level>{level}'
+                f'<type>{name}\n'
                 )
 
     def __write_end_section(self):
@@ -142,7 +144,7 @@ class HeadingsToSections:
         self.__write_obj.write('mi<tg<close_____<section\n')
 
     def __default_func(self, line):
-        """
+        '''
         Required:
             self, line
         Returns:
@@ -151,7 +153,7 @@ class HeadingsToSections:
             Look for the start of a paragraph definition. If one is found, check if
             it contains a list-id. If it does, start a list. Change the state to
             in_pard.
-            """
+        '''
         if self.__token_info == 'mi<mk<sect-start':
             self.__section_num[0] += 1
             self.__section_num = self.__section_num[0:1]
@@ -200,13 +202,13 @@ class HeadingsToSections:
         self.__write_obj.write(line)
 
     def make_sections(self):
-        """
+        '''
         Required:
             nothing
         Returns:
             original file will be changed
         Logic:
-        """
+        '''
         self.__initiate_values()
         read_obj = open_for_read(self.__file)
         self.__write_obj = open_for_write(self.__write_to)
@@ -221,6 +223,6 @@ class HeadingsToSections:
         self.__write_obj.close()
         copy_obj = copy.Copy(bug_handler=self.__bug_handler)
         if self.__copy:
-            copy_obj.copy_file(self.__write_to, "sections_to_headings.data")
+            copy_obj.copy_file(self.__write_to, 'sections_to_headings.data')
         copy_obj.rename(self.__write_to, self.__file)
         os.remove(self.__write_to)
