@@ -1,18 +1,20 @@
-#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.13
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import sys, os, unicodedata
+import os
+import sys
+import unicodedata
 
-from calibre import prints, as_unicode, walk
-from calibre.constants import iswindows, __appname__
-from calibre.ptempfile import TemporaryDirectory
+from calibre import as_unicode, prints, walk
+from calibre.constants import __appname__, iswindows
 from calibre.libunzip import extract as zipextract
-from calibre.utils.zipfile import ZipFile, ZIP_DEFLATED, ZIP_STORED
+from calibre.ptempfile import TemporaryDirectory
 from calibre.utils.ipc.simple_worker import WorkerError
+from calibre.utils.zipfile import ZIP_DEFLATED, ZIP_STORED, ZipFile
 
 
 class Error(ValueError):
@@ -27,7 +29,8 @@ def ask_cli_question(msg):
         import msvcrt
         ans = msvcrt.getch()
     else:
-        import tty, termios
+        import termios
+        import tty
         old_settings = termios.tcgetattr(sys.stdin.fileno())
         try:
             tty.setraw(sys.stdin.fileno())
@@ -42,7 +45,7 @@ def ask_cli_question(msg):
 
 
 def mobi_exploder(path, tdir, question=lambda x:True):
-    from calibre.ebooks.mobi.tweak import explode, BadFormat
+    from calibre.ebooks.mobi.tweak import BadFormat, explode
     try:
         return explode(path, tdir, question=question)
     except BadFormat as e:
@@ -104,12 +107,12 @@ def explode(ebook_file, output_dir):
     if not os.path.exists(output_dir):
         os.makedirs(output_dir)
     if not os.path.isdir(output_dir):
-        raise SystemExit('%s is not a directory' % output_dir)
+        raise SystemExit(f'{output_dir} is not a directory')
     output_dir = os.path.abspath(output_dir)
     fmt = ebook_file.rpartition('.')[-1].lower()
     exploder, rebuilder = get_tools(fmt)
     if exploder is None:
-        raise SystemExit('Cannot tweak %s files. Supported formats are: EPUB, HTMLZ, AZW3, MOBI, DOCX' % fmt.upper())
+        raise SystemExit(f'Cannot tweak {fmt.upper()} files. Supported formats are: EPUB, HTMLZ, AZW3, MOBI, DOCX')
     try:
         opf = exploder(ebook_file, output_dir, question=ask_cli_question)
     except WorkerError as e:
@@ -124,7 +127,7 @@ def explode(ebook_file, output_dir):
         # The question was answered with No
         return
     h = '_' if iswindows else '.'
-    with lopen(os.path.join(output_dir, h + '__explode_fmt__'), 'wb') as f:
+    with open(os.path.join(output_dir, h + '__explode_fmt__'), 'wb') as f:
         f.write(fmt.encode('utf-8'))
     prints('Book extracted to', output_dir)
     prints('Make your changes and once you are done, use --implode-book to rebuild')
@@ -135,14 +138,14 @@ def implode(output_dir, ebook_file):
     fmt = ebook_file.rpartition('.')[-1].lower()
     exploder, rebuilder = get_tools(fmt)
     if rebuilder is None:
-        raise SystemExit('Cannot tweak %s files. Supported formats are: EPUB, HTMLZ, AZW3, MOBI, DOCX' % fmt.upper())
+        raise SystemExit(f'Cannot tweak {fmt.upper()} files. Supported formats are: EPUB, HTMLZ, AZW3, MOBI, DOCX')
     h = '_' if iswindows else '.'
     efmt_path = os.path.join(output_dir, h + '__explode_fmt__')
     try:
-        with lopen(efmt_path, 'rb') as f:
+        with open(efmt_path, 'rb') as f:
             efmt = f.read().decode('utf-8')
     except Exception:
-        raise SystemExit('The folder %s does not seem to have been created by --explode-book' % output_dir)
+        raise SystemExit(f'The folder {output_dir} does not seem to have been created by --explode-book')
     if efmt != fmt:
         raise SystemExit('You must use the same format of file as was used when exploding the book')
     os.remove(efmt_path)
@@ -160,7 +163,7 @@ def tweak(ebook_file):
     fmt = ebook_file.rpartition('.')[-1].lower()
     exploder, rebuilder = get_tools(fmt)
     if exploder is None:
-        prints('Cannot tweak %s files. Supported formats are: EPUB, HTMLZ, AZW3, MOBI' % fmt.upper()
+        prints(f'Cannot tweak {fmt.upper()} files. Supported formats are: EPUB, HTMLZ, AZW3, MOBI'
                 , file=sys.stderr)
         raise SystemExit(1)
 
