@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
@@ -8,8 +8,7 @@ __docformat__ = 'restructuredtext en'
 from functools import partial
 from zipfile import ZipFile
 
-from qt.core import (QToolButton, QAction, QIcon, QObject, QMenu,
-        QKeySequence)
+from qt.core import QAction, QIcon, QKeySequence, QMenu, QObject, QPoint, QTimer, QToolButton
 
 from calibre import prints
 from calibre.constants import ismacos
@@ -18,12 +17,74 @@ from calibre.gui2.keyboard import NameConflict
 from polyglot.builtins import string_or_bytes
 
 
+def toolbar_widgets_for_action(gui, action):
+    # Search the toolbars for the widget associated with an action, passing
+    # them to the caller for further processing
+    for x in gui.bars_manager.bars:
+        try:
+            w = x.widgetForAction(action)
+            # It seems that multiple copies of the action can exist, such as
+            # when the device-connected menu is changed while the device is
+            # connected. Use the one that has an actual position.
+            if w is None or w.pos().x() == 0:
+                continue
+            # The button might be hidden
+            if not w.isVisible():
+                continue
+            yield w
+        except Exception:
+            continue
+
+
+def show_menu_under_widget(gui, menu, action, name):
+    # First try the tool bar
+    for w in toolbar_widgets_for_action(gui, action):
+        try:
+            # The w.height() assures that the menu opens below the button.
+            menu.exec(w.mapToGlobal(QPoint(0, w.height())))
+            return
+        except Exception:
+            continue
+    # Now try the menu bar
+    for x in gui.bars_manager.menu_bar.added_actions:
+        # This depends on no two menus with the same name.
+        # I don't know if this works on a Mac
+        if x.text() == name:
+            try:
+                # The menu item might be hidden
+                if not x.isVisible():
+                    continue
+                # We can't use x.trigger() because it doesn't put the menu
+                # in the right place. Instead get the position of the menu
+                # widget on the menu bar
+                p = x.parent().menu_bar
+                r = p.actionGeometry(x)
+                # Make sure that the menu item is actually displayed in the menu
+                # and not the overflow
+                if p.geometry().width() < (r.x() + r.width()):
+                    continue
+                # Show the menu under the name in the menu bar
+                menu.exec(p.mapToGlobal(QPoint(r.x()+2, r.height()-2)))
+                return
+            except Exception:
+                continue
+    # Is it one of the status bar buttons?
+    for button in gui.status_bar_extra_buttons:
+        if name == button.action_name and button.isVisible():
+            r = button.geometry()
+            p = gui.status_bar
+            menu.exec(p.mapToGlobal(QPoint(r.x()+2, r.height()-2)))
+            return
+    # No visible button found. Fall back to displaying in upper left corner
+    # of the library view.
+    menu.exec(gui.library_view.mapToGlobal(QPoint(10, 10)))
+
+
 def menu_action_unique_name(plugin, unique_name):
-    return '%s : menu action : %s'%(plugin.unique_name, unique_name)
+    return f'{plugin.unique_name} : menu action : {unique_name}'
 
 
 class InterfaceAction(QObject):
-
     '''
     A plugin representing an "action" that can be taken in the graphical user
     interface. All the items in the toolbar and context menus are implemented
@@ -71,14 +132,18 @@ class InterfaceAction(QObject):
     #: key is held down.
     auto_repeat = False
 
-    #: Of the form: (text, icon_path, tooltip, keyboard shortcut)
-    #: icon, tooltip and keyboard shortcut can be None
-    #: shortcut must be a string, None or tuple of shortcuts.
+    #: Of the form: (text, icon_path, tooltip, keyboard shortcut).
+    #: icon, tooltip and keyboard shortcut can be None.
+    #: keyboard shortcut must be either a string, None or tuple of shortcuts.
     #: If None, a keyboard shortcut corresponding to the action is not
     #: registered. If you pass an empty tuple, then the shortcut is registered
     #: with no default key binding.
     action_spec = ('text', 'icon', None, None)
 
+    #: If not None, used for the name displayed to the user when customizing
+    #: the keyboard shortcuts for the above action spec instead of action_spec[0]
+    action_shortcut_name = None
+
     #: If True, a menu is automatically created and added to self.qaction
     action_add_menu = False
 
@@ -97,7 +162,7 @@ class InterfaceAction(QObject):
 
     all_locations = frozenset(['toolbar', 'toolbar-device', 'context-menu',
         'context-menu-device', 'toolbar-child', 'menubar', 'menubar-device',
-        'context-menu-cover-browser', 'context-menu-split'])
+        'context-menu-cover-browser', 'context-menu-split', 'searchbar'])
 
     #: Type of action
     #: 'current' means acts on the current view
@@ -151,14 +216,14 @@ class InterfaceAction(QObject):
         bn = self.__class__.__name__
         if getattr(self.interface_action_base_plugin, 'name'):
             bn = self.interface_action_base_plugin.name
-        return 'Interface Action: %s (%s)'%(bn, self.name)
+        return f'Interface Action: {bn} ({self.name})'
 
     def create_action(self, spec=None, attr='qaction', shortcut_name=None, persist_shortcut=False):
         if spec is None:
             spec = self.action_spec
         text, icon, tooltip, shortcut = spec
         if icon is not None:
-            action = QAction(QIcon(I(icon)), text, self.gui)
+            action = QAction(QIcon.ic(icon), text, self.gui)
         else:
             action = QAction(text, self.gui)
         if attr == 'qaction':
@@ -181,9 +246,11 @@ class InterfaceAction(QObject):
         if shortcut is not None:
             keys = ((shortcut,) if isinstance(shortcut, string_or_bytes) else
                     tuple(shortcut))
-            if shortcut_name is None and spec[0]:
-                shortcut_name = str(spec[0])
-
+            if shortcut_name is None:
+                if self.action_shortcut_name is not None:
+                    shortcut_name = self.action_shortcut_name
+                elif spec[0]:
+                    shortcut_name = str(spec[0])
             if shortcut_name and self.action_spec[0] and not (
                     attr == 'qaction' and self.popup_type == QToolButton.ToolButtonPopupMode.InstantPopup):
                 try:
@@ -229,7 +296,7 @@ class InterfaceAction(QObject):
             an UUID to it.
         :param text: The text of the action.
         :param icon: Either a QIcon or a file name. The file name is passed to
-            the I() builtin, so you do not need to pass the full path to the images
+            the QIcon.ic() builtin, so you do not need to pass the full path to the images
             folder.
         :param shortcut: A string, a list of strings, None or False. If False,
             no keyboard shortcut is registered for this action. If None, a keyboard
@@ -253,7 +320,7 @@ class InterfaceAction(QObject):
         ac = menu.addAction(text)
         if icon is not None:
             if not isinstance(icon, QIcon):
-                icon = QIcon(I(icon))
+                icon = QIcon.ic(icon)
             ac.setIcon(icon)
         keys = ()
         if shortcut is not None and shortcut is not False:
@@ -376,3 +443,36 @@ class InterfaceAction(QObject):
         long periods of time.
         '''
         pass
+
+
+class InterfaceActionWithLibraryDrop(InterfaceAction):
+    '''
+    Subclass of InterfaceAction that implements methods to execute the default action
+    by drop some books from the library.
+
+    Inside the do_drop() method, the ids of the dropped books are provided
+    by the attribute self.dropped_ids
+    '''
+
+    accepts_drops = True
+    mimetype_for_drop = 'application/calibre+from_library'
+
+    def accept_enter_event(self, event, mime_data):
+        if mime_data.hasFormat(self.mimetype_for_drop):
+            return True
+        return False
+
+    def accept_drag_move_event(self, event, mime_data):
+        if mime_data.hasFormat(self.mimetype_for_drop):
+            return True
+        return False
+
+    def drop_event(self, event, mime_data):
+        if mime_data.hasFormat(self.mimetype_for_drop):
+            self.dropped_ids = tuple(map(int, mime_data.data(self.mimetype_for_drop).data().split()))
+            QTimer.singleShot(1, self.do_drop)
+            return True
+        return False
+
+    def do_drop(self):
+        raise NotImplementedError()
