@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
@@ -6,11 +6,11 @@ __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 
-from qt.core import pyqtSignal, QModelIndex, QThread, Qt
+from qt.core import QModelIndex, Qt, QThread, pyqtSignal
 
+from calibre.devices.usbms.device import Device
 from calibre.gui2 import error_dialog
 from calibre.gui2.actions import InterfaceAction
-from calibre.devices.usbms.device import Device
 from calibre.gui2.dialogs.progress import ProgressDialog
 from polyglot.builtins import iteritems
 
@@ -102,7 +102,7 @@ class FetchAnnotationsAction(InterfaceAction):
                 path = get_device_path_from_id(id)
                 mi = db.get_metadata(id, index_is_id=True)
                 a_path = device.create_annotations_path(mi, device_path=path)
-                path_map[id] = dict(path=a_path, fmts=get_formats(id))
+                path_map[id] = {'path': a_path, 'fmts': get_formats(id)}
             return path_map
 
         device = self.gui.device_manager.device
