@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.12
 
 
 __license__   = 'GPL v3'
@@ -8,16 +8,17 @@ __docformat__ = 'restructuredtext en'
 import os
 from functools import partial
 
-from qt.core import QModelIndex, QTimer
+from qt.core import QModelIndex
 
-from calibre.gui2 import error_dialog, Dispatcher, gprefs
-from calibre.gui2.tools import convert_single_ebook, convert_bulk_ebook
+from calibre.customize.ui import plugin_for_input_format, run_plugins_on_postconvert
+from calibre.gui2 import Dispatcher, error_dialog, gprefs
+from calibre.gui2.actions import InterfaceActionWithLibraryDrop
+from calibre.gui2.tools import convert_bulk_ebook, convert_single_ebook
 from calibre.utils.config import prefs, tweaks
-from calibre.gui2.actions import InterfaceAction
-from calibre.customize.ui import plugin_for_input_format
+from calibre.utils.localization import ngettext
 
 
-class ConvertAction(InterfaceAction):
+class ConvertAction(InterfaceActionWithLibraryDrop):
 
     name = 'Convert Books'
     action_spec = (_('Convert books'), 'convert.png', _('Convert books between different e-book formats'), _('C'))
@@ -25,26 +26,6 @@ class ConvertAction(InterfaceAction):
     action_type = 'current'
     action_add_menu = True
 
-    accepts_drops = True
-
-    def accept_enter_event(self, event, mime_data):
-        if mime_data.hasFormat("application/calibre+from_library"):
-            return True
-        return False
-
-    def accept_drag_move_event(self, event, mime_data):
-        if mime_data.hasFormat("application/calibre+from_library"):
-            return True
-        return False
-
-    def drop_event(self, event, mime_data):
-        mime = 'application/calibre+from_library'
-        if mime_data.hasFormat(mime):
-            self.dropped_ids = tuple(map(int, mime_data.data(mime).data().split()))
-            QTimer.singleShot(1, self.do_drop)
-            return True
-        return False
-
     def do_drop(self):
         book_ids = self.dropped_ids
         del self.dropped_ids
@@ -196,7 +177,7 @@ class ConvertAction(InterfaceAction):
             converted_func, extra_job_args=[], rows_are_ids=False):
         for func, args, desc, fmt, id, temp_files in jobs:
             func, _, parts = func.partition(':')
-            parts = {x for x in parts.split(';')}
+            parts = set(parts.split(';'))
             input_file = args[0]
             input_fmt = os.path.splitext(input_file)[1]
             core_usage = 1
@@ -274,6 +255,7 @@ class ConvertAction(InterfaceAction):
 
             with open(temp_files[-1].name, 'rb') as data:
                 db.add_format(book_id, fmt, data, index_is_id=True)
+            run_plugins_on_postconvert(db, book_id, fmt)
             self.gui.book_converted.emit(book_id, fmt)
             self.gui.status_bar.show_message(job.description + ' ' +
                     _('completed'), 2000)
