@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.9
+#!/usr/local/bin/python3.12
 
 
 __license__ = 'GPL v3'
@@ -6,12 +6,12 @@ __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
 from functools import partial
 
-from qt.core import (QTimer, QApplication, Qt, QEvent, QDialog, QMenu, QIcon,
-                     QDialogButtonBox, QPushButton, QLabel, QGridLayout)
+from qt.core import QApplication, QDialog, QDialogButtonBox, QEvent, QGridLayout, QIcon, QLabel, QMenu, QPushButton, Qt
 
 from calibre.gui2 import error_dialog
-from calibre.gui2.actions import InterfaceAction
+from calibre.gui2.actions import InterfaceActionWithLibraryDrop
 from calibre.gui2.widgets2 import HistoryComboBox
+from calibre.startup import connect_lambda
 from calibre.utils.icu import sort_key
 
 
@@ -37,7 +37,7 @@ class MarkWithTextDialog(QDialog):
 
         history = textbox.all_items
         button_rows = min(4, len(history)-1)
-        for i in range(0, button_rows):
+        for i in range(button_rows):
             if i == 0:
                 layout.addWidget(QLabel(_('Recently used values:')), 0, 0, 1, 2)
             button = QPushButton()
@@ -79,7 +79,7 @@ class MarkWithTextDialog(QDialog):
 mark_books_with_text = None
 
 
-class MarkBooksAction(InterfaceAction):
+class MarkBooksAction(InterfaceActionWithLibraryDrop):
 
     name = 'Mark Books'
     action_spec = (_('Mark books'), 'marked.png', _('Temporarily mark books for easy access'), 'Ctrl+M')
@@ -89,26 +89,6 @@ class MarkBooksAction(InterfaceAction):
         'context-menu-device', 'menubar-device', 'context-menu-cover-browser'])
     action_menu_clone_qaction = _('Toggle mark for selected books')
 
-    accepts_drops = True
-
-    def accept_enter_event(self, event, mime_data):
-        if mime_data.hasFormat("application/calibre+from_library"):
-            return True
-        return False
-
-    def accept_drag_move_event(self, event, mime_data):
-        if mime_data.hasFormat("application/calibre+from_library"):
-            return True
-        return False
-
-    def drop_event(self, event, mime_data):
-        mime = 'application/calibre+from_library'
-        if mime_data.hasFormat(mime):
-            self.dropped_ids = tuple(map(int, mime_data.data(mime).data().split()))
-            QTimer.singleShot(1, self.do_drop)
-            return True
-        return False
-
     def do_drop(self):
         book_ids = self.dropped_ids
         del self.dropped_ids
@@ -121,6 +101,8 @@ class MarkBooksAction(InterfaceAction):
         self.menu = m = self.qaction.menu()
         m.aboutToShow.connect(self.about_to_show_menu)
         ma = partial(self.create_menu_action, m)
+        self.show_marked_action = a = ma('mark_selected', _('Mark all selected books'), icon='marked.png')
+        a.triggered.connect(self.mark_all_selected)
         self.show_marked_action = a = ma('mark_with_text', _('Mark books with text label'), icon='marked.png')
         a.triggered.connect(partial(self.mark_with_text, book_ids=None))
         global mark_books_with_text
@@ -146,7 +128,7 @@ class MarkBooksAction(InterfaceAction):
         connect_lambda(a.triggered, self, lambda self: self.mark_field('series', False))
 
     def gui_layout_complete(self):
-        for x in self.gui.bars_manager.main_bars + self.gui.bars_manager.child_bars:
+        for x in self.gui.bars_manager.bars:
             try:
                 w = x.widgetForAction(self.qaction)
                 w.installEventFilter(self)
@@ -165,9 +147,9 @@ class MarkBooksAction(InterfaceAction):
         db = self.gui.current_db
         marked_ids = db.data.marked_ids
         num = len(frozenset(marked_ids).intersection(db.new_api.all_book_ids()))
-        text = _('Show marked book') if num == 1 else (_('Show marked books') + (' (%d)' % num))
+        text = _('Show marked book') if num == 1 else (_('Show marked books') + f' ({num})')
         self.show_marked_action.setText(text)
-        counts = dict()
+        counts = {}
         for v in marked_ids.values():
             counts[v] = counts.get(v, 0) + 1
         labels = sorted(counts.keys(), key=sort_key)
@@ -207,6 +189,9 @@ class MarkBooksAction(InterfaceAction):
     def toggle_ids(self, book_ids):
         self.gui.current_db.data.toggle_marked_ids(book_ids)
 
+    def add_ids(self, book_ids):
+        self.gui.current_db.data.add_marked_ids(book_ids)
+
     def show_marked(self):
         self.gui.search.set_search_string('marked:true')
 
@@ -237,6 +222,12 @@ class MarkBooksAction(InterfaceAction):
                 mids.pop(book_id, None)
         db.data.set_marked_ids(mids)
 
+    def mark_all_selected(self):
+        book_ids = self._get_selected_ids()
+        if not book_ids:
+            return
+        self.gui.current_db.data.add_marked_ids(book_ids)
+
     def mark_with_text(self, book_ids=None):
         if book_ids is None:
             book_ids = self._get_selected_ids()
